      SUBROUTINE BCINIT
 
C     *************************************************************************
C
C     BCINIT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES THE BOUNDARY CONDITIONS
C
C     *************************************************************************


C     FUNCTION
C     ========
      DOUBLE PRECISION RANGAB
      EXTERNAL RANGAB


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION RMGTOL
      PARAMETER(RMGTOL = 1.0D-6)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION RNSTAT(NRNGMX)
      DOUBLE PRECISION UINPRE(NCMPMX),UINMRE(NCMPMX)
      DOUBLE PRECISION UINPIM(NCMPMX),UINMIM(NCMPMX)
      DOUBLE PRECISION TMCORR
      DOUBLE PRECISION RMODE1,RMODE2,RMAGLO,RMAGHI,RMAGSQ,RMAGNI
      DOUBLE PRECISION HCBASE,ARGMNT,OTPHYR
      INTEGER JCELL,JFACE,JSPEC,IGLOBC
      INTEGER KMODE1,KMODE2,KMAGLO,KMAGHI,IKLIST
      INTEGER NXCELL,NYCELL,NZCELL,NTCELL,ITCELL
      INTEGER NCOUNT,IROOT


C     BEGIN
C     =====

C     =========================================================================

C     TURBULENT INLET VELOCITY FIELD
      FLNTRB = .FALSE.

C     =========================================================================

C     RUN THROUGH THE FACES OF THE GLOBAL DOMAIN
C     ------------------------------------------
      DO JFACE = 1, NFACE

C       =======================================================================

C       CHECK GLOBAL BC CODE
        IGLOBC = NGLOBC(JFACE)
        IF(IGLOBC.EQ.NSBCO1)THEN

C         ---------------------------------------------------------------------

C         SUBSONIC NON-REFLECTING OUTFLOW WITH PRESSURE-AT-INFINITY
C         ---------------------------------------------------------
C         SET CONSTANT PRESSURE-AT-INFINITY FOR EACH FACE
          PBCINF(JFACE) = RGBPRM(1,JFACE)
          PBCNST(JFACE) = RGBPRM(2,JFACE)*(ONE-RGBPRM(3,JFACE))
     +                    /(TWO*XGDLEN)

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCO2)THEN

C         ---------------------------------------------------------------------

C         SUBSONIC REFLECTING OUTFLOW WITH SPECIFIED PRESSURE
C         ---------------------------------------------------
C         SET CONSTANT PRESSURE FOR EACH FACE
          PBCINF(JFACE) = RGBPRM(1,JFACE)
          PBCNST(JFACE) = ZERO

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCO3)THEN

C         ---------------------------------------------------------------------

C         SUBSONIC NON-REFLECTING OUTFLOW WITH VELOCITY-AT-INFINITY
C         ---------------------------------------------------------
C         SET CONSTANT PRESSURE-AT-INFINITY FOR EACH FACE
          PBCINF(JFACE) = RGBPRM(1,JFACE)
          PBCNST(JFACE) = RGBPRM(2,JFACE)*(ONE-RGBPRM(3,JFACE))
     +                    /(TWO*XGDLEN)

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCO4)THEN

C         ---------------------------------------------------------------------

C         SUBSONIC NON-REFLECTING OUTFLOW WITH PRESSURE-AT-INFINITY
C         ---------------------------------------------------------
C         SET CONSTANT PRESSURE-AT-INFINITY FOR EACH FACE
          PBCINF(JFACE) = RGBPRM(1,JFACE)
          PBCNST(JFACE) = RGBPRM(2,JFACE)*(ONE-RGBPRM(3,JFACE))
     +                    /(TWO*XGDLEN)

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCI1)THEN

C         ---------------------------------------------------------------------

C         SUBSONIC NON-REFLECTING INFLOW
C         ------------------------------
C         SET CONSTANT PRESSURE-AT-INFINITY FOR EACH FACE
          PBCINF(JFACE) = RGBPRM(1,JFACE)
          PBCNST(JFACE) = RGBPRM(2,JFACE)*(ONE-RGBPRM(3,JFACE))
     +                    /(TWO*XGDLEN)

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCI2)THEN

C         ---------------------------------------------------------------------

C         SUBSONIC REFLECTING INFLOW WITH SPECIFIED VELOCITY AND TEMPERATURE
C         ------------------------------------------------------------------

C         CHECK INLET BC CODE
          IF(NGBPRM(1,JFACE).EQ.1)THEN

C           CONSTANT INLET NORMAL VELOCITY
C           PARAMETERS: I1=1; R1=VELOCITY
            UBCSET = RGBPRM(1,JFACE)

          ELSEIF(NGBPRM(1,JFACE).EQ.2)THEN

C           SINUSOIDAL INLET NORMAL VELOCITY
C           PARAMETERS: I1=2;
C           R1=MEAN VELOCITY, R2=AMPLITUDE, R3=ANGULAR FREQ, R4=PHASE OFFSET
            UBCSET = RGBPRM(1,JFACE)
            UBCMAX = RGBPRM(2,JFACE)
            UBCFRQ = TWO*PI/RGBPRM(3,JFACE)
            UBCPHA = RGBPRM(4,JFACE)

          ELSEIF(NGBPRM(1,JFACE).EQ.3)THEN

            CONTINUE

          ELSE

            CALL ERHAND(13,33,4)

          ENDIF

C         SET THE TEMPERATURE AND MASS FRACTIONS
          TBCSET = TRIN
          DO JSPEC = 1, NSPEC
            YBCSET(JSPEC) = YRIN(JSPEC)
          ENDDO

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCI3)THEN

C         ---------------------------------------------------------------------

C         SUBSONIC REFLECTING INFLOW WITH SPECIFIED VELOCITY AND DENSITY
C         --------------------------------------------------------------

C         CHECK INLET BC CODE
          IF(NGBPRM(1,JFACE).EQ.1)THEN

C           CONSTANT INLET NORMAL VELOCITY
C           PARAMETERS: I1=1; R1=VELOCITY
            UBCSET = RGBPRM(1,JFACE)

          ELSEIF(NGBPRM(1,JFACE).EQ.2)THEN

C           SINUSOIDAL INLET NORMAL VELOCITY
C           PARAMETERS: I1=2;
C           R1=MEAN VELOCITY, R2=AMPLITUDE, R3=ANGULAR FREQ, R4=PHASE OFFSET
            UBCSET = RGBPRM(1,JFACE)
            UBCMAX = RGBPRM(2,JFACE)
            UBCFRQ = TWO*PI/RGBPRM(3,JFACE)
            UBCPHA = RGBPRM(4,JFACE)

          ELSEIF(NGBPRM(1,JFACE).EQ.3)THEN

C           TURBULENT INLET VELOCITY FIELD
            FLNTRB = .TRUE.

            IF(MOD(ICFLAG,2).EQ.0)THEN

C             COLD START

C             INLET TURBULENCE PARAMETERS
              KMAGLO = NGBPRM(2,JFACE)
              KMAGHI = NGBPRM(3,JFACE)
              IBSEED = NGBPRM(4,JFACE)
              UBCSET = RGBPRM(1,JFACE)
              GSIGMA = RGBPRM(2,JFACE)
              TINCOR = RGBPRM(3,JFACE)
              OMTCOR = ONE/TINCOR

C             BUILD THE LIST Of FORCED WAVENUMBERS
              RMAGLO = REAL(KMAGLO) - RMGTOL
              RMAGHI = REAL(KMAGHI) + RMGTOL
              IKLIST = 0
              DO KMODE2 = 0,KMAGHI
                DO KMODE1 = 0,KMAGHI
                  RMODE1 = REAL(KMODE1)
                  RMODE2 = REAL(KMODE2)
                  RMAGSQ = RMODE1*RMODE1 + RMODE2*RMODE2
                  RMAGNI = SQRT(RMAGSQ)
                  IF((RMAGNI.GT.RMAGLO).AND.(RMAGNI.LT.RMAGHI))THEN
                    IKLIST = IKLIST + 1
                    KMLIST(1,IKLIST) = KMODE1
                    KMLIST(2,IKLIST) = KMODE2
                  ENDIF
                ENDDO
              ENDDO
              NKLIST = IKLIST
              TOVRNK = FOUR/REAL(NKLIST)

C             INITIALISE THE GAUSSIAN RANDOM NUMBER GENERATOR
              CALL RANGIB(IBSEED)

C             INITIALISE THE INLET VELOCITY FIELD
              TMCORR = GSIGMA
              DO IKLIST = 1, NKLIST

                BINPRE(1,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINPIM(1,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINMRE(1,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINMIM(1,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINPRE(2,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINPIM(2,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINMRE(2,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINMIM(2,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINPRE(3,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINPIM(3,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINMRE(3,IKLIST) = RANGAB(IBSEED)*TMCORR
                BINMIM(3,IKLIST) = RANGAB(IBSEED)*TMCORR

              ENDDO

C             INITIALISE THE STORED TRIG FUNCTIONS
              HCBASE = HCSIZE(NBLEVL)
              NXCELL = NINT(XGPHYR/HCBASE)
              NYCELL = NINT(YGPHYR/HCBASE)
              NZCELL = NINT(ZGPHYR/HCBASE)
              NTCELL = MAX(NXCELL,NYCELL,NZCELL)
              OTPHYR = REAL(NTCELL)*HCBASE
              OTPHYR = ONE/OTPHYR
              DO ITCELL = 1, NTCELL
                DO IKLIST = 0, KMAGHI
                  ARGMNT = (REAL(ITCELL)-HALF)*HCBASE
                  ARGMNT = TWOPI*REAL(IKLIST)*ARGMNT*OTPHYR
                  SCOS(IKLIST,ITCELL) = COS(ARGMNT)
                  SSIN(IKLIST,ITCELL) = SIN(ARGMNT)
                ENDDO
              ENDDO

            ELSE

C             WARM START

C             TURBULENT INLET VELOCITY DATA FROM BC I/O FILE
              IF(IPROC.EQ.0)THEN

                OPEN(UNIT=NCBCIO,FILE=FNBCIO,FORM='UNFORMATTED')
                READ(NCBCIO)NKLIST
                READ(NCBCIO)KMLIST
                READ(NCBCIO)RNSTAT
                READ(NCBCIO)BINPRE
                READ(NCBCIO)BINPIM
                READ(NCBCIO)BINMRE
                READ(NCBCIO)BINMIM
                READ(NCBCIO)SCOS
                READ(NCBCIO)SSIN
                CLOSE(NCBCIO)

              ENDIF

C             BROADCAST (OR RECEIVE) THE COUNTER
              IROOT = 0
              CALL P_BCST(NKLIST,1,1,IROOT)
              NCOUNT = 3*NKLIST

C             BROADCAST (OR RECEIVE) THE DATA
              CALL P_BCST(BINPRE,NBCMAX,NCOUNT,IROOT)
              CALL P_BCST(BINPIM,NBCMAX,NCOUNT,IROOT)
              CALL P_BCST(BINMRE,NBCMAX,NCOUNT,IROOT)
              CALL P_BCST(BINMIM,NBCMAX,NCOUNT,IROOT)
              CALL P_BCST(SCOS,NBCMAX,NKLIST,IROOT)
              CALL P_BCST(SSIN,NBCMAX,NKLIST,IROOT)
              CALL P_BCST(RNSTAT,NRNGMX,NRNGMX,IROOT)
              CALL P_BCSI(KMLIST,NBCMAX,NKLIST,IROOT)

C             INLET TURBULENCE PARAMETERS FROM CONTROL FILE
              IBSEED = NGBPRM(4,JFACE)
              UBCSET = RGBPRM(1,JFACE)
              GSIGMA = RGBPRM(2,JFACE)
              TINCOR = RGBPRM(3,JFACE)
              OMTCOR = ONE/TINCOR

              TOVRNK = FOUR/REAL(NKLIST)

C             RESET THE GAUSSIAN RANDOM NUMBER GENERATOR
              CALL RANSEB(RNSTAT)

            ENDIF

          ELSE

            CALL ERHAND(13,33,4)

          ENDIF

C         SET THE TURBULENT VELOCITIES
          DO IKLIST = 1, NKLIST

            UINPRE(1) = BINPRE(1,IKLIST)
            UINPIM(1) = BINPIM(1,IKLIST)
            UINMRE(1) = BINMRE(1,IKLIST)
            UINMIM(1) = BINMIM(1,IKLIST)

            UINPRE(2) = BINPRE(2,IKLIST)
            UINPIM(2) = BINPIM(2,IKLIST)
            UINMRE(2) = BINMRE(2,IKLIST)
            UINMIM(2) = BINMIM(2,IKLIST)

            UINPRE(3) = BINPRE(3,IKLIST)
            UINPIM(3) = BINPIM(3,IKLIST)
            UINMRE(3) = BINMRE(3,IKLIST)
            UINMIM(3) = BINMIM(3,IKLIST)

C           FOURIER-SPACE SYMMETRIES
            KMODE1 = KMLIST(1,IKLIST)
            IF(KMODE1.EQ.0)THEN

              UINPRE(1) = HALF*UINPRE(1)
              UINMRE(1) = UINPRE(1)
              UINPIM(1) = HALF*UINPIM(1)
              UINMIM(1) = UINPIM(1)

              UINPRE(2) = HALF*UINPRE(2)
              UINMRE(2) = UINPRE(2)
              UINPIM(2) = HALF*UINPIM(2)
              UINMIM(2) = UINPIM(2)

              UINPRE(3) = HALF*UINPRE(3)
              UINMRE(3) = UINPRE(3)
              UINPIM(3) = HALF*UINPIM(3)
              UINMIM(3) = UINPIM(3)

            ENDIF

            KMODE2 = KMLIST(2,IKLIST)
            IF(KMODE2.EQ.0)THEN

              UINPRE(1) = HALF*UINPRE(1)
              UINMRE(1) = UINPRE(1)
              UINPIM(1) = HALF*UINPIM(1)
              UINMIM(1) = -UINPIM(1)

              UINPRE(2) = HALF*UINPRE(2)
              UINMRE(2) = UINPRE(2)
              UINPIM(2) = HALF*UINPIM(2)
              UINMIM(2) = -UINPIM(2)

              UINPRE(3) = HALF*UINPRE(3)
              UINMRE(3) = UINPRE(3)
              UINPIM(3) = HALF*UINPIM(3)
              UINMIM(3) = -UINPIM(3)

            ENDIF

            USUMRE(1,IKLIST) = UINPRE(1) + UINMRE(1)
            UDIFRE(1,IKLIST) = UINPRE(1) - UINMRE(1)
            USUMIM(1,IKLIST) = UINPIM(1) + UINMIM(1)
            UDIFIM(1,IKLIST) = UINPIM(1) - UINMIM(1)

            USUMRE(2,IKLIST) = UINPRE(2) + UINMRE(2)
            UDIFRE(2,IKLIST) = UINPRE(2) - UINMRE(2)
            USUMIM(2,IKLIST) = UINPIM(2) + UINMIM(2)
            UDIFIM(2,IKLIST) = UINPIM(2) - UINMIM(2)

            USUMRE(3,IKLIST) = UINPRE(3) + UINMRE(3)
            UDIFRE(3,IKLIST) = UINPRE(3) - UINMRE(3)
            USUMIM(3,IKLIST) = UINPIM(3) + UINMIM(3)
            UDIFIM(3,IKLIST) = UINPIM(3) - UINMIM(3)

          ENDDO

C         STATISTICS
          NBCAVE = 0
          UBCAVE(1) = ZERO
          UBCAVE(2) = ZERO
          UBCAVE(3) = ZERO
          UBCVAR(1) = ZERO
          UBCVAR(2) = ZERO
          UBCVAR(3) = ZERO

C         INITIALISE THE STORED BC FLAG ARRAY
          DO JCELL = 1,NBCVAL
            FBCSTO(JCELL) = .FALSE.
          ENDDO

C         SET THE DENSITY AND MASS FRACTIONS
          DBCSET = DRIN
          DO JSPEC = 1, NSPEC
            YBCSET(JSPEC) = YRIN(JSPEC)
          ENDDO

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCW1)THEN

C         ---------------------------------------------------------------------

C         BOUNDARY CONDITION: ADIABATIC WALL
C         ----------------------------------
C         NOTHING TO SET AT THIS POINT
          CONTINUE

C         ---------------------------------------------------------------------

        ELSEIF(IGLOBC.EQ.NSBCW2)THEN

C         ---------------------------------------------------------------------

C         BOUNDARY CONDITION: ISOTHERMAL WALL
C         -----------------------------------
C         SET CONSTANT WALL TEMPERATURE
C         PARAMETERS: I1=1; R4=WALL TEMPERATURE
          TBCSET = RGBPRM(4,JFACE)

          CONTINUE

        ELSEIF(IGLOBC.EQ.NSBCW3)THEN

C         ---------------------------------------------------------------------

C         BOUNDARY CONDITION: ADIABATIC SLIP WALL
C         ---------------------------------------
C         NOTHING TO SET AT THIS POINT
          CONTINUE

C         ---------------------------------------------------------------------
C         ---------------------------------------------------------------------

        ENDIF
C       CHECK GLOBAL BC CODE

C       =======================================================================

      ENDDO
C     RUN THROUGH FACES

C     =========================================================================


      RETURN
      END
