      SUBROUTINE BCSYNC
 
C     *************************************************************************
C
C     BCSYNC
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SYNCHRONISES THE TIME-DEPENDENT BOUNDARY CONDITIONS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     FUNCTIONS
C     =========
      DOUBLE PRECISION RANGAB
      EXTERNAL RANGAB


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION TDELTA,TMCORR,OMCORR
      DOUBLE PRECISION UINPRE(NCMPMX),UINMRE(NCMPMX)
      DOUBLE PRECISION UINPIM(NCMPMX),UINMIM(NCMPMX)
      DOUBLE PRECISION OBCAVE,TKEVAR
      INTEGER JCELL,IKLIST,KMODE1,KMODE2


C     BEGIN
C     =====

C     =========================================================================

      IF(FLNTRB)THEN

C       UPDATE THE INLET TURBULENT VELOCITY FIELD 

C       TIME CORRELATION
        TDELTA = TSTEP
        IF(IRKSTP.LT.NRKSTP)TDELTA = RKTIM(IRKSTP+1)
        TDELTA = TDELTA - RKTIM(IRKSTP)
        TMCORR = TDELTA*OMTCOR
        OMCORR = ONE - TMCORR
        TMCORR = SQRT(TWO*ABS(TMCORR))*GSIGMA

C       UHLENBECK-ORNSTEIN PROCESSES
        DO IKLIST = 1, NKLIST

C         UPDATE THE STORED U-O VALUES
          BINPRE(1,IKLIST) = BINPRE(1,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINPIM(1,IKLIST) = BINPIM(1,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINMRE(1,IKLIST) = BINMRE(1,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINMIM(1,IKLIST) = BINMIM(1,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINPRE(2,IKLIST) = BINPRE(2,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINPIM(2,IKLIST) = BINPIM(2,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINMRE(2,IKLIST) = BINMRE(2,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINMIM(2,IKLIST) = BINMIM(2,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINPRE(3,IKLIST) = BINPRE(3,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINPIM(3,IKLIST) = BINPIM(3,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINMRE(3,IKLIST) = BINMRE(3,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR
          BINMIM(3,IKLIST) = BINMIM(3,IKLIST)*OMCORR
     +                     + RANGAB(IBSEED)*TMCORR

        ENDDO

C       SET THE TURBULENT VELOCITIES
        DO IKLIST = 1, NKLIST

          UINPRE(1) = BINPRE(1,IKLIST)
          UINPIM(1) = BINPIM(1,IKLIST)
          UINMRE(1) = BINMRE(1,IKLIST)
          UINMIM(1) = BINMIM(1,IKLIST)

          UINPRE(2) = BINPRE(2,IKLIST)
          UINPIM(2) = BINPIM(2,IKLIST)
          UINMRE(2) = BINMRE(2,IKLIST)
          UINMIM(2) = BINMIM(2,IKLIST)

          UINPRE(3) = BINPRE(3,IKLIST)
          UINPIM(3) = BINPIM(3,IKLIST)
          UINMRE(3) = BINMRE(3,IKLIST)
          UINMIM(3) = BINMIM(3,IKLIST)

C         FOURIER-SPACE SYMMETRIES
          KMODE1 = KMLIST(1,IKLIST)
          IF(KMODE1.EQ.0)THEN

            UINPRE(1) = HALF*UINPRE(1)
            UINMRE(1) = UINPRE(1)
            UINPIM(1) = HALF*UINPIM(1)
            UINMIM(1) = UINPIM(1)

            UINPRE(2) = HALF*UINPRE(2)
            UINMRE(2) = UINPRE(2)
            UINPIM(2) = HALF*UINPIM(2)
            UINMIM(2) = UINPIM(2)

            UINPRE(3) = HALF*UINPRE(3)
            UINMRE(3) = UINPRE(3)
            UINPIM(3) = HALF*UINPIM(3)
            UINMIM(3) = UINPIM(3)

          ENDIF

          KMODE2 = KMLIST(2,IKLIST)
          IF(KMODE2.EQ.0)THEN

            UINPRE(1) = HALF*UINPRE(1)
            UINMRE(1) = UINPRE(1)
            UINPIM(1) = HALF*UINPIM(1)
            UINMIM(1) = -UINPIM(1)

            UINPRE(2) = HALF*UINPRE(2)
            UINMRE(2) = UINPRE(2)
            UINPIM(2) = HALF*UINPIM(2)
            UINMIM(2) = -UINPIM(2)

            UINPRE(3) = HALF*UINPRE(3)
            UINMRE(3) = UINPRE(3)
            UINPIM(3) = HALF*UINPIM(3)
            UINMIM(3) = -UINPIM(3)

          ENDIF

          USUMRE(1,IKLIST) = UINPRE(1) + UINMRE(1)
          UDIFRE(1,IKLIST) = UINPRE(1) - UINMRE(1)
          USUMIM(1,IKLIST) = UINPIM(1) + UINMIM(1)
          UDIFIM(1,IKLIST) = UINPIM(1) - UINMIM(1)

          USUMRE(2,IKLIST) = UINPRE(2) + UINMRE(2)
          UDIFRE(2,IKLIST) = UINPRE(2) - UINMRE(2)
          USUMIM(2,IKLIST) = UINPIM(2) + UINMIM(2)
          UDIFIM(2,IKLIST) = UINPIM(2) - UINMIM(2)

          USUMRE(3,IKLIST) = UINPRE(3) + UINMRE(3)
          UDIFRE(3,IKLIST) = UINPRE(3) - UINMRE(3)
          USUMIM(3,IKLIST) = UINPIM(3) + UINMIM(3)
          UDIFIM(3,IKLIST) = UINPIM(3) - UINMIM(3)

        ENDDO

C       STATISTICS
        OBCAVE = ONE
        IF(NBCAVE.GT.1)OBCAVE = ONE/REAL(NBCAVE)
        UBCAVE(1) = UBCAVE(1)*OBCAVE
        UBCAVE(2) = UBCAVE(2)*OBCAVE
        UBCAVE(3) = UBCAVE(3)*OBCAVE
        UBCVAR(1) = UBCVAR(1)*OBCAVE
        UBCVAR(2) = UBCVAR(2)*OBCAVE
        UBCVAR(3) = UBCVAR(3)*OBCAVE
        TKEVAR = HALF*(UBCVAR(1)+UBCVAR(2)+UBCVAR(3))

C        WRITE(6,'(3I5,3(1PE12.4))')ITIME,IRKSTP,NBCAVE,
C     +                             UBCAVE(1),UBCAVE(2),UBCAVE(3)
C        WRITE(6,'(15X,4(1PE12.4))')UBCVAR(1),UBCVAR(2),UBCVAR(3),TKEVAR
C        WRITE(6,'(15X,4(1PE12.4))')
C     +      SQRT(UBCVAR(1)),SQRT(UBCVAR(2)),SQRT(UBCVAR(3)),
C     +      THRD*(SQRT(UBCVAR(1))+SQRT(UBCVAR(2))+SQRT(UBCVAR(3)))

        NBCAVE = 0
        UBCAVE(1) = ZERO
        UBCAVE(2) = ZERO
        UBCAVE(3) = ZERO
        UBCVAR(1) = ZERO
        UBCVAR(2) = ZERO
        UBCVAR(3) = ZERO

C       REINITIALISE THE STORED BC FLAG ARRAY
        DO JCELL = 1, NBCVAL
          FBCSTO(JCELL) = .FALSE.
        ENDDO

      ENDIF

C     =========================================================================


      RETURN
      END
