      SUBROUTINE BCVELC(JINDEX,BTIME,UBCVAL)

C     *************************************************************************
C
C     BCVELC
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     20-JUL-2008:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     IMPOSES BC FOR VELOCITY COMPONENTS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION RTOLER
      PARAMETER(RTOLER = 1.0D-8)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION UBCVAL(NCMPMX)
      DOUBLE PRECISION BTIME
      INTEGER JINDEX


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION UBCTRB(NCMPMX)
      DOUBLE PRECISION SSUMRE,SDIFRE,SSUMIM,SDIFIM
      DOUBLE PRECISION SCOSK1,SSINK1,SCOSK2,SSINK2
      DOUBLE PRECISION BLTIME,XCOORD,YCOORD,ZCOORD
      DOUBLE PRECISION ARGMNT,OVBASE
      DOUBLE PRECISION RCELL1,RCELL2
      INTEGER JBCELL,JBFACE,JBDIRN
      INTEGER ICELL1,ICELL2,KMODE1,KMODE2,IKLIST
      LOGICAL FLINBM


C     =========================================================================

C     BEGIN
C     =====

C     FIND BC COORDINATES
      BLTIME = BTIME
      XCOORD = BCOORD(1,JINDEX)
      YCOORD = BCOORD(2,JINDEX)
      ZCOORD = BCOORD(3,JINDEX)
      JBCELL = NBCCEL(JINDEX)
      JBFACE = NBCFAC(JINDEX)
      JBDIRN = MDIRN(JBFACE)

C     INITIALISE LOCAL INLET VELOCITIES
      UBCVAL(1) = ZERO
      UBCVAL(2) = ZERO
      UBCVAL(3) = ZERO

C     =========================================================================

      IF(NGBPRM(1,JBFACE).EQ.1)THEN

C       -----------------------------------------------------------------------

C       CONSTANT INLET NORMAL VELOCITY
        UBCVAL(JBDIRN) = UBCSET

C       -----------------------------------------------------------------------


      ELSEIF(NGBPRM(1,JBFACE).EQ.2)THEN

C       -----------------------------------------------------------------------

C       SINUSOIDAL U-VELOCITY
        ARGMNT = UBCFRQ*BLTIME + UBCPHA
        UBCVAL(JBDIRN) = UBCSET + UBCMAX*SIN(ARGMNT)

C       -----------------------------------------------------------------------

      ELSEIF(NGBPRM(1,JBFACE).EQ.3)THEN

C       -----------------------------------------------------------------------

C       TURBULENT VELOCITY FIELD
C       PARAMETER I1=3

C       CHECK FOR STORED VALUES
        IF(FBCSTO(JINDEX))THEN

C         VALUES STORED - RETRIEVE
          UBCVAL(1) = UBCSTO(1,JINDEX) 
          UBCVAL(2) = UBCSTO(2,JINDEX) 
          UBCVAL(3) = UBCSTO(3,JINDEX) 

        ELSE

C         NO VALUES STORED - EVALUATE ANEW

C         LOCAL COORDINATES
          OVBASE = ONE/HCSIZE(NBLEVL)
          RCELL1 = ZERO
          RCELL2 = ZERO
          IF(JBDIRN.EQ.1)THEN
            RCELL1 = YCOORD*OVBASE
            RCELL2 = ZCOORD*OVBASE
          ELSE IF(JBDIRN.EQ.2)THEN
            RCELL1 = XCOORD*OVBASE
            RCELL2 = ZCOORD*OVBASE
          ELSE IF(JBDIRN.EQ.3)THEN
            RCELL1 = XCOORD*OVBASE
            RCELL2 = YCOORD*OVBASE
          ENDIF
          RCELL1 = RCELL1 + HALF
          RCELL2 = RCELL2 + HALF
          ICELL1 = NINT(RCELL1)
          ICELL2 = NINT(RCELL2)
          FLINBM = (ABS(RCELL1-REAL(ICELL1)).LT.RTOLER)
          FLINBM = FLINBM.AND.(ABS(RCELL2-REAL(ICELL2)).LT.RTOLER)

C         EVALUATE DFT
          UBCTRB(1) = ZERO
          UBCTRB(2) = ZERO
          UBCTRB(3) = ZERO
          IF(FLINBM)THEN

C           POINT IS ON THE INLET BASE MESH
            DO IKLIST = 1, NKLIST

              KMODE1 = KMLIST(1,IKLIST)
              KMODE2 = KMLIST(2,IKLIST)

              SSUMRE = SCOS(KMODE1,ICELL1)*SCOS(KMODE2,ICELL2)
              SDIFRE = SSIN(KMODE1,ICELL1)*SSIN(KMODE2,ICELL2)
              SSUMIM = SCOS(KMODE1,ICELL1)*SSIN(KMODE2,ICELL2)
              SDIFIM = SSIN(KMODE1,ICELL1)*SCOS(KMODE2,ICELL2)
  
              UBCTRB(1) = UBCTRB(1) + USUMRE(1,IKLIST)*SSUMRE
     +                              - UDIFRE(1,IKLIST)*SDIFRE
     +                              + USUMIM(1,IKLIST)*SSUMIM
     +                              + UDIFIM(1,IKLIST)*SDIFIM

              UBCTRB(2) = UBCTRB(2) + USUMRE(2,IKLIST)*SSUMRE
     +                              - UDIFRE(2,IKLIST)*SDIFRE
     +                              + USUMIM(2,IKLIST)*SSUMIM
     +                              + UDIFIM(2,IKLIST)*SDIFIM
  
              UBCTRB(3) = UBCTRB(3) + USUMRE(3,IKLIST)*SSUMRE
     +                              - UDIFRE(3,IKLIST)*SDIFRE
     +                              + USUMIM(3,IKLIST)*SSUMIM
     +                              + UDIFIM(3,IKLIST)*SDIFIM

            ENDDO
            
          ELSE

C           POINT IS OFF THE INLET BASE MESH
            DO IKLIST = 1, NKLIST

              KMODE1 = KMLIST(1,IKLIST)
              KMODE2 = KMLIST(2,IKLIST)

              ARGMNT = TWOPI*REAL(KMODE1)*RCELL1
              SCOSK1 = COS(ARGMNT)
              SSINK1 = SIN(ARGMNT)
              ARGMNT = TWOPI*REAL(KMODE2)*RCELL2
              SCOSK2 = COS(ARGMNT)
              SSINK2 = SIN(ARGMNT)

              SSUMRE = SCOSK1*SCOSK2
              SDIFRE = SSINK1*SSINK2
              SSUMIM = SCOSK1*SSINK2
              SDIFIM = SSINK1*SCOSK2

              UBCTRB(1) = UBCTRB(1) + USUMRE(1,IKLIST)*SSUMRE
     +                              - UDIFRE(1,IKLIST)*SDIFRE
     +                              + USUMIM(1,IKLIST)*SSUMIM
     +                              + UDIFIM(1,IKLIST)*SDIFIM

              UBCTRB(2) = UBCTRB(2) + USUMRE(2,IKLIST)*SSUMRE
     +                              - UDIFRE(2,IKLIST)*SDIFRE
     +                              + USUMIM(2,IKLIST)*SSUMIM
     +                              + UDIFIM(2,IKLIST)*SDIFIM

              UBCTRB(3) = UBCTRB(3) + USUMRE(3,IKLIST)*SSUMRE
     +                              - UDIFRE(3,IKLIST)*SDIFRE
     +                              + USUMIM(3,IKLIST)*SSUMIM
     +                              + UDIFIM(3,IKLIST)*SDIFIM

            ENDDO

          ENDIF

C         NORMALISE
          UBCVAL(1) = UBCTRB(1)*TOVRNK
          UBCVAL(2) = UBCTRB(2)*TOVRNK
          UBCVAL(3) = UBCTRB(3)*TOVRNK

C         STATISTICS
          NBCAVE = NBCAVE + 1
          UBCAVE(1) = UBCAVE(1) + UBCVAL(1)
          UBCAVE(2) = UBCAVE(2) + UBCVAL(2)
          UBCAVE(3) = UBCAVE(3) + UBCVAL(3)
          UBCVAR(1) = UBCVAR(1) + UBCVAL(1)*UBCVAL(1)
          UBCVAR(2) = UBCVAR(2) + UBCVAL(2)*UBCVAL(2)
          UBCVAR(3) = UBCVAR(3) + UBCVAL(3)*UBCVAL(3)

C         ADD THE MEAN INLET VELOCITY
          UBCVAL(JBDIRN) = UBCTRB(JBDIRN) + UBCSET

C         STORE
          UBCSTO(1,JINDEX) = UBCVAL(1)
          UBCSTO(2,JINDEX) = UBCVAL(2)
          UBCSTO(3,JINDEX) = UBCVAL(3)
          FBCSTO(JINDEX) = .TRUE.

        ENDIF

C       -----------------------------------------------------------------------

      ELSE

C       -----------------------------------------------------------------------

C       ERROR CONDITION - ALREADY HANDLED IN BCINIT
        CONTINUE

C       -----------------------------------------------------------------------

      ENDIF

C     =========================================================================


      RETURN
      END
