      SUBROUTINE CDVALY(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
 
C     *************************************************************************
C
C     CDVALY
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     27-SEP-2014:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     POLYNOMIAL EVALUATION
C     FOR ALL DERIVATIVES
C     IN Y FOR FLUX EVALUATION
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      DOUBLE PRECISION YDVAL(NSPCMX)
      DOUBLE PRECISION UDVAL(NCMPMX,NCMPMX)
      DOUBLE PRECISION DDVAL(NCMPMX)
      DOUBLE PRECISION XLOC,YLOC,ZLOC
      DOUBLE PRECISION EDVAL


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION PMONOM(NPCFMX)
      INTEGER IYSPEC


C     BEGIN
C     =====

C     =========================================================================

C     BUILD THE MONOMIALS
C     -------------------
      PMONOM(1) = XLOC
      PMONOM(2) = XLOC*XLOC

      PMONOM(3) = YLOC
      PMONOM(4) = YLOC*XLOC
      PMONOM(5) = YLOC*YLOC

      PMONOM(6) = ZLOC
      PMONOM(7) = ZLOC*XLOC
      PMONOM(8) = ZLOC*YLOC
      PMONOM(9) = ZLOC*ZLOC

C     =========================================================================

C     ASSEMBLE THE POLYNOMIAL FOR THE DERIVATIVES
C     -----------------------

C     DENSITY
C     -------
C     X-DERIVATIVE
      DDVAL(1) = PCODFD(1)
     +         + PCODFD(2)*PMONOM(1)*TWO
     +         + PCODFD(3)*PMONOM(2)*THREE
     +         + PCODFD(5)*PMONOM(3)
     +         + PCODFD(7)*PMONOM(4)*TWO
     +         + PCODFD(8)*PMONOM(5)
     +         + PCODFD(11)*PMONOM(6)
     +         + PCODFD(14)*PMONOM(7)*TWO
     +         + PCODFD(15)*PMONOM(8)
     +         + PCODFD(16)*PMONOM(9)

C     Y-DERIVATIVE
      DDVAL(2) = PCODFD(4)
     +         + PCODFD(5)*PMONOM(1)
     +         + PCODFD(6)*PMONOM(3)*TWO
     +         + PCODFD(7)*PMONOM(2)
     +         + PCODFD(8)*PMONOM(4)*TWO
     +         + PCODFD(9)*PMONOM(5)*THREE
     +         + PCODFD(12)*PMONOM(6)
     +         + PCODFD(15)*PMONOM(7)
     +         + PCODFD(17)*PMONOM(8)*TWO
     +         + PCODFD(18)*PMONOM(9)

C     Z-DERIVATIVE
      DDVAL(3) = PCODFD(10)
     +         + PCODFD(11)*PMONOM(1)
     +         + PCODFD(12)*PMONOM(3)
     +         + PCODFD(13)*PMONOM(6)*TWO
     +         + PCODFD(14)*PMONOM(2)
     +         + PCODFD(15)*PMONOM(4)
     +         + PCODFD(16)*PMONOM(7)*TWO
     +         + PCODFD(17)*PMONOM(5)
     +         + PCODFD(18)*PMONOM(8)*TWO
     +         + PCODFD(19)*PMONOM(9)*THREE

C     -------------------------------------------------------------------------

C     VELOCITY
C     --------
C     DUDX,DUDY
      UDVAL(1,1) = PCODFU(1,1)
     +           + PCODFU(2,1)*PMONOM(1)*TWO
     +           + PCODFU(3,1)*PMONOM(2)*THREE
     +           + PCODFU(5,1)*PMONOM(3)
     +           + PCODFU(7,1)*PMONOM(4)*TWO
     +           + PCODFU(8,1)*PMONOM(5)
     +           + PCODFU(11,1)*PMONOM(6)
     +           + PCODFU(14,1)*PMONOM(7)*TWO
     +           + PCODFU(15,1)*PMONOM(8)
     +           + PCODFU(16,1)*PMONOM(9)

      UDVAL(1,2) = PCODFU(4,1)
     +           + PCODFU(5,1)*PMONOM(1)
     +           + PCODFU(6,1)*PMONOM(3)*TWO
     +           + PCODFU(7,1)*PMONOM(2)
     +           + PCODFU(8,1)*PMONOM(4)*TWO
     +           + PCODFU(9,1)*PMONOM(5)*THREE
     +           + PCODFU(12,1)*PMONOM(6)
     +           + PCODFU(15,1)*PMONOM(7)
     +           + PCODFU(17,1)*PMONOM(8)*TWO
     +           + PCODFU(18,1)*PMONOM(9)

C     DVDX,DVDY,DVDZ
      UDVAL(2,1) = PCODFU(1,2)
     +           + PCODFU(2,2)*PMONOM(1)*TWO
     +           + PCODFU(3,2)*PMONOM(2)*THREE
     +           + PCODFU(5,2)*PMONOM(3)
     +           + PCODFU(7,2)*PMONOM(4)*TWO
     +           + PCODFU(8,2)*PMONOM(5)
     +           + PCODFU(11,2)*PMONOM(6)
     +           + PCODFU(14,2)*PMONOM(7)*TWO
     +           + PCODFU(15,2)*PMONOM(8)
     +           + PCODFU(16,2)*PMONOM(9)

      UDVAL(2,2) = PCODFU(4,2)
     +           + PCODFU(5,2)*PMONOM(1)
     +           + PCODFU(6,2)*PMONOM(3)*TWO
     +           + PCODFU(7,2)*PMONOM(2)
     +           + PCODFU(8,2)*PMONOM(4)*TWO
     +           + PCODFU(9,2)*PMONOM(5)*THREE
     +           + PCODFU(12,2)*PMONOM(6)
     +           + PCODFU(15,2)*PMONOM(7)
     +           + PCODFU(17,2)*PMONOM(8)*TWO
     +           + PCODFU(18,2)*PMONOM(9)

      UDVAL(2,3) = PCODFU(10,2)
     +           + PCODFU(11,2)*PMONOM(1)
     +           + PCODFU(12,2)*PMONOM(3)
     +           + PCODFU(13,2)*PMONOM(6)*TWO
     +           + PCODFU(14,2)*PMONOM(2)
     +           + PCODFU(15,2)*PMONOM(4)
     +           + PCODFU(16,2)*PMONOM(7)*TWO
     +           + PCODFU(17,2)*PMONOM(5)
     +           + PCODFU(18,2)*PMONOM(8)*TWO
     +           + PCODFU(19,2)*PMONOM(9)*THREE

C     DWDY,DWDZ
      UDVAL(3,2) = PCODFU(4,3)
     +           + PCODFU(5,3)*PMONOM(1)
     +           + PCODFU(6,3)*PMONOM(3)*TWO
     +           + PCODFU(7,3)*PMONOM(2)
     +           + PCODFU(8,3)*PMONOM(4)*TWO
     +           + PCODFU(9,3)*PMONOM(5)*THREE
     +           + PCODFU(12,3)*PMONOM(6)
     +           + PCODFU(15,3)*PMONOM(7)
     +           + PCODFU(17,3)*PMONOM(8)*TWO
     +           + PCODFU(18,3)*PMONOM(9)

      UDVAL(3,3) = PCODFU(10,3)
     +           + PCODFU(11,3)*PMONOM(1)
     +           + PCODFU(12,3)*PMONOM(3)
     +           + PCODFU(13,3)*PMONOM(6)*TWO
     +           + PCODFU(14,3)*PMONOM(2)
     +           + PCODFU(15,3)*PMONOM(4)
     +           + PCODFU(16,3)*PMONOM(7)*TWO
     +           + PCODFU(17,3)*PMONOM(5)
     +           + PCODFU(18,3)*PMONOM(8)*TWO
     +           + PCODFU(19,3)*PMONOM(9)*THREE


C     DUDZ,DWDX NOT REQUIRED HERE
      UDVAL(1,3) = ZERO
      UDVAL(3,1) = ZERO

C     -------------------------------------------------------------------------

C     INTERNAL ENERGY
C     ---------------
C     Y-DERIVATIVE
      EDVAL = PCODFE(4)
     +      + PCODFE(5)*PMONOM(1)
     +      + PCODFE(6)*PMONOM(3)*TWO
     +      + PCODFE(7)*PMONOM(2)
     +      + PCODFE(8)*PMONOM(4)*TWO
     +      + PCODFE(9)*PMONOM(5)*THREE
     +      + PCODFE(12)*PMONOM(6)
     +      + PCODFE(15)*PMONOM(7)
     +      + PCODFE(17)*PMONOM(8)*TWO
     +      + PCODFE(18)*PMONOM(9)

C     -------------------------------------------------------------------------

C     SPECIES MASS FRACTION
C     ---------------------
C     Y-DERIVATIVES
      DO IYSPEC = 1,NSPEC

        YDVAL(IYSPEC) = PCODFY(4,IYSPEC)
     +                + PCODFY(5,IYSPEC)*PMONOM(1)
     +                + PCODFY(6,IYSPEC)*PMONOM(3)*TWO
     +                + PCODFY(7,IYSPEC)*PMONOM(2)
     +                + PCODFY(8,IYSPEC)*PMONOM(4)*TWO
     +                + PCODFY(9,IYSPEC)*PMONOM(5)*THREE
     +                + PCODFY(12,IYSPEC)*PMONOM(6)
     +                + PCODFY(15,IYSPEC)*PMONOM(7)
     +                + PCODFY(17,IYSPEC)*PMONOM(8)*TWO
     +                + PCODFY(18,IYSPEC)*PMONOM(9)

      ENDDO

C     =========================================================================


      RETURN
      END
