      SUBROUTINE CEVLGP(IFCODE)
 
C     *************************************************************************
C
C     CEVLGP
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-AUG-2014:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     POLYNOMIAL RECONSTRUCTION IN EACH CELL
C     EVALUATES THE POLYNOMIAL AT ALL GAUSS POINTS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENT
C     ========
      INTEGER IFCODE(NFACMX)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YEVAL(NSPCMX),YDVAL(NSPCMX)
      DOUBLE PRECISION UDVAL(NCMPMX,NCMPMX)
      DOUBLE PRECISION UEVAL(NCMPMX)
      DOUBLE PRECISION DDVAL(NCMPMX)
      DOUBLE PRECISION XLOC,YLOC,ZLOC
      DOUBLE PRECISION DEVAL,EEVAL,TEVAL,PEVAL
      DOUBLE PRECISION EDVAL,TDVAL,PDVAL,DPVAL
      DOUBLE PRECISION CPEVAL,CVEVAL,RCEVAL
      DOUBLE PRECISION VISTNL,VIST1L,VIST2L,DIVUCF
      DOUBLE PRECISION OCSIZE
      INTEGER IDIRN,ILEVL
      INTEGER IDRP1,IDRP2
      INTEGER IFSUB
      INTEGER JFACE,JSPEC
      INTEGER JGPVOL,JGPFAC,JGPIND

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In CEVLGP',ICELL

C     =========================================================================

C     LEVEL AND RECIPROCAL SIZE OF LOCAL CELL
      ILEVL = NLEVEL(ICELL)
      OCSIZE = OVSIZE(ILEVL)

C     =========================================================================

C     CELL CENTRE VALUES
      XLOC = ZERO
      YLOC = ZERO
      ZLOC = ZERO

C     EVALUATE THE POLYNOMIAL
      CALL CEVALL(XLOC,YLOC,ZLOC,DEVAL,UEVAL,EEVAL,YEVAL)

C     TEMPERATURE AND PRIMITIVE VARIABLES
      CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C     MIXTURE SPECIFIC HEAT CP AND MIXTURE GAS CONSTANT
      CALL CPMVAL(TEVAL,YEVAL,CPEVAL,RCEVAL)

C     PRESSURE
      PEVAL = DEVAL*RCEVAL*TEVAL - PREFER

C     STORE THE VARIABLES
      DEVALC = DEVAL
      UEVALC(1) = UEVAL(1)
      UEVALC(2) = UEVAL(2)
      UEVALC(3) = UEVAL(3)
      TRUN(ICELL) = TEVAL
      PRUN(ICELL) = PEVAL
      DO JSPEC = 1, NSPEC
        YEVALC(JSPEC) = YEVAL(JSPEC)
      ENDDO

C     =========================================================================

C     EVALUATE THE POLYNOMIAL AT THE GAUSS POINTS
C     ===========================================

C     RUN THROUGH THE GAUSS POINTS FOR VOLUME INTEGRATION
      DO JGPVOL = 1, NGPVOL

C       LOCATION OF GAUSS POINT
        XLOC = GAUSVX(JGPVOL)
        YLOC = GAUSVY(JGPVOL)
        ZLOC = GAUSVZ(JGPVOL)

C       EVALUATE THE POLYNOMIAL
        CALL CEVALL(XLOC,YLOC,ZLOC,DEVAL,UEVAL,EEVAL,YEVAL)

C       TEMPERATURE AND PRIMITIVE VARIABLES
        CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C       MIXTURE SPECIFIC HEAT CP AND MIXTURE GAS CONSTANT
        CALL CPMVAL(TEVAL,YEVAL,CPEVAL,RCEVAL)

C       PRESSURE
        PEVAL = DEVAL*RCEVAL*TEVAL - PREFER

C       STORE THE VARIABLES
        DGVALS(JGPVOL) = DEVAL
        UGVALS(1,JGPVOL) = UEVAL(1)
        UGVALS(2,JGPVOL) = UEVAL(2)
        UGVALS(3,JGPVOL) = UEVAL(3)
        TGVALS(JGPVOL) = TEVAL
        PGVALS(JGPVOL) = PEVAL
        DO JSPEC = 1, NSPEC
          YGVALS(JSPEC,JGPVOL) = YEVAL(JSPEC)
        ENDDO
        CPVALS(JGPVOL) = CPEVAL
        RGVALS(JGPVOL) = RCEVAL

      ENDDO

C     -------------------------------------------------------------------------

C     RUN THROUGH ALL FACES
      DO JFACE = 1, NFACE

C       -----------------------------------------------------------------------

C       DIRECTION INDEX
        IDIRN = MDIRN(JFACE)

C       -----------------------------------------------------------------------

        IF(IFCODE(JFACE).EQ.NCLEAN)THEN

C         ---------------------------------------------------------------------

C         RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION
          DO JGPFAC = 1, NGPFAC

C           LOCATION OF GAUSS POINT
            XLOC = GAUSFX(JGPFAC,JFACE)
            YLOC = GAUSFY(JGPFAC,JFACE)
            ZLOC = GAUSFZ(JGPFAC,JFACE)

C           CELL FACE VALUES

C           EVALUATE THE POLYNOMIAL
            CALL CEVALL(XLOC,YLOC,ZLOC,DEVAL,UEVAL,EEVAL,YEVAL)

C           TEMPERATURE AND PRIMITIVE VARIABLES
            CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C           MIXTURE SPECIFIC HEAT CP AND MIXTURE GAS CONSTANT
            CALL CPMVAL(TEVAL,YEVAL,CPEVAL,RCEVAL)
            CVEVAL = CPEVAL - RCEVAL

C           PRESSURE
            PEVAL = DEVAL*RCEVAL*TEVAL - PREFER

C           STORE THE VARIABLES
            DGVALF(JGPFAC,JFACE) = DEVAL
            UGVALF(1,JGPFAC,JFACE) = UEVAL(1)
            UGVALF(2,JGPFAC,JFACE) = UEVAL(2)
            UGVALF(3,JGPFAC,JFACE) = UEVAL(3)
            EGVALF(JGPFAC,JFACE) = EEVAL
            TGVALF(JGPFAC,JFACE) = TEVAL
            PGVALF(JGPFAC,JFACE) = PEVAL
            DO JSPEC = 1, NSPEC
              YGVALF(JSPEC,JGPFAC,JFACE) = YEVAL(JSPEC)
            ENDDO
            CPVALF(JGPFAC,JFACE) = CPEVAL
            RGVALF(JGPFAC,JFACE) = RCEVAL
  
C           CELL FACE VALUES FOR MOLECULAR TRANSPORT
            DMVALF(JGPFAC,JFACE) = DEVAL
            TMVALF(JGPFAC,JFACE) = TEVAL
            PMVALF(JGPFAC,JFACE) = PEVAL
            DO JSPEC = 1, NSPEC
              YMVALF(JSPEC,JGPFAC,JFACE) = YEVAL(JSPEC)
            ENDDO

C           CELL FACE DERIVATIVES

C           EVALUATE THE POLYNOMIAL
            IF(IDIRN.EQ.1)
     +        CALL CDVALX(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
            IF(IDIRN.EQ.2)
     +        CALL CDVALY(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
            IF(IDIRN.EQ.3)
     +        CALL CDVALZ(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)

C           DERIVATIVES OF PRIMITIVE VARIABLES
            CALL CDPRIM(DEVAL,EEVAL,UEVAL,TEVAL,YEVAL,CVEVAL,
     +                  DDVAL,EDVAL,UDVAL,TDVAL,YDVAL,IDIRN)

C           VISCOUS STRESS TENSOR
            IDRP1 = MNVIST(1,IDIRN)
            IDRP2 = MNVIST(2,IDIRN)
            DIVUCF = UDVAL(1,1) + UDVAL(2,2) + UDVAL(3,3)
            VISTNL = TWO*UDVAL(IDIRN,IDIRN) - TTHD*DIVUCF
            VIST1L = UDVAL(IDRP1,IDIRN) + UDVAL(IDIRN,IDRP1)
            VIST2L = UDVAL(IDRP2,IDIRN) + UDVAL(IDIRN,IDRP2)
            UGRADF(1,JGPFAC,JFACE) = VISTNL*OCSIZE
            UGRADF(2,JGPFAC,JFACE) = VIST1L*OCSIZE
            UGRADF(3,JGPFAC,JFACE) = VIST2L*OCSIZE

C           DIFFUSIVE TERMS
            TGRADF(JGPFAC,JFACE) = TDVAL*OCSIZE
            DO JSPEC = 1, NSPEC
              YGRADF(JSPEC,JGPFAC,JFACE) = YDVAL(JSPEC)*OCSIZE
            ENDDO
 
          ENDDO
C         END OF RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(JFACE).EQ.N2HTOH)THEN

C         ---------------------------------------------------------------------

C         GAUSS POINT INDEXING
          JGPIND = 0

C         RUN THROUGH ALL SUBFACES
          DO IFSUB = 1,NFSUB

C           RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION
            DO JGPFAC = 1, NGPFAC

C             LOCATION OF GAUSS POINT
              XLOC = HALF*GAUSFX(JGPFAC,JFACE) + SUBFAX(IFSUB,JFACE)
              YLOC = HALF*GAUSFY(JGPFAC,JFACE) + SUBFAY(IFSUB,JFACE)
              ZLOC = HALF*GAUSFZ(JGPFAC,JFACE) + SUBFAZ(IFSUB,JFACE)

C             CELL FACE VALUES

C             EVALUATE THE POLYNOMIAL
              CALL CEVALL(XLOC,YLOC,ZLOC,DEVAL,UEVAL,EEVAL,YEVAL)

C             TEMPERATURE AND PRIMITIVE VARIABLES
              CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C             MIXTURE SPECIFIC HEAT CP AND MIXTURE GAS CONSTANT
              CALL CPMVAL(TEVAL,YEVAL,CPEVAL,RCEVAL)
              CVEVAL = CPEVAL - RCEVAL

C             PRESSURE
              PEVAL = DEVAL*RCEVAL*TEVAL - PREFER

C             STORE THE VARIABLES
              JGPIND = JGPIND + 1
              DGVALF(JGPIND,JFACE) = DEVAL
              UGVALF(1,JGPIND,JFACE) = UEVAL(1)
              UGVALF(2,JGPIND,JFACE) = UEVAL(2)
              UGVALF(3,JGPIND,JFACE) = UEVAL(3)
              TGVALF(JGPIND,JFACE) = TEVAL
              PGVALF(JGPIND,JFACE) = PEVAL
              DO JSPEC = 1, NSPEC
                YGVALF(JSPEC,JGPIND,JFACE) = YEVAL(JSPEC)
              ENDDO
              CPVALF(JGPIND,JFACE) = CPEVAL
              RGVALF(JGPIND,JFACE) = RCEVAL
  
C             CELL FACE VALUES FOR MOLECULAR TRANSPORT
              DMVALF(JGPIND,JFACE) = DEVAL
              TMVALF(JGPIND,JFACE) = TEVAL
              PMVALF(JGPIND,JFACE) = PEVAL
              DO JSPEC = 1, NSPEC
                YMVALF(JSPEC,JGPIND,JFACE) = YEVAL(JSPEC)
              ENDDO

C             CELL FACE DERIVATIVES

C             EVALUATE THE POLYNOMIAL
              IF(IDIRN.EQ.1)
     +          CALL CDVALX(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
              IF(IDIRN.EQ.2)
     +          CALL CDVALY(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
              IF(IDIRN.EQ.3)
     +          CALL CDVALZ(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)

C             DERIVATIVES OF PRIMITIVE VARIABLES
              CALL CDPRIM(DEVAL,EEVAL,UEVAL,TEVAL,YEVAL,CVEVAL,
     +                    DDVAL,EDVAL,UDVAL,TDVAL,YDVAL,IDIRN)

C             VISCOUS STRESS TENSOR
              IDRP1 = MNVIST(1,IDIRN)
              IDRP2 = MNVIST(2,IDIRN)
              DIVUCF = UDVAL(1,1) + UDVAL(2,2) + UDVAL(3,3)
              VISTNL = TWO*UDVAL(IDIRN,IDIRN) - TTHD*DIVUCF
              VIST1L = UDVAL(IDRP1,IDIRN) + UDVAL(IDIRN,IDRP1)
              VIST2L = UDVAL(IDRP2,IDIRN) + UDVAL(IDIRN,IDRP2)
              UGRADF(1,JGPIND,JFACE) = VISTNL*OCSIZE
              UGRADF(2,JGPIND,JFACE) = VIST1L*OCSIZE
              UGRADF(3,JGPIND,JFACE) = VIST2L*OCSIZE

C             DIFFUSIVE TERMS
              TGRADF(JGPIND,JFACE) = TDVAL*OCSIZE
              DO JSPEC = 1, NSPEC
                YGRADF(JSPEC,JGPIND,JFACE) = YDVAL(JSPEC)*OCSIZE
              ENDDO
 
            ENDDO
C           END OF RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION

          ENDDO
C         END OF RUN THROUGH ALL SUBFACES

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(JFACE).GE.NHTO2H)THEN

C         ---------------------------------------------------------------------

C         RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION
          DO JGPFAC = 1, NGPFAC

C           LOCATION OF GAUSS POINT
            XLOC = GAUSFX(JGPFAC,JFACE)
            YLOC = GAUSFY(JGPFAC,JFACE)
            ZLOC = GAUSFZ(JGPFAC,JFACE)

C           CELL FACE VALUES

C           EVALUATE THE POLYNOMIAL
            CALL CEVALL(XLOC,YLOC,ZLOC,DEVAL,UEVAL,EEVAL,YEVAL)

C           TEMPERATURE AND PRIMITIVE VARIABLES
            CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C           MIXTURE SPECIFIC HEAT CP AND MIXTURE GAS CONSTANT
            CALL CPMVAL(TEVAL,YEVAL,CPEVAL,RCEVAL)
            CVEVAL = CPEVAL - RCEVAL

C           PRESSURE
            PEVAL = DEVAL*RCEVAL*TEVAL - PREFER

C           STORE THE VARIABLES
            DGVALF(JGPFAC,JFACE) = DEVAL
            UGVALF(1,JGPFAC,JFACE) = UEVAL(1)
            UGVALF(2,JGPFAC,JFACE) = UEVAL(2)
            UGVALF(3,JGPFAC,JFACE) = UEVAL(3)
            TGVALF(JGPFAC,JFACE) = TEVAL
            PGVALF(JGPFAC,JFACE) = PEVAL
            DO JSPEC = 1, NSPEC
              YGVALF(JSPEC,JGPFAC,JFACE) = YEVAL(JSPEC)
            ENDDO
            CPVALF(JGPFAC,JFACE) = CPEVAL
            RGVALF(JGPFAC,JFACE) = RCEVAL
  
C           CELL FACE VALUES FOR MOLECULAR TRANSPORT
            DMVALF(JGPFAC,JFACE) = DEVAL
            TMVALF(JGPFAC,JFACE) = TEVAL
            PMVALF(JGPFAC,JFACE) = PEVAL
            DO JSPEC = 1, NSPEC
              YMVALF(JSPEC,JGPFAC,JFACE) = YEVAL(JSPEC)
            ENDDO

C           CELL FACE DERIVATIVES

C           EVALUATE THE POLYNOMIAL
            IF(IDIRN.EQ.1)
     +        CALL CDVALX(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
            IF(IDIRN.EQ.2)
     +        CALL CDVALY(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
            IF(IDIRN.EQ.3)
     +        CALL CDVALZ(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)

C           DERIVATIVES OF PRIMITIVE VARIABLES
            CALL CDPRIM(DEVAL,EEVAL,UEVAL,TEVAL,YEVAL,CVEVAL,
     +                  DDVAL,EDVAL,UDVAL,TDVAL,YDVAL,IDIRN)

C           VISCOUS STRESS TENSOR
            IDRP1 = MNVIST(1,IDIRN)
            IDRP2 = MNVIST(2,IDIRN)
            DIVUCF = UDVAL(1,1) + UDVAL(2,2) + UDVAL(3,3)
            VISTNL = TWO*UDVAL(IDIRN,IDIRN) - TTHD*DIVUCF
            VIST1L = UDVAL(IDRP1,IDIRN) + UDVAL(IDIRN,IDRP1)
            VIST2L = UDVAL(IDRP2,IDIRN) + UDVAL(IDIRN,IDRP2)
            UGRADF(1,JGPFAC,JFACE) = VISTNL*OCSIZE
            UGRADF(2,JGPFAC,JFACE) = VIST1L*OCSIZE
            UGRADF(3,JGPFAC,JFACE) = VIST2L*OCSIZE

C           DIFFUSIVE TERMS
            TGRADF(JGPFAC,JFACE) = TDVAL*OCSIZE
            DO JSPEC = 1, NSPEC
              YGRADF(JSPEC,JGPFAC,JFACE) = YDVAL(JSPEC)*OCSIZE
            ENDDO
 
          ENDDO
C         END OF RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(JFACE).GE.NBCOND)THEN

C         ---------------------------------------------------------------------

C         RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION
          DO JGPFAC = 1, NGPFAC

C           LOCATION OF GAUSS POINT
            XLOC = GAUSFX(JGPFAC,JFACE)
            YLOC = GAUSFY(JGPFAC,JFACE)
            ZLOC = GAUSFZ(JGPFAC,JFACE)

C           CELL FACE VALUES

C           EVALUATE THE POLYNOMIAL
            CALL CEVALL(XLOC,YLOC,ZLOC,DEVAL,UEVAL,EEVAL,YEVAL)

C           TEMPERATURE AND PRIMITIVE VARIABLES
            CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C           MIXTURE SPECIFIC HEAT CP AND MIXTURE GAS CONSTANT
            CALL CPMVAL(TEVAL,YEVAL,CPEVAL,RCEVAL)
            CVEVAL = CPEVAL - RCEVAL

C           PRESSURE
            PEVAL = DEVAL*RCEVAL*TEVAL - PREFER

C           STORE THE VARIABLES
            DGVALF(JGPFAC,JFACE) = DEVAL
            UGVALF(1,JGPFAC,JFACE) = UEVAL(1)
            UGVALF(2,JGPFAC,JFACE) = UEVAL(2)
            UGVALF(3,JGPFAC,JFACE) = UEVAL(3)
            EGVALF(JGPFAC,JFACE) = EEVAL
            TGVALF(JGPFAC,JFACE) = TEVAL
            PGVALF(JGPFAC,JFACE) = PEVAL
            DO JSPEC = 1, NSPEC
              YGVALF(JSPEC,JGPFAC,JFACE) = YEVAL(JSPEC)
            ENDDO
            CPVALF(JGPFAC,JFACE) = CPEVAL
            RGVALF(JGPFAC,JFACE) = RCEVAL
  
C           CELL FACE VALUES FOR MOLECULAR TRANSPORT
            DMVALF(JGPFAC,JFACE) = DEVAL
            TMVALF(JGPFAC,JFACE) = TEVAL
            PMVALF(JGPFAC,JFACE) = PEVAL
            DO JSPEC = 1, NSPEC
              YMVALF(JSPEC,JGPFAC,JFACE) = YEVAL(JSPEC)
            ENDDO

C           CELL FACE DERIVATIVES

C           EVALUATE THE POLYNOMIAL
            IF(IDIRN.EQ.1)
     +        CALL CDVALX(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
            IF(IDIRN.EQ.2)
     +        CALL CDVALY(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)
            IF(IDIRN.EQ.3)
     +        CALL CDVALZ(XLOC,YLOC,ZLOC,DDVAL,UDVAL,EDVAL,YDVAL)

C           DERIVATIVES OF PRIMITIVE VARIABLES
            CALL CDPRIM(DEVAL,EEVAL,UEVAL,TEVAL,YEVAL,CVEVAL,
     +                  DDVAL,EDVAL,UDVAL,TDVAL,YDVAL,IDIRN)

C           DERIVATIVES FOR NSCBC
            IDRP1 = MNVIST(1,IDIRN)
            IDRP2 = MNVIST(2,IDIRN)
            DPVAL = DDVAL(IDIRN)
            CALL CDPRES(DEVAL,TEVAL,RCEVAL,
     +                  DPVAL,TDVAL,YDVAL,PDVAL)
            DGRADB(JGPFAC,JFACE) = DPVAL*OCSIZE
            UGRADB(IDIRN,JGPFAC,JFACE) = UDVAL(IDIRN,IDIRN)*OCSIZE
            UGRADB(IDRP1,JGPFAC,JFACE) = UDVAL(IDRP1,IDIRN)*OCSIZE
            UGRADB(IDRP2,JGPFAC,JFACE) = UDVAL(IDRP2,IDIRN)*OCSIZE
            PGRADB(JGPFAC,JFACE) = PDVAL*OCSIZE

C           VISCOUS STRESS TENSOR
            IDRP1 = MNVIST(1,IDIRN)
            IDRP2 = MNVIST(2,IDIRN)
            DIVUCF = UDVAL(1,1) + UDVAL(2,2) + UDVAL(3,3)
            VISTNL = TWO*UDVAL(IDIRN,IDIRN) - TTHD*DIVUCF
            VIST1L = UDVAL(IDRP1,IDIRN) + UDVAL(IDIRN,IDRP1)
            VIST2L = UDVAL(IDRP2,IDIRN) + UDVAL(IDIRN,IDRP2)
            UGRADF(1,JGPFAC,JFACE) = VISTNL*OCSIZE
            UGRADF(2,JGPFAC,JFACE) = VIST1L*OCSIZE
            UGRADF(3,JGPFAC,JFACE) = VIST2L*OCSIZE

C           DIFFUSIVE TERMS
            TGRADF(JGPFAC,JFACE) = TDVAL*OCSIZE
            DO JSPEC = 1, NSPEC
              YGRADF(JSPEC,JGPFAC,JFACE) = YDVAL(JSPEC)*OCSIZE
            ENDDO
 
          ENDDO
C         END OF RUN THROUGH THE GAUSS POINTS FOR FACE INTEGRATION

C         ---------------------------------------------------------------------

        ELSE

C         ---------------------------------------------------------------------

C         ERROR CONDITION - invalid cell face code
          CALL ERHAND(11,21,4)

C         ---------------------------------------------------------------------

        ENDIF

C       -----------------------------------------------------------------------

      ENDDO
C     END OF RUN THROUGH ALL FACES

C     =========================================================================


      RETURN
      END
