      SUBROUTINE CRERES
 
C     *************************************************************************
C
C     CRERES
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-NOV-2018:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     POLYNOMIAL RECONSTRUCTION IN EACH CELL
C     FOR REFINEMENT/DEREFINEMENT CRITERIA
C     USING STORED MOORE-PENROSE PSEUDO-INVERSE
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION TOLSVD
      PARAMETER(TOLSVD = 1.0D-5)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION BVECTR(NCSTMX)
      DOUBLE PRECISION DFCENT,UFCENT,EFCENT,YFCENT
      DOUBLE PRECISION TFCENT,PFCENT
      INTEGER NJSTEN,NKPOLY
      INTEGER JSCELL,JCSTEN,JKPOLY,JCOMP,JSPEC
      INTEGER INVNDX

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In CRESTR',ICELL

C     =========================================================================

C     PRELIMINARIES
C     -------------
      INVNDX = NREPOL(ICELL)
      NJSTEN = NOSSTN(ICELL)
      NKPOLY = NPCAC4

C     -------------------------------------------------------------------------

C     EVALUATE THE POLYNOMIAL COEFFICIENTS
C     ------------------------------------

C     DENSITY
C     -------
      DFCENT = DSOR(ICELL)
      DO JCSTEN = 1, NJSTEN
        JSCELL = NSSTEN(JCSTEN,ICELL)
        BVECTR(JCSTEN) = DSOR(JSCELL) - DFCENT
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCOEFD(JKPOLY) = ZERO
        DO JCSTEN = 1, NJSTEN
          PCOEFD(JKPOLY) = PCOEFD(JKPOLY)
     +      + AMPINV(JCSTEN,JKPOLY,INVNDX)*BVECTR(JCSTEN)
        ENDDO
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCODFD(JKPOLY) = PCOEFD(JKPOLY)
      ENDDO
      PCOEFD(0) = DFCENT - TWLFTH*(PCOEFD(2) + PCOEFD(6) + PCOEFD(13))
      PCODFD(0) = PCOEFD(0)

C     -------------------------------------------------------------------------

C     VELOCITY
C     --------
      DO JCOMP = 1, NCOMP

        UFCENT = USOR(JCOMP,ICELL)
        DO JCSTEN = 1, NJSTEN
          JSCELL = NSSTEN(JCSTEN,ICELL)
          BVECTR(JCSTEN) = USOR(JCOMP,JSCELL) - UFCENT
        ENDDO

        DO JKPOLY = 1, NKPOLY
          PCOEFU(JKPOLY,JCOMP) = ZERO
          DO JCSTEN = 1, NJSTEN
            PCOEFU(JKPOLY,JCOMP) = PCOEFU(JKPOLY,JCOMP)
     +        + AMPINV(JCSTEN,JKPOLY,INVNDX)*BVECTR(JCSTEN)
          ENDDO
        ENDDO

        DO JKPOLY = 1, NKPOLY
          PCODFU(JKPOLY,JCOMP) = PCOEFU(JKPOLY,JCOMP)
        ENDDO
        PCOEFU(0,JCOMP) = UFCENT
     +   - TWLFTH*(PCOEFU(2,JCOMP) + PCOEFU(6,JCOMP) + PCOEFU(13,JCOMP))
        PCODFU(0,JCOMP) = PCOEFU(0,JCOMP)

      ENDDO

C     -------------------------------------------------------------------------

C     INTERNAL ENERGY
C     ---------------
      EFCENT = ESOR(ICELL)
      DO JCSTEN = 1, NJSTEN
        JSCELL = NSSTEN(JCSTEN,ICELL)
        BVECTR(JCSTEN) = ESOR(JSCELL) - EFCENT
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCOEFE(JKPOLY) = ZERO
        DO JCSTEN = 1, NJSTEN
          PCOEFE(JKPOLY) = PCOEFE(JKPOLY)
     +      + AMPINV(JCSTEN,JKPOLY,INVNDX)*BVECTR(JCSTEN)
        ENDDO
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCODFE(JKPOLY) = PCOEFE(JKPOLY)
      ENDDO
      PCOEFE(0) = EFCENT - TWLFTH*(PCOEFE(2) + PCOEFE(6) + PCOEFE(13))
      PCODFE(0) = PCOEFE(0)

C     -------------------------------------------------------------------------

C     TEMPERATURE
C     -----------
      TFCENT = TRUN(ICELL)
      DO JCSTEN = 1, NJSTEN
        JSCELL = NSSTEN(JCSTEN,ICELL)
        BVECTR(JCSTEN) = TRUN(JSCELL) - TFCENT
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCOEFT(JKPOLY) = ZERO
        DO JCSTEN = 1, NJSTEN
          PCOEFT(JKPOLY) = PCOEFT(JKPOLY)
     +      + AMPINV(JCSTEN,JKPOLY,INVNDX)*BVECTR(JCSTEN)
        ENDDO
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCODFT(JKPOLY) = PCOEFT(JKPOLY)
      ENDDO
      PCOEFT(0) = TFCENT - TWLFTH*(PCOEFT(2) + PCOEFT(6) + PCOEFT(13))
      PCODFT(0) = PCOEFT(0)

C     -------------------------------------------------------------------------

C     PRESSURE
C     --------
      PFCENT = PRUN(ICELL)
      DO JCSTEN = 1, NJSTEN
        JSCELL = NSSTEN(JCSTEN,ICELL)
        BVECTR(JCSTEN) = PRUN(JSCELL) - PFCENT
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCOEFP(JKPOLY) = ZERO
        DO JCSTEN = 1, NJSTEN
          PCOEFP(JKPOLY) = PCOEFP(JKPOLY)
     +      + AMPINV(JCSTEN,JKPOLY,INVNDX)*BVECTR(JCSTEN)
        ENDDO
      ENDDO

      DO JKPOLY = 1, NKPOLY
        PCODFP(JKPOLY) = PCOEFP(JKPOLY)
      ENDDO
      PCOEFP(0) = PFCENT - TWLFTH*(PCOEFP(2) + PCOEFP(6) + PCOEFP(13))
      PCODFP(0) = PCOEFP(0)

C     -------------------------------------------------------------------------

C     SPECIES MASS FRACTION
C     ---------------------
      DO JSPEC = 1, NSPEC

        YFCENT = YSOR(JSPEC,ICELL)
        DO JCSTEN = 1, NJSTEN
          JSCELL = NSSTEN(JCSTEN,ICELL)
          BVECTR(JCSTEN) = YSOR(JSPEC,JSCELL) - YFCENT
        ENDDO

        DO JKPOLY = 1, NKPOLY
          PCOEFY(JKPOLY,JSPEC) = ZERO
          DO JCSTEN = 1, NJSTEN
            PCOEFY(JKPOLY,JSPEC) = PCOEFY(JKPOLY,JSPEC)
     +        + AMPINV(JCSTEN,JKPOLY,INVNDX)*BVECTR(JCSTEN)
          ENDDO
        ENDDO

        DO JKPOLY = 1, NKPOLY
          PCODFY(JKPOLY,JSPEC) = PCOEFY(JKPOLY,JSPEC)
        ENDDO
        PCOEFY(0,JSPEC) = YFCENT
     +   - TWLFTH*(PCOEFY(2,JSPEC) + PCOEFY(6,JSPEC) + PCOEFY(13,JSPEC))
        PCODFY(0,JSPEC) = PCOEFY(0,JSPEC)

      ENDDO

C     =========================================================================


      RETURN
      END
