      SUBROUTINE FLAMIN 
 
C     *************************************************************************
C
C     FLAMIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     28-DEC-2003:  CREATED FOR SENGA2
C     31-JUL-2010:  ADOPTED FOR HAMISH
C 
C     DESCRIPTION
C     -----------
C     SETS INITIAL THERMOCHEMICAL FIELD
C     3D GAUSSIAN ACOUSTIC PULSE
C     
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     PRESSURE PULSE AMPLITUDE
      DOUBLE PRECISION PRPEAK
      PARAMETER(PRPEAK = 1.0D2)

C     PRESSURE PULSE LOCATION AND THICKNESS
      DOUBLE PRECISION XLOCAT,XTHICK,YLOCAT,YTHICK,ZLOCAT,ZTHICK
      PARAMETER(XLOCAT = 1.6D-3, XTHICK = 5.0D-4)
      PARAMETER(YLOCAT = 1.6D-3, YTHICK = 5.0D-4)
      PARAMETER(ZLOCAT = 1.6D-3, ZTHICK = 5.0D-4)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION XCOORD,YCOORD,ZCOORD,ARGMNT,FORNOW
      DOUBLE PRECISION CPLOCL,CVLOCL,RGLOCL,GAMRAT
      INTEGER MCRUN(NINDMX)
      INTEGER MLEVEL,MX,MY,MZ
      INTEGER JCELL,JSPEC
      INTEGER ITINT,ICP
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 0
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In EHRATE',IPROC

C     =========================================================================

C     SPECIFY INITIAL THERMOCHEMICAL FIELD HERE
C     =========================================


C     SET PRESSURE PROFILE
C     --------------------
      DO JCELL = 1, NCELL

C       LOCAL CELL-CENTRE COORDINATES
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
        MLEVEL = NLEVEL(JCELL)
        CALL MCCXYZ(MCRUN,MLEVEL,MX,MY,MZ)
        XCOORD = (REAL(MX) + HALF)*HCSIZE(MLEVEL)
        YCOORD = (REAL(MY) + HALF)*HCSIZE(MLEVEL)
        ZCOORD = (REAL(MZ) + HALF)*HCSIZE(MLEVEL)

C       GAUSSIAN FOR PRESSURE
        ARGMNT = ZERO
        FORNOW = (XCOORD-XLOCAT)/XTHICK
        ARGMNT = ARGMNT + FORNOW*FORNOW
C        FORNOW = (YCOORD-YLOCAT)/YTHICK
C        ARGMNT = ARGMNT + FORNOW*FORNOW
C        FORNOW = (ZCOORD-ZLOCAT)/ZTHICK
C        ARGMNT = ARGMNT + FORNOW*FORNOW
        PRUN(JCELL) = PRIN + PRPEAK*EXP(-ARGMNT)

      ENDDO


C     SET TEMPERATURE PROFILE ASSUMING CONSTANT ENTROPY
C     -------------------------------------------------
      CPLOCL = ZERO
      RGLOCL = ZERO
      DO JSPEC = 1,NSPEC

C       TEMPERATURE INTERVAL INDEX
        ITINT = 1
1010    CONTINUE
          IF(TRIN.GT.TINTHI(ITINT,JSPEC))THEN
            IF(ITINT.LT.NTINT(JSPEC))THEN
              ITINT = ITINT + 1
              GOTO 1010
            ENDIF
          ENDIF
C       END OF LOOP 1010

C       SPECIFIC HEAT CP
        FORNOW = AMASCP(NCPOLY(ITINT,JSPEC),ITINT,JSPEC)
        DO ICP = NCPOM1(ITINT,JSPEC),1,-1
          FORNOW = FORNOW*TRIN + AMASCP(ICP,ITINT,JSPEC)
        ENDDO
        CPLOCL = CPLOCL + FORNOW*YRIN(JSPEC)
 
C       MIXTURE GAS CONSTANT
        RGLOCL = RGLOCL + RGSPEC(JSPEC)*YRIN(JSPEC)

      ENDDO

C     SPECIFIC HEAT CP
      CVLOCL = CPLOCL - RGLOCL

C     GAMMA = CP/CV
      GAMRAT = CPLOCL/CVLOCL
      GAMRAT = (GAMRAT-ONE)/GAMRAT

C     SET TEMPERATURE
      DO JCELL = 1,NCELL

        TRUN(JCELL) = TRIN*EXP(GAMRAT*LOG(PRUN(JCELL)/PRIN))

      ENDDO

C     =========================================================================


      RETURN
      END
