      SUBROUTINE GLODOM
 
C     *************************************************************************
C
C     GLODOM
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     05-OCT-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES THE DOMAIN PARAMETERS ON RESTART
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION CELTOL
      PARAMETER(CELTOL = 1.0D-10)


C     LOCAL DATA
C     ==========
      INTEGER MCRUN(NINDMX)
      INTEGER MX,MY,MZ,MLEVEL
      INTEGER NLVLOC
      INTEGER JCELL,JLEVEL


C     =========================================================================

C     VGRAD1 = TEMPORARY STORAGE FOR CELL-CENTRE COORDINATES (X,Y,Z)

C     =========================================================================

C     BEGIN
C     =====

C     =========================================================================

C     MAX LEVEL IN OCTREE
C     -------------------
      NLEVL = NLEVRQ

C     =========================================================================

C     NUMBER OF DIMENSIONS
C     --------------------
      NDIMS = NDIMAX
      IF(ZGDLEN.LT.CELTOL)THEN
        NDIMS = 2
        IF(YGDLEN.LT.CELTOL)THEN
          NDIMS = 1
        ENDIF
      ENDIF
      NOFSP1 = 2**NDIMS
      NOFSET = NOFSP1 - 1
      DREFAV = ONE/REAL(NOFSP1)

C     =========================================================================

C     NUMBER OF COMPONENTS AND FACES
C     ------------------------------
      NFACE = NFACMX
      NCOMP = NCMPMX

C     =========================================================================

C     NUMBER OF SUB-FACES
C     -------------------
      NFSUB = 2**(NDIMS-1)

C     =========================================================================

C     NUMBER OF SECTORS
C     -----------------
      NSCTR = NFACMX

C     =========================================================================

C     BASE MESH
C     ---------

C     COMPUTE THE CELL SIZE AT EACH LEVEL
      OVSIZE(0) = ONE/HCSIZE(0)
      DO JLEVEL = 1,NLEVL
        HCSIZE(JLEVEL) = HALF*HCSIZE(JLEVEL-1)
        OVSIZE(JLEVEL) = ONE/HCSIZE(JLEVEL)
      ENDDO

C     PHYSICAL LOWER BOUNDS OF GLOBAL COMPUTATIONAL DOMAIN
      XGPHYL = ZERO
      YGPHYL = ZERO
      ZGPHYL = ZERO

C     MIN AND MAX CELL NUMBERS IN EACH DIRECTION
      DO JLEVEL = 1,NLEVL
        MPMIN(1,JLEVEL) = 0
        MPMAX(1,JLEVEL) = NINT(XGPHYR*OVSIZE(JLEVEL)) - 1
        MPMAX(1,JLEVEL) = MAX(MPMAX(1,JLEVEL),0)
        MPMIN(2,JLEVEL) = 0
        MPMAX(2,JLEVEL) = NINT(YGPHYR*OVSIZE(JLEVEL)) - 1
        MPMAX(2,JLEVEL) = MAX(MPMAX(2,JLEVEL),0)
        MPMIN(3,JLEVEL) = 0
        MPMAX(3,JLEVEL) = NINT(ZGPHYR*OVSIZE(JLEVEL)) - 1
        MPMAX(3,JLEVEL) = MAX(MPMAX(3,JLEVEL),0)
      ENDDO

C     =========================================================================

C     INITIALISE M-CODE MANIPULATION
C     ------------------------------
      CALL MCINIT

C     =========================================================================

C     COMPUTE M-CODES FROM CELL-CENTRE COORDINATES
C     --------------------------------------------
C     AND GET THE LOCAL MAX LEVEL
      NLVLOC = 0
      IF(NDIMS.EQ.3)THEN
        DO JCELL = 1,NCELL
          MLEVEL = NLEVEL(JCELL)
          NLVLOC = MAX(NLVLOC,MLEVEL)
          MX = NINT(XCENTR(JCELL)/HCSIZE(MLEVEL) - HALF)
          MY = NINT(YCENTR(JCELL)/HCSIZE(MLEVEL) - HALF)
          MZ = NINT(ZCENTR(JCELL)/HCSIZE(MLEVEL) - HALF)
          CALL MCXYZC(MCRUN,MLEVEL,MX,MY,MZ)
          CALL MCSTOR(MCRUN,NMCODE,NSIZMX,JCELL)
        ENDDO
      ENDIF
      IF(NDIMS.EQ.2)THEN
        DO JCELL = 1,NCELL
          MLEVEL = NLEVEL(JCELL)
          NLVLOC = MAX(NLVLOC,MLEVEL)
          MX = NINT(XCENTR(JCELL)/HCSIZE(MLEVEL) - HALF)
          MY = NINT(YCENTR(JCELL)/HCSIZE(MLEVEL) - HALF)
          MZ = 0
          CALL MCXYZC(MCRUN,MLEVEL,MX,MY,MZ)
          CALL MCSTOR(MCRUN,NMCODE,NSIZMX,JCELL)
        ENDDO
      ENDIF
      IF(NDIMS.EQ.1)THEN
        DO JCELL = 1,NCELL
          MLEVEL = NLEVEL(JCELL)
          NLVLOC = MAX(NLVLOC,MLEVEL)
          MX = NINT(XCENTR(JCELL)/HCSIZE(MLEVEL) - HALF)
          MY = 0
          MZ = 0
          CALL MCXYZC(MCRUN,MLEVEL,MX,MY,MZ)
          CALL MCSTOR(MCRUN,NMCODE,NSIZMX,JCELL)
        ENDDO
      ENDIF
      NNCODE = NCELL

C     GET THE GLOBAL MAX LEVEL
      CALL P_GMXI(NLVLOC,NLVHWM)

C     =========================================================================

C     BUILD THE PARTITION INTERVAL TABLE
      CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NNCODE)
      CALL PTABLE(MCRUN)

C     =========================================================================


      RETURN
      END
