      SUBROUTINE C2H2HF(IFACE)

C     *************************************************************************
C
C     C2H2HF
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     03-MAY-2009:  CREATED
C     11-AUG-2014:  RSC REWRITTEN FOR WENO
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES CELL FACE FLUXES FOR ALL PDEs
C     INCLUDES MULTIPLE SCALARS AND MULTI-STEP CHEMISTRY
C     2h-h TRANSITION
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER IFACE


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YEVALL(NSPCMX),YEVALR(NSPCMX)
      DOUBLE PRECISION YDVALL(NSPCMX),YDVALR(NSPCMX)
      DOUBLE PRECISION YMVALL(NSPCMX),YMVALR(NSPCMX)
      DOUBLE PRECISION YEVALF(NSPCMX),YDIFFF(NSPCMX)
      DOUBLE PRECISION YFLUXF(NSPCMX)
      DOUBLE PRECISION DIFFMH(NSPCMX)
      DOUBLE PRECISION HEVALF(NSPCMX)
      DOUBLE PRECISION VISTAU(NCMPMX,NCMPMX)
      DOUBLE PRECISION UEVALL(NCMPMX),UEVALR(NCMPMX),UEVALF(NCMPMX)
      DOUBLE PRECISION UFLUXF(NCMPMX)
      DOUBLE PRECISION DEVALL,TEVALL,PEVALL,AEVALL
      DOUBLE PRECISION DEVALR,TEVALR,PEVALR,AEVALR
      DOUBLE PRECISION DMVALL,TMVALL,PMVALL
      DOUBLE PRECISION DMVALR,TMVALR,PMVALR
      DOUBLE PRECISION TMVALH,CPVALH
      DOUBLE PRECISION DEVALF,TEVALF,PEVALF,EEVALF
      DOUBLE PRECISION CPVALL,RGVALL,GAMMAL
      DOUBLE PRECISION CPVALR,RGVALR,GAMMAR
      DOUBLE PRECISION DFLUXF,EFLUXF
      DOUBLE PRECISION TDVALL,TDVALR
      DOUBLE PRECISION TCONDF,VISTRF
      DOUBLE PRECISION VISTNL,VIST1L,VIST2L
      DOUBLE PRECISION VISTNR,VIST1R,VIST2R
      DOUBLE PRECISION VISCOH,TCONDH
      DOUBLE PRECISION YDCORR,YDCORY
      DOUBLE PRECISION OCSIZE,FFACTR,GFACTR
      INTEGER INEXT(4)
      INTEGER JNEXT
      INTEGER ILEVL,IDIRN,ISPEC
      INTEGER IEFACE,JEFACE
      INTEGER IDRP1,IDRP2
      INTEGER JGPFAC,JGPIND
      INTEGER IFSUB,JFACE

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In C2H2HF',ICELL,IFACE
C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,'(6I5)')(NXDOOR(JFACE,ICELL),JFACE=1,6)

C     =========================================================================

C     NEIGHBOUR ARRAY CONTAINS CELL INDEX FOR LOWEST-INDEXED NEIGHBOUR CELL
      INEXT(1) = NXDOOR(IFACE,ICELL)

C     GET THE OTHER THREE USING THEIR OWN CONNECTIVITY
      JFACE = MCFACE(1,1,IFACE)
      INEXT(2) = NXDOOR(JFACE,INEXT(1))
      JFACE = MCFACE(2,1,IFACE)
      INEXT(3) = NXDOOR(JFACE,INEXT(1))
      JFACE = MCFACE(2,2,IFACE)
      INEXT(4) = NXDOOR(JFACE,INEXT(2))

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,'(5I5)')(INEXT(JFACE),JFACE=1,4)

C     LEVEL, SIZE AND (SIGNED) RECIPROCAL SIZE OF LOCAL CELL
      ILEVL = NLEVEL(ICELL)
      OCSIZE = OVSIZE(ILEVL)

C     DIRECTION INDEX
      IDIRN = MDIRN(IFACE)

C     GAUSS POINT INDEXING
      JGPIND = 0

C     =========================================================================

C     RUN THROUGH ALL SUBFACES
      DO IFSUB = 1,NFSUB

C       CHECK FOR PRE-EXISTING CELL FACE VALUES
C       ---------------------------------------
C       IN THIS SITUATION ONLY (2h TO h WITH PRE-EXISTING VALUES)
C       STACK INDICES ARE STORED IN *NEXT-DOOR* CELLS
        JNEXT = INEXT(IFSUB)
        JFACE = MAPOPP(IFACE)
        IEFACE = NEFACE(JFACE,JNEXT)

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)WRITE(6,'("2H",5I6)')
     +              ICELL,IFACE,IFSUB,JNEXT,IEFACE

        IF(IEFACE.EQ.0)THEN

C         NO PRE-EXISTING CELL FACE VALUES
C         --------------------------------
C         FOR THIS SUBFACE

C         POP A FREE SLOT OFF THE FREE SLOT STACK
          IF(ISTACK.GT.1)THEN
            JEFACE = NFSTAK(ISTACK)
            ISTACK = ISTACK - 1 
          ELSE
            JEFACE = NFSTAK(1) + 1
            NFSTAK(1) = JEFACE
          ENDIF

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("NO PRE",2I6)')IFSUB,JEFACE
          ENDIF

C         STORE INDEX IN NEXT-DOOR CELL
          JFACE = MAPOPP(IFACE)
          NEFACE(JFACE,INEXT(IFSUB)) = JEFACE

C         =====================================================================

C         COMPUTE NEW CELL FACE VALUES
C         -----------------------------
C         NOTE THAT L/R DISTINCTION IS IRRELEVANT AT THIS STAGE
C         USE L FOR TEMPORARY STORAGE

C         RUN THROUGH THE GAUSS POINTS
          DO JGPFAC = 1, NGPFAC

C           -------------------------------------------------------------------

C           STORE PRECOMPUTED CELL FACE VALUES
            JGPIND = JGPIND + 1
            DEFACE(JGPFAC,JEFACE) = DGVALF(JGPIND,IFACE)
            UEFACE(1,JGPFAC,JEFACE) = UGVALF(1,JGPIND,IFACE)
            UEFACE(2,JGPFAC,JEFACE) = UGVALF(2,JGPIND,IFACE)
            UEFACE(3,JGPFAC,JEFACE) = UGVALF(3,JGPIND,IFACE)
            TEFACE(JGPFAC,JEFACE) = TGVALF(JGPIND,IFACE)
            PEFACE(JGPFAC,JEFACE) = PGVALF(JGPIND,IFACE)
            DO ISPEC = 1, NSPEC
              YEFACE(ISPEC,JGPFAC,JEFACE) = YGVALF(ISPEC,JGPIND,IFACE)
            ENDDO
            CPFACE(JGPFAC,JEFACE) = CPVALF(JGPIND,IFACE)
            RGFACE(JGPFAC,JEFACE) = RGVALF(JGPIND,IFACE)

C           DIAGNOSTICS
            GPFACE(1,JGPFAC,JEFACE) = GPLOCF(1,JGPIND,IFACE)
            GPFACE(2,JGPFAC,JEFACE) = GPLOCF(2,JGPIND,IFACE)
            GPFACE(3,JGPFAC,JEFACE) = GPLOCF(3,JGPIND,IFACE)

C           -------------------------------------------------------------------

C           STORE PRECOMPUTED CELL FACE VALUES FOR MOLECULAR TRANSPORT
            DMFACE(JGPFAC,JEFACE) = DMVALF(JGPIND,IFACE)
            TMFACE(JGPFAC,JEFACE) = TMVALF(JGPIND,IFACE)
            PMFACE(JGPFAC,JEFACE) = PMVALF(JGPIND,IFACE)
            DO ISPEC = 1, NSPEC
              YMFACE(ISPEC,JGPFAC,JEFACE) = YMVALF(ISPEC,JGPIND,IFACE)
            ENDDO

C           -------------------------------------------------------------------

C           STORE PRECOMPUTED CELL FACE GRADIENTS
            UDFACE(1,JGPFAC,JEFACE) = UGRADF(1,JGPIND,IFACE)
            UDFACE(2,JGPFAC,JEFACE) = UGRADF(2,JGPIND,IFACE)
            UDFACE(3,JGPFAC,JEFACE) = UGRADF(3,JGPIND,IFACE)
            TDFACE(JGPFAC,JEFACE) = TGRADF(JGPIND,IFACE)
            DO ISPEC = 1, NSPEC
              YDFACE(ISPEC,JGPFAC,JEFACE) = YGRADF(ISPEC,JGPIND,IFACE)
            ENDDO

C           -------------------------------------------------------------------

          ENDDO
C         END OF RUN THROUGH THE GAUSS POINTS

C         =====================================================================

        ELSE

C         =====================================================================

C         PRE-EXISTING CELL FACE VALUES
C         -----------------------------
C         FOR THIS SUBFACE

          JEFACE = IEFACE

C  `      DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,'("PRE",2I5)')IFSUB,JEFACE

C         RUN THROUGH THE GAUSS POINTS
          DO JGPFAC = 1, NGPFAC

C           -------------------------------------------------------------------

C           GET THE CELL FACE VALUES
C           ------------------------

C           RETRIEVED VALUES
C           ----------------
C           STORED IN L

C           CELL FACE VALUES
            DEVALL = DEFACE(JGPFAC,JEFACE)
            UEVALL(1) = UEFACE(1,JGPFAC,JEFACE)
            UEVALL(2) = UEFACE(2,JGPFAC,JEFACE)
            UEVALL(3) = UEFACE(3,JGPFAC,JEFACE)
            TEVALL = TEFACE(JGPFAC,JEFACE)
            PEVALL = PEFACE(JGPFAC,JEFACE)
            DO ISPEC = 1, NSPEC
              YEVALL(ISPEC) = YEFACE(ISPEC,JGPFAC,JEFACE)
            ENDDO
            CPVALL = CPFACE(JGPFAC,JEFACE)
            RGVALL = RGFACE(JGPFAC,JEFACE)

C           CELL FACE VALUES FOR MOLECULAR TRANSPORT
            DMVALL = DMFACE(JGPFAC,JEFACE)
            TMVALL = TMFACE(JGPFAC,JEFACE)
            PMVALL = PMFACE(JGPFAC,JEFACE)
            DO ISPEC = 1, NSPEC
              YMVALL(ISPEC) = YMFACE(ISPEC,JGPFAC,JEFACE)
            ENDDO

C           VISCOUS STRESS TENSOR
            VISTNL = UDFACE(1,JGPFAC,JEFACE)
            VIST1L = UDFACE(2,JGPFAC,JEFACE)
            VIST2L = UDFACE(3,JGPFAC,JEFACE)

C           CELL FACE GRADIENTS
            TDVALL = TDFACE(JGPFAC,JEFACE)
            DO ISPEC = 1, NSPEC
              YDVALL(ISPEC) = YDFACE(ISPEC,JGPFAC,JEFACE)
            ENDDO

C           -------------------------------------------------------------------

C           NEW CELL FACE VALUES
C           --------------------
C           STORED IN R

C           CELL FACE VALUES
            JGPIND = JGPIND + 1
            DEVALR = DGVALF(JGPIND,IFACE)
            UEVALR(1) = UGVALF(1,JGPIND,IFACE)
            UEVALR(2) = UGVALF(2,JGPIND,IFACE)
            UEVALR(3) = UGVALF(3,JGPIND,IFACE)
            TEVALR = TGVALF(JGPIND,IFACE)
            PEVALR = PGVALF(JGPIND,IFACE)
            DO ISPEC = 1, NSPEC
              YEVALR(ISPEC) = YGVALF(ISPEC,JGPIND,IFACE)
            ENDDO
            CPVALR = CPVALF(JGPIND,IFACE)
            RGVALR = RGVALF(JGPIND,IFACE)

C           CELL FACE VALUES FOR MOLECULAR TRANSPORT
            DMVALR = DMVALF(JGPIND,IFACE)
            TMVALR = TMVALF(JGPIND,IFACE)
            PMVALR = PMVALF(JGPIND,IFACE)
            DO ISPEC = 1, NSPEC
              YMVALR(ISPEC) = YMVALF(ISPEC,JGPIND,IFACE)
            ENDDO

C           VISCOUS STRESS TENSOR
            VISTNR = UGRADF(1,JGPIND,IFACE)
            VIST1R = UGRADF(2,JGPIND,IFACE)
            VIST2R = UGRADF(3,JGPIND,IFACE)

C           CELL FACE GRADIENTS
            TDVALR = TGRADF(JGPIND,IFACE)
            DO ISPEC = 1, NSPEC
              YDVALR(ISPEC) = YGRADF(ISPEC,JGPIND,IFACE)
            ENDDO

C           -------------------------------------------------------------------

C           SPEED OF SOUUND
            GAMMAL = CPVALL*RGVALL/(CPVALL - RGVALL)
            GAMMAR = CPVALR*RGVALR/(CPVALR - RGVALR)
            AEVALL = SQRT(GAMMAL*TEVALL)
            AEVALR = SQRT(GAMMAR*TEVALR)

C           -------------------------------------------------------------------

C           CONVECTIVE TERMS
C           RIEMANN PROBLEM AT EACH GAUSS POINT

C           CHECK L/R BASED ON COORDINATE DIRECTION AT THE FACE
            IF(MSIGN(IFACE).GT.0)THEN

C             L AND R ARE CORRECT
              CALL RIEMAN(DEVALL,UEVALL,PEVALL,YEVALL,AEVALL,
     +                    DEVALR,UEVALR,PEVALR,YEVALR,AEVALR,
     +                    DEVALF,UEVALF,PEVALF,YEVALF,IDIRN)

            ELSE

C             SWAP L AND R
              CALL RIEMAN(DEVALR,UEVALR,PEVALR,YEVALR,AEVALR,
     +                    DEVALL,UEVALL,PEVALL,YEVALL,AEVALL,
     +                    DEVALF,UEVALF,PEVALF,YEVALF,IDIRN)

CC             DIAGNOSTICS
C              WRITE(6,*)"2hRL",IPROC,ITIME,IRKSTP,ICELL,IFACE

            ENDIF

C           -------------------------------------------------------------------

C           TEMPERATURE FROM THERMAL EOS
            CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C           SPECIES ENTHALPIES AND MIXTURE INTERNAL ENERGY FROM CALORIC EOS
            CALL ENTHPY(DEVALF,TEVALF,HEVALF)

C           -------------------------------------------------------------------

C           MIXTURE INTERNAL ENERGY
            EEVALF = UEVALF(1)*UEVALF(1)
     +             + UEVALF(2)*UEVALF(2)
     +             + UEVALF(3)*UEVALF(3)
            EEVALF = HALF*EEVALF
            DO ISPEC = 1, NSPEC
              EEVALF = EEVALF + YEVALF(ISPEC)*HEVALF(ISPEC)
            ENDDO

C           -------------------------------------------------------------------

C           MOLECULAR TRANSPORT COEFFICIENTS
            TMVALH = HALF*(TMVALL + TMVALR)
            CPVALH = HALF*(CPVALL + CPVALR)
            CALL TRANCO(TMVALH,CPVALH,VISCOH,TCONDH,DIFFMH)

C           -------------------------------------------------------------------

C           VISCOUS TERMS
            IDRP1 = MNVIST(1,IDIRN)
            IDRP2 = MNVIST(2,IDIRN)
            VISTAU(IDIRN,IDIRN) = HALF*VISCOH*(VISTNL + VISTNR)
            VISTAU(IDRP1,IDIRN) = HALF*VISCOH*(VIST1L + VIST1R)
            VISTAU(IDRP2,IDIRN) = HALF*VISCOH*(VIST2L + VIST2R)
            VISTRF = UEVALF(1)*VISTAU(1,IDIRN)
     +             + UEVALF(2)*VISTAU(2,IDIRN)
     +             + UEVALF(3)*VISTAU(3,IDIRN)

C           THERMAL CONDUCTIVITY TERMS
            TCONDF = HALF*TCONDH*(TDVALL + TDVALR)

C           MASS DIFFUSIVITY TERMS
            DO ISPEC = 1, NSPEC
              YDIFFF(ISPEC) = HALF*DIFFMH(ISPEC)*(YDVALL(ISPEC)
     +                                          + YDVALR(ISPEC))
            ENDDO

C           -------------------------------------------------------------------

C           FLUXES
C           ------
C           MASS
            DFLUXF = DEVALF*UEVALF(IDIRN)

C           MOMENTUM
            UFLUXF(1) = DFLUXF*UEVALF(1) + PEVALF*RDELTA(1,IDIRN)
     +                - VISTAU(1,IDIRN)
            UFLUXF(2) = DFLUXF*UEVALF(2) + PEVALF*RDELTA(2,IDIRN)
     +                - VISTAU(2,IDIRN)
            UFLUXF(3) = DFLUXF*UEVALF(3) + PEVALF*RDELTA(3,IDIRN)
     +                - VISTAU(3,IDIRN)

C           ENERGY
            EFLUXF = DFLUXF*EEVALF - TCONDF - VISTRF

C           SPECIES MASS FRACTION
C           ALSO COMPUTE THE DIFFUSIVE CORRECTION
            YDCORR = ZERO
            DO ISPEC = 1, NSPEC
              YFLUXF(ISPEC) = DFLUXF*YEVALF(ISPEC) - YDIFFF(ISPEC)
              YDCORR = YDCORR + YDIFFF(ISPEC)
            ENDDO

C           SPECIES MASS FRACTION
C           APPLY DIFFUSIVE CORRECTION
C           ENERGY EQUATION
C           SPECIES ENTHALPY TRANSPORT
            DO ISPEC = 1, NSPEC
              YDCORY = YDCORR*YEVALF(ISPEC)
              YFLUXF(ISPEC) = YFLUXF(ISPEC) + YDCORY
              EFLUXF = EFLUXF - (YDIFFF(ISPEC)-YDCORY)*HEVALF(ISPEC)
            ENDDO

C           -------------------------------------------------------------------

C           ADD FLUXES TO RHS ACCUMULATORS
C           ------------------------------

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)WRITE(6,'(I4,6(1PE15.7))')IFSUB,DFLUXF,
     +                  UFLUXF(1),UFLUXF(2)
C     +                  UFLUXF(1),UFLUXF(2),EFLUXF,YFLUXF(1),YFLUXF(2)


C           LOCAL CELL
C            GFACTR = GPFLUX*OCSIZE*RSIGN(IFACE)
            GFACTR = GPFLUX*RSIGN(IFACE)
            FFACTR = GPFLUX*GFACTR
            DRHS(ICELL) = DRHS(ICELL) + FFACTR*DFLUXF
            URHS(1,ICELL) = URHS(1,ICELL) + FFACTR*UFLUXF(1)
            URHS(2,ICELL) = URHS(2,ICELL) + FFACTR*UFLUXF(2)
            URHS(3,ICELL) = URHS(3,ICELL) + FFACTR*UFLUXF(3)
            ERHS(ICELL) = ERHS(ICELL) + FFACTR*EFLUXF
            DO ISPEC = 1, NSPEC
              YRHS(ISPEC,ICELL) = YRHS(ISPEC,ICELL)
     +                          + FFACTR*YFLUXF(ISPEC)
            ENDDO

C           NEIGHBOUR
C            FFACTR = TWO*GFACTR
            FFACTR = GFACTR
            DRHS(JNEXT) = DRHS(JNEXT) - FFACTR*DFLUXF
            URHS(1,JNEXT) = URHS(1,JNEXT) - FFACTR*UFLUXF(1)
            URHS(2,JNEXT) = URHS(2,JNEXT) - FFACTR*UFLUXF(2)
            URHS(3,JNEXT) = URHS(3,JNEXT) - FFACTR*UFLUXF(3)
            ERHS(JNEXT) = ERHS(JNEXT) - FFACTR*EFLUXF
            DO ISPEC = 1, NSPEC
              YRHS(ISPEC,JNEXT) = YRHS(ISPEC,JNEXT)
     +                          - FFACTR*YFLUXF(ISPEC)
            ENDDO

C           -------------------------------------------------------------------

          ENDDO
C         END OF RUN THROUGH THE GAUSS POINTS

C         ---------------------------------------------------------------------

C         PUSH A FREE SLOT ONTO THE FREE SLOT STACK
          IF(JEFACE.LT.NFSTAK(1))THEN
            ISTACK = ISTACK + 1
            NFSTAK(ISTACK) = JEFACE
          ELSE
            NFSTAK(1) = NFSTAK(1) - 1
          ENDIF

C         FREE THE SLOT IN *NEXT-DOOR* CELL
          JFACE = MAPOPP(IFACE)
          NEFACE(JFACE,JNEXT) = 0
                
C         =====================================================================

        ENDIF
C       END OF CHECK FOR EXISTING CELL FACE VALUES

C       =======================================================================

      ENDDO
C     END OF RUN THROUGH ALL SUBFACES

C     =========================================================================

C     FREE THE SLOT IN *LOCAL* CELL
      NEFACE(IFACE,ICELL) = 0
 
C     =========================================================================


      RETURN
      END
