      SUBROUTINE CDPRIM(DEVAL,EEVAL,UEVAL,TEVAL,YEVAL,CVEVAL,
     +                  DDVAL,EDVAL,UDVAL,TDVAL,YDVAL,IDIRN)
 
C     *************************************************************************
C
C     CDPRIM
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-MAR-2017:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES DERIVATIVES OF PRIMITIVE VARIABLES
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      DOUBLE PRECISION YEVAL(NSPCMX),YDVAL(NSPCMX)
      DOUBLE PRECISION UDVAL(NCMPMX,NCMPMX)
      DOUBLE PRECISION UEVAL(NCMPMX),DDVAL(NCMPMX)
      DOUBLE PRECISION DEVAL,EEVAL,TEVAL,CVEVAL
      DOUBLE PRECISION TDVAL,EDVAL
      INTEGER IDIRN


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION HEVAL(NSPCMX)
      DOUBLE PRECISION OVDENS,PKTERM,YGTERM,ROTERM,TDENOM
      INTEGER ISPEC

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     =========================================================================

C     SPECIES GRADIENT TERM FOR TEMPERATURE
      CALL ENTHPY(DEVAL,TEVAL,HEVAL)
      YGTERM = ZERO
      DO ISPEC = 1, NSPEC
        YGTERM = YGTERM
     +         + (HEVAL(ISPEC) - RGSPEC(ISPEC)*TEVAL)*YDVAL(ISPEC)
      ENDDO
      
C     KE TERM FOR TEMPERATURE
      PKTERM = UEVAL(1)*UDVAL(1,IDIRN)
     +       + UEVAL(2)*UDVAL(2,IDIRN)
     +       + UEVAL(3)*UDVAL(3,IDIRN)

C     DENSITY TERM FOR TEMPERATURE
      ROTERM = HALF*(UEVAL(1)*UEVAL(1)
     +             + UEVAL(2)*UEVAL(2)
     +             + UEVAL(3)*UEVAL(3))*DDVAL(IDIRN)

C     DENOMINATOR FOR TEMPERATURE
      TDENOM = DEVAL*CVEVAL

C     TEMPERATURE
      TDVAL  = (EDVAL - YGTERM - PKTERM + ROTERM)/TDENOM

C     VELOCITY
      OVDENS = ONE/DEVAL
      UDVAL(1,1) = (UDVAL(1,1) - UEVAL(1)*DDVAL(1))*OVDENS
      UDVAL(1,2) = (UDVAL(1,2) - UEVAL(1)*DDVAL(2))*OVDENS
      UDVAL(1,3) = (UDVAL(1,3) - UEVAL(1)*DDVAL(3))*OVDENS
      UDVAL(2,1) = (UDVAL(2,1) - UEVAL(2)*DDVAL(1))*OVDENS
      UDVAL(2,2) = (UDVAL(2,2) - UEVAL(2)*DDVAL(2))*OVDENS
      UDVAL(2,3) = (UDVAL(2,3) - UEVAL(2)*DDVAL(3))*OVDENS
      UDVAL(3,1) = (UDVAL(3,1) - UEVAL(3)*DDVAL(1))*OVDENS
      UDVAL(3,2) = (UDVAL(3,2) - UEVAL(3)*DDVAL(2))*OVDENS
      UDVAL(3,3) = (UDVAL(3,3) - UEVAL(3)*DDVAL(3))*OVDENS

C     SPECIES MASS FRACTION
      DO ISPEC = 1,NSPEC
        YDVAL(ISPEC) = (YDVAL(ISPEC) - YEVAL(ISPEC)*DDVAL(IDIRN))*OVDENS
      ENDDO

C     INTERNAL ENERGY
      EDVAL = (EDVAL - EEVAL*DDVAL(IDIRN))*OVDENS

C     =========================================================================


      RETURN
      END
