      SUBROUTINE CFLUXF(IFACE)

C     *************************************************************************
C
C     CFLUXF
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     03-MAY-2009:  CREATED
C     11-AUG-2014:  RSC REWRITTEN FOR WENO
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES CELL FACE FLUXES FOR ALL PDEs
C     INCLUDES MULTIPLE SCALARS AND MULTI-STEP CHEMISTRY
C     CLEAN CELL FACE
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER IFACE


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YEVALL(NSPCMX),YEVALR(NSPCMX)
      DOUBLE PRECISION YDVALL(NSPCMX),YDVALR(NSPCMX)
      DOUBLE PRECISION YMVALL(NSPCMX),YMVALR(NSPCMX)
      DOUBLE PRECISION YEVALF(NSPCMX),YDIFFF(NSPCMX)
      DOUBLE PRECISION YFLUXF(NSPCMX)
      DOUBLE PRECISION DIFFMH(NSPCMX)
      DOUBLE PRECISION HEVALF(NSPCMX)
      DOUBLE PRECISION VISTAU(NCMPMX,NCMPMX)
      DOUBLE PRECISION UEVALL(NCMPMX),UEVALR(NCMPMX),UEVALF(NCMPMX)
      DOUBLE PRECISION UFLUXF(NCMPMX)
      DOUBLE PRECISION DEVALL,TEVALL,PEVALL,AEVALL
      DOUBLE PRECISION DEVALR,TEVALR,PEVALR,AEVALR
      DOUBLE PRECISION DMVALL,TMVALL,PMVALL,EEVALL
      DOUBLE PRECISION DMVALR,TMVALR,PMVALR,EEVALR
      DOUBLE PRECISION TMVALH,CPVALH
      DOUBLE PRECISION DEVALF,TEVALF,PEVALF,EEVALF
      DOUBLE PRECISION CPVALL,RGVALL,GAMMAL
      DOUBLE PRECISION CPVALR,RGVALR,GAMMAR
      DOUBLE PRECISION DFLUXF,EFLUXF
      DOUBLE PRECISION TDVALL,TDVALR
      DOUBLE PRECISION TCONDF,VISTRF
      DOUBLE PRECISION VISTNL,VIST1L,VIST2L
      DOUBLE PRECISION VISTNR,VIST1R,VIST2R
      DOUBLE PRECISION VISCOH,TCONDH
      DOUBLE PRECISION YDCORR,YDCORY
      DOUBLE PRECISION OCSIZE,FFACTR
      INTEGER INEXT,ILEVL,IDIRN,ISPEC
      INTEGER IEFACE,JEFACE
      INTEGER JFACE
      INTEGER IDRP1,IDRP2
      INTEGER JGPFAC

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In CFLUXF'

C     =========================================================================

C     NEXT-DOOR CELL INDEX
      INEXT = NXDOOR(IFACE,ICELL)

C     LEVEL AND RECIPROCAL SIZE OF LOCAL CELL
      ILEVL = NLEVEL(ICELL)
      OCSIZE = OVSIZE(ILEVL)

C     DIRECTION INDEX
      IDIRN = MDIRN(IFACE)

C     =========================================================================

C     CHECK FOR PRE-EXISTING CELL FACE VALUES
C     ---------------------------------------
      IEFACE = NEFACE(IFACE,ICELL)

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,'("CF",4I6)')ICELL,IFACE,INEXT,IEFACE

      IF(IEFACE.EQ.0)THEN

C       NO PRE-EXISTING CELL FACE VALUES
C       --------------------------------

C       POP A FREE SLOT OFF THE FREE SLOT STACK
        IF(ISTACK.GT.1)THEN
          JEFACE = NFSTAK(ISTACK)
          ISTACK = ISTACK - 1 
        ELSE
          JEFACE = NFSTAK(1) + 1
          NFSTAK(1) = JEFACE
        ENDIF

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
CC          NCONT1 = MAX(NCONT1,JEFACE)
CC          WRITE(6,'(20X,2I9)')JEFACE,NCONT1
          WRITE(6,'("NO PRE",I6)')JEFACE
        ENDIF

C       STORE INDEX IN NEXT-DOOR CELL
        JFACE = MAPOPP(IFACE)
        NEFACE(JFACE,INEXT) = JEFACE

C       =======================================================================

C       COMPUTE NEW CELL FACE VALUES
C       -----------------------------
C       NOTE THAT L/R DISTINCTION IS IRRELEVANT AT THIS STAGE
C       USE L FOR TEMPORARY STORAGE

C       RUN THROUGH THE GAUSS POINTS
        DO JGPFAC = 1, NGPFAC

C         ---------------------------------------------------------------------

C         STORE PRECOMPUTED CELL FACE VALUES
          DEFACE(JGPFAC,JEFACE) = DGVALF(JGPFAC,IFACE)
          UEFACE(1,JGPFAC,JEFACE) = UGVALF(1,JGPFAC,IFACE)
          UEFACE(2,JGPFAC,JEFACE) = UGVALF(2,JGPFAC,IFACE)
          UEFACE(3,JGPFAC,JEFACE) = UGVALF(3,JGPFAC,IFACE)
          EEFACE(JGPFAC,JEFACE) = EGVALF(JGPFAC,IFACE)
          TEFACE(JGPFAC,JEFACE) = TGVALF(JGPFAC,IFACE)
          PEFACE(JGPFAC,JEFACE) = PGVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YEFACE(ISPEC,JGPFAC,JEFACE) = YGVALF(ISPEC,JGPFAC,IFACE)
          ENDDO
          CPFACE(JGPFAC,JEFACE) = CPVALF(JGPFAC,IFACE)
          RGFACE(JGPFAC,JEFACE) = RGVALF(JGPFAC,IFACE)

C         ---------------------------------------------------------------------

C         STORE PRECOMPUTED CELL FACE VALUES FOR MOLECULAR TRANSPORT
          DMFACE(JGPFAC,JEFACE) = DMVALF(JGPFAC,IFACE)
          TMFACE(JGPFAC,JEFACE) = TMVALF(JGPFAC,IFACE)
          PMFACE(JGPFAC,JEFACE) = PMVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YMFACE(ISPEC,JGPFAC,JEFACE) = YMVALF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         ---------------------------------------------------------------------

C         STORE PRECOMPUTED CELL FACE GRADIENTS
          UDFACE(1,JGPFAC,JEFACE) = UGRADF(1,JGPFAC,IFACE)
          UDFACE(2,JGPFAC,JEFACE) = UGRADF(2,JGPFAC,IFACE)
          UDFACE(3,JGPFAC,JEFACE) = UGRADF(3,JGPFAC,IFACE)
          TDFACE(JGPFAC,JEFACE) = TGRADF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YDFACE(ISPEC,JGPFAC,JEFACE) = YGRADF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'(3(1PE12.4))')UDFACE(1,JGPFAC,JEFACE),
     +                             UDFACE(2,JGPFAC,JEFACE),
     +                             UDFACE(3,JGPFAC,JEFACE)
          ENDIF
C         ---------------------------------------------------------------------

        ENDDO
C       END OF RUN THROUGH THE GAUSS POINTS

C       =======================================================================

      ELSE

C       =======================================================================

C       PRE-EXISTING CELL FACE VALUES
C       -----------------------------

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("PRE",I6)')IEFACE
        ENDIF

C       RUN THROUGH THE GAUSS POINTS
        DO JGPFAC = 1, NGPFAC

C         ---------------------------------------------------------------------

C         GET THE CELL FACE VALUES
C         ------------------------

C         RETRIEVED VALUES
C         ----------------
C         STORED IN L

C         CELL FACE VALUES
          DEVALL = DEFACE(JGPFAC,IEFACE)
          UEVALL(1) = UEFACE(1,JGPFAC,IEFACE)
          UEVALL(2) = UEFACE(2,JGPFAC,IEFACE)
          UEVALL(3) = UEFACE(3,JGPFAC,IEFACE)
          EEVALL = EEFACE(JGPFAC,IEFACE)
          TEVALL = TEFACE(JGPFAC,IEFACE)
          PEVALL = PEFACE(JGPFAC,IEFACE)
          DO ISPEC = 1, NSPEC
            YEVALL(ISPEC) = YEFACE(ISPEC,JGPFAC,IEFACE)
          ENDDO
          CPVALL = CPFACE(JGPFAC,IEFACE)
          RGVALL = RGFACE(JGPFAC,IEFACE)

C         CELL FACE VALUES FOR MOLECULAR TRANSPORT
          DMVALL = DMFACE(JGPFAC,IEFACE)
          TMVALL = TMFACE(JGPFAC,IEFACE)
          PMVALL = PMFACE(JGPFAC,IEFACE)
          DO ISPEC = 1, NSPEC
            YMVALL(ISPEC) = YMFACE(ISPEC,JGPFAC,IEFACE)
          ENDDO

C         VISCOUS STRESS TENSOR
          VISTNL = UDFACE(1,JGPFAC,IEFACE)
          VIST1L = UDFACE(2,JGPFAC,IEFACE)
          VIST2L = UDFACE(3,JGPFAC,IEFACE)

C         CELL FACE GRADIENTS
          TDVALL = TDFACE(JGPFAC,IEFACE)
          DO ISPEC = 1, NSPEC
            YDVALL(ISPEC) = YDFACE(ISPEC,JGPFAC,IEFACE)
          ENDDO

C         ---------------------------------------------------------------------

C         NEW CELL FACE VALUES
C         --------------------
C         STORED IN R

C         CELL FACE VALUES
          DEVALR = DGVALF(JGPFAC,IFACE)
          UEVALR(1) = UGVALF(1,JGPFAC,IFACE)
          UEVALR(2) = UGVALF(2,JGPFAC,IFACE)
          UEVALR(3) = UGVALF(3,JGPFAC,IFACE)
          EEVALR = EGVALF(JGPFAC,IFACE)
          TEVALR = TGVALF(JGPFAC,IFACE)
          PEVALR = PGVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YEVALR(ISPEC) = YGVALF(ISPEC,JGPFAC,IFACE)
          ENDDO
          CPVALR = CPVALF(JGPFAC,IFACE)
          RGVALR = RGVALF(JGPFAC,IFACE)

C         CELL FACE VALUES FOR MOLECULAR TRANSPORT
          DMVALR = DMVALF(JGPFAC,IFACE)
          TMVALR = TMVALF(JGPFAC,IFACE)
          PMVALR = PMVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YMVALR(ISPEC) = YMVALF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         VISCOUS STRESS TENSOR
          VISTNR = UGRADF(1,JGPFAC,IFACE)
          VIST1R = UGRADF(2,JGPFAC,IFACE)
          VIST2R = UGRADF(3,JGPFAC,IFACE)

C         CELL FACE GRADIENTS
          TDVALR = TGRADF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YDVALR(ISPEC) = YGRADF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         ---------------------------------------------------------------------

C         SPEED OF SOUND
          GAMMAL = CPVALL*RGVALL/(CPVALL - RGVALL)
          GAMMAR = CPVALR*RGVALR/(CPVALR - RGVALR)
          AEVALL = SQRT(GAMMAL*TEVALL)
          AEVALR = SQRT(GAMMAR*TEVALR)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("CF",I5)')IPROC
            WRITE(6,'(2(1PE15.7))')DEVALL,DEVALR
            WRITE(6,'(2(1PE15.7))')UEVALL(1),UEVALR(1)
            WRITE(6,'(2(1PE15.7))')UEVALL(2),UEVALR(2)
            WRITE(6,'(2(1PE15.7))')UEVALL(3),UEVALR(3)
            WRITE(6,'(2(1PE15.7))')TEVALL,TEVALR
            WRITE(6,'(2(1PE15.7))')PEVALL,PEVALR
            DO ISPEC = 1, NSPEC
            WRITE(6,'(I5,2(1PE15.7))')ISPEC,YEVALL(ISPEC),YEVALR(ISPEC)
            ENDDO
C            WRITE(6,'(2(1PE15.7))')YEVALL(1),YEVALR(1)
C            WRITE(6,'(2(1PE15.7))')YEVALL(2),YEVALR(2)
C            WRITE(6,'(2(1PE15.7))')YEVALL(1)+YEVALL(2),
C     +                             YEVALR(1)+YEVALR(2)
          ENDIF

C         ---------------------------------------------------------------------

C         CONVECTIVE TERMS
C         RIEMANN PROBLEM AT EACH GAUSS POINT

C         CHECK L/R BASED ON COORDINATE DIRECTION AT THE FACE
          IF(MSIGN(IFACE).GT.0)THEN

C           L AND R ARE CORRECT
            CALL RIEMAN(DEVALL,UEVALL,PEVALL,YEVALL,AEVALL,
     +                  DEVALR,UEVALR,PEVALR,YEVALR,AEVALR,
     +                  DEVALF,UEVALF,PEVALF,YEVALF,IDIRN)

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'(I5,2(1PE15.7))')MSIGN(IFACE),DEVALF,PEVALF
              WRITE(6,'(5X,3(1PE15.7))')UEVALF(1),UEVALF(2),UEVALF(3)
              DO ISPEC = 1, NSPEC
                WRITE(6,'(I5,1PE15.7)')ISPEC,YEVALF(ISPEC)
              ENDDO
C              WRITE(6,'(5X,2(1PE15.7))')YEVALF(1),YEVALF(2)
            ENDIF

          ELSE

C           SWAP L AND R
            CALL RIEMAN(DEVALR,UEVALR,PEVALR,YEVALR,AEVALR,
     +                  DEVALL,UEVALL,PEVALL,YEVALL,AEVALL,
     +                  DEVALF,UEVALF,PEVALF,YEVALF,IDIRN)

CC           DIAGNOSTICS
C            WRITE(6,*)"cfRL",IPROC,ITIME,IRKSTP,ICELL,IFACE

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'(I5,2(1PE15.7))')MSIGN(IFACE),DEVALF,PEVALF
              WRITE(6,'(5X,3(1PE15.7))')UEVALF(1),UEVALF(2),UEVALF(3)
              WRITE(6,'(5X,2(1PE15.7))')YEVALF(1),YEVALF(2)
            ENDIF

          ENDIF

C         ---------------------------------------------------------------------

C         TEMPERATURE FROM THERMAL EOS
          CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C         SPECIES ENTHALPIES FROM CALORIC EOS
          CALL ENTHPY(DEVALF,TEVALF,HEVALF)

C         ---------------------------------------------------------------------

C         MIXTURE INTERNAL ENERGY
          EEVALF = UEVALF(1)*UEVALF(1)
     +           + UEVALF(2)*UEVALF(2)
     +           + UEVALF(3)*UEVALF(3)
          EEVALF = HALF*EEVALF
          DO ISPEC = 1, NSPEC
            EEVALF = EEVALF + YEVALF(ISPEC)*HEVALF(ISPEC)
          ENDDO

C         ---------------------------------------------------------------------

C         MOLECULAR TRANSPORT COEFFICIENTS
          TMVALH = HALF*(TMVALL + TMVALR)
          CPVALH = HALF*(CPVALL + CPVALR)
          CALL TRANCO(TMVALH,CPVALH,VISCOH,TCONDH,DIFFMH)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'(4(1PE12.4))')VISCOH,TCONDH,DIFFMH(1),DIFFMH(2)
          ENDIF

C         ---------------------------------------------------------------------

C         VISCOUS TERMS
          IDRP1 = MNVIST(1,IDIRN)
          IDRP2 = MNVIST(2,IDIRN)
          VISTAU(IDIRN,IDIRN) = HALF*VISCOH*(VISTNL + VISTNR)
          VISTAU(IDRP1,IDIRN) = HALF*VISCOH*(VIST1L + VIST1R)
          VISTAU(IDRP2,IDIRN) = HALF*VISCOH*(VIST2L + VIST2R)
          VISTRF = UEVALF(1)*VISTAU(1,IDIRN)
     +           + UEVALF(2)*VISTAU(2,IDIRN)
     +           + UEVALF(3)*VISTAU(3,IDIRN)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'(6(1PE12.4))')VISTNL,VISTNR,VIST1L,VIST1R,
     +                                           VIST2L,VIST2R
            WRITE(6,'(3I5)')IDIRN,IDRP1,IDRP2
            WRITE(6,'(3(1PE12.4))')VISTAU(1,IDIRN),
     +                             VISTAU(2,IDIRN),
     +                             VISTAU(3,IDIRN)
            WRITE(6,'(3(1PE12.4))')VISTAU(1,1),VISTAU(1,2),VISTAU(1,3)
            WRITE(6,'(3(1PE12.4))')VISTAU(2,1),VISTAU(2,2),VISTAU(2,3)
            WRITE(6,'(3(1PE12.4))')VISTAU(3,1),VISTAU(3,2),VISTAU(3,3)
          ENDIF

C         THERMAL CONDUCTIVITY TERMS
          TCONDF = HALF*TCONDH*(TDVALL + TDVALR)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'(3(1PE12.4))')TDVALL,TDVALR,TCONDF
          ENDIF

C         MASS DIFFUSIVITY TERMS
          DO ISPEC = 1, NSPEC
            YDIFFF(ISPEC) = HALF*DIFFMH(ISPEC)*(YDVALL(ISPEC)
     +                                        + YDVALR(ISPEC))

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'(3(1PE12.4))')YDVALL(ISPEC),YDVALR(ISPEC),
     +                               YDIFFF(ISPEC)
            ENDIF

          ENDDO

C         ---------------------------------------------------------------------

C         FLUXES
C         ------
C         MASS
          DFLUXF = DEVALF*UEVALF(IDIRN)

C         MOMENTUM
          UFLUXF(1) = DFLUXF*UEVALF(1) + PEVALF*RDELTA(1,IDIRN)
     +              - VISTAU(1,IDIRN)
          UFLUXF(2) = DFLUXF*UEVALF(2) + PEVALF*RDELTA(2,IDIRN)
     +              - VISTAU(2,IDIRN)
          UFLUXF(3) = DFLUXF*UEVALF(3) + PEVALF*RDELTA(3,IDIRN)
     +              - VISTAU(3,IDIRN)
C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'(I5,5(1PE12.4))')IDIRN,DFLUXF,UEVALF(3),PEVALF,
     +                                RDELTA(3,IDIRN),VISTAU(3,IDIRN)
          ENDIF

C         ENERGY
          EFLUXF = DFLUXF*EEVALF - TCONDF - VISTRF

C         SPECIES MASS FRACTION
C         ALSO COMPUTE THE DIFFUSIVE CORRECTION
          YDCORR = ZERO
          DO ISPEC = 1, NSPEC
            YFLUXF(ISPEC) = DFLUXF*YEVALF(ISPEC) - YDIFFF(ISPEC)
            YDCORR = YDCORR + YDIFFF(ISPEC)
          ENDDO

C         SPECIES MASS FRACTION
C         APPLY DIFFUSIVE CORRECTION
C         ENERGY EQUATION
C         SPECIES ENTHALPY TRANSPORT
          DO ISPEC = 1, NSPEC
            YDCORY = YDCORR*YEVALF(ISPEC)
            YFLUXF(ISPEC) = YFLUXF(ISPEC) + YDCORY
            EFLUXF = EFLUXF - (YDIFFF(ISPEC)-YDCORY)*HEVALF(ISPEC)
          ENDDO

C         ---------------------------------------------------------------------

C         ADD FLUXES TO RHS ACCUMULATORS
C         ------------------------------

C         SCALING
C          FFACTR = GPFLUX*OCSIZE*RSIGN(IFACE)
          FFACTR = GPFLUX*RSIGN(IFACE)
          DFLUXF = FFACTR*DFLUXF
          UFLUXF(1) = FFACTR*UFLUXF(1)
          UFLUXF(2) = FFACTR*UFLUXF(2)
          UFLUXF(3) = FFACTR*UFLUXF(3)
          EFLUXF = FFACTR*EFLUXF

C         LOCAL CELL
          DRHS(ICELL) = DRHS(ICELL) + DFLUXF
          URHS(1,ICELL) = URHS(1,ICELL) + UFLUXF(1)
          URHS(2,ICELL) = URHS(2,ICELL) + UFLUXF(2)
          URHS(3,ICELL) = URHS(3,ICELL) + UFLUXF(3)
          ERHS(ICELL) = ERHS(ICELL) + EFLUXF
          DO ISPEC = 1, NSPEC
            YFLUXF(ISPEC) = FFACTR*YFLUXF(ISPEC)
            YRHS(ISPEC,ICELL) = YRHS(ISPEC,ICELL) + YFLUXF(ISPEC)
          ENDDO

C         NEIGHBOUR
          DRHS(INEXT) = DRHS(INEXT) - DFLUXF
          URHS(1,INEXT) = URHS(1,INEXT) - UFLUXF(1)
          URHS(2,INEXT) = URHS(2,INEXT) - UFLUXF(2)
          URHS(3,INEXT) = URHS(3,INEXT) - UFLUXF(3)
          ERHS(INEXT) = ERHS(INEXT) - EFLUXF
          DO ISPEC = 1, NSPEC
            YRHS(ISPEC,INEXT) = YRHS(ISPEC,INEXT) - YFLUXF(ISPEC)
          ENDDO

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'(2(1PE12.4))')GPFLUX,FFACTR
            WRITE(6,'(4(1PE12.4))')DFLUXF,UFLUXF(1),UFLUXF(2),UFLUXF(3)
            WRITE(6,'(3(1PE12.4))')EFLUXF,YFLUXF(1),YFLUXF(2)
          ENDIF

C         ---------------------------------------------------------------------

        ENDDO
C       END OF RUN THROUGH THE GAUSS POINTS

C       -----------------------------------------------------------------------

C       PUSH A FREE SLOT ONTO THE FREE SLOT STACK
        IF(IEFACE.LT.NFSTAK(1))THEN
          ISTACK = ISTACK + 1
          NFSTAK(ISTACK) = IEFACE
        ELSE
          NFSTAK(1) = NFSTAK(1) - 1
        ENDIF
                
C       FREE THE SLOT
        NEFACE(IFACE,ICELL) = 0
 
C       =======================================================================

      ENDIF
C     END OF CHECK FOR EXISTING CELL FACE VALUES

C     =========================================================================


      RETURN
      END
