      SUBROUTINE CH22HF(IFACE,IFCODE)

C     *************************************************************************
C
C     CH22HF
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     16-MAY-2009:  CREATED
C     11-AUG-2014:  RSC REWRITTEN FOR WENO
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES CELL FACE FLUXES FOR ALL PDEs
C     INCLUDES MULTIPLE SCALARS AND MULTI-STEP CHEMISTRY
C     h-2h TRANSITION
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER IFCODE(NFACMX)
      INTEGER IFACE


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YEVALL(NSPCMX),YEVALR(NSPCMX)
      DOUBLE PRECISION YDVALL(NSPCMX),YDVALR(NSPCMX)
      DOUBLE PRECISION YMVALL(NSPCMX),YMVALR(NSPCMX)
      DOUBLE PRECISION YEVALF(NSPCMX),YDIFFF(NSPCMX)
      DOUBLE PRECISION YFLUXF(NSPCMX)
      DOUBLE PRECISION DIFFMH(NSPCMX)
      DOUBLE PRECISION HEVALF(NSPCMX)
      DOUBLE PRECISION VISTAU(NCMPMX,NCMPMX)
      DOUBLE PRECISION UEVALL(NCMPMX),UEVALR(NCMPMX),UEVALF(NCMPMX)
      DOUBLE PRECISION UFLUXF(NCMPMX)
      DOUBLE PRECISION DEVALL,TEVALL,PEVALL,AEVALL
      DOUBLE PRECISION DEVALR,TEVALR,PEVALR,AEVALR
      DOUBLE PRECISION DMVALL,TMVALL,PMVALL
      DOUBLE PRECISION DMVALR,TMVALR,PMVALR
      DOUBLE PRECISION TMVALH,CPVALH
      DOUBLE PRECISION DEVALF,TEVALF,PEVALF,EEVALF
      DOUBLE PRECISION CPVALL,RGVALL,GAMMAL
      DOUBLE PRECISION CPVALR,RGVALR,GAMMAR
      DOUBLE PRECISION DFLUXF,EFLUXF
      DOUBLE PRECISION TDVALL,TDVALR
      DOUBLE PRECISION TCONDF,VISTRF
      DOUBLE PRECISION VISTNL,VIST1L,VIST2L
      DOUBLE PRECISION VISTNR,VIST1R,VIST2R
      DOUBLE PRECISION VISCOH,TCONDH
      DOUBLE PRECISION YDCORR,YDCORY
      DOUBLE PRECISION OCSIZE,FFACTR
      INTEGER INEXT
      INTEGER ILEVL,IDIRN,ISPEC
      INTEGER IEFACE,JEFACE
      INTEGER JFACE
      INTEGER IDRP1,IDRP2
      INTEGER JGPFAC
      INTEGER IOFSUB
      INTEGER IQUART

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In CH22HF',ICELL,IFACE

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,'(6I5)')(NXDOOR(JFACE,ICELL),JFACE=1,6)

C     =========================================================================

C     NEIGHBOUR ARRAY CONTAINS CELL INDEX FOR THE 2h NEIGHBOUR CELL
      INEXT = NXDOOR(IFACE,ICELL)

C     GET QUARTET INDEX OF LOCAL CELL FROM THE CELL FACE CODE
      IQUART = IFCODE(IFACE) - NHTO2H + 1
C     GET SUB-FACE OFFSET
      IOFSUB = IQUART - 1

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,'(6I6)')IFCODE(IFACE),IQUART,IOFSUB

C     LEVEL AND RECIPROCAL SIZE OF LOCAL CELL
      ILEVL = NLEVEL(ICELL)
      OCSIZE = OVSIZE(ILEVL)

C     DIRECTION INDEX
      IDIRN = MDIRN(IFACE)

C     =========================================================================

C     CHECK FOR PRE-EXISTING CELL FACE VALUES
C     ---------------------------------------
      IEFACE = NEFACE(IFACE,ICELL)

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,'("H2",6I6)')ICELL,IFACE,INEXT,IEFACE

      IF(IEFACE.EQ.0)THEN

C       NO PRE-EXISTING CELL FACE VALUES
C       --------------------------------

C       POP A SLOT OFF THE FREE SLOT STACK
        IF(ISTACK.GT.1)THEN
          JEFACE = NFSTAK(ISTACK)
          ISTACK = ISTACK - 1 
        ELSE
          JEFACE = NFSTAK(1) + 1
          NFSTAK(1) = JEFACE
        ENDIF

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("NO PRE",I9)')JEFACE
        ENDIF

C       IN THIS SITUATION ONLY (h To 2h WITH NO PRE-EXISTING VALUES)
C       STORE STACK INDEX IN NEXT-DOOR CELL
C       FOR SIGNALLING ONLY
C       STORE STACK INDEX IN *LOCAL* CELL
        JFACE = MAPOPP(IFACE)
        NEFACE(JFACE,INEXT) = JEFACE
        NEFACE(IFACE,ICELL) = JEFACE

C       =======================================================================

C       COMPUTE NEW CELL FACE VALUES
C       ----------------------------
C       NOTE THAT L/R DISTINCTION IS IRRELEVANT AT THIS STAGE
C       USE L FOR TEMPORARY STORAGE

C       RUN THROUGH THE GAUSS POINTS
        DO JGPFAC = 1, NGPFAC

C         ---------------------------------------------------------------------

C         STORE PRECOMPUTED CELL FACE VALUES
          DEFACE(JGPFAC,JEFACE) = DGVALF(JGPFAC,IFACE)
          UEFACE(1,JGPFAC,JEFACE) = UGVALF(1,JGPFAC,IFACE)
          UEFACE(2,JGPFAC,JEFACE) = UGVALF(2,JGPFAC,IFACE)
          UEFACE(3,JGPFAC,JEFACE) = UGVALF(3,JGPFAC,IFACE)
          TEFACE(JGPFAC,JEFACE) = TGVALF(JGPFAC,IFACE)
          PEFACE(JGPFAC,JEFACE) = PGVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YEFACE(ISPEC,JGPFAC,JEFACE) = YGVALF(ISPEC,JGPFAC,IFACE)
          ENDDO
          CPFACE(JGPFAC,JEFACE) = CPVALF(JGPFAC,IFACE)
          RGFACE(JGPFAC,JEFACE) = RGVALF(JGPFAC,IFACE)

C         DIAGNOSTICS
          GPFACE(1,JGPFAC,JEFACE) = GPLOCF(1,JGPFAC,IFACE)
          GPFACE(2,JGPFAC,JEFACE) = GPLOCF(2,JGPFAC,IFACE)
          GPFACE(3,JGPFAC,JEFACE) = GPLOCF(3,JGPFAC,IFACE)

C         ---------------------------------------------------------------------

C         STORE PRECOMPUTED CELL FACE VALUES FOR MOLECULAR TRANSPORT
          DMFACE(JGPFAC,JEFACE) = DMVALF(JGPFAC,IFACE)
          TMFACE(JGPFAC,JEFACE) = TMVALF(JGPFAC,IFACE)
          PMFACE(JGPFAC,JEFACE) = PMVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YMFACE(ISPEC,JGPFAC,JEFACE) = YMVALF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         ---------------------------------------------------------------------

C         STORE PRECOMPUTED CELL FACE GRADIENTS
          UDFACE(1,JGPFAC,JEFACE) = UGRADF(1,JGPFAC,IFACE)
          UDFACE(2,JGPFAC,JEFACE) = UGRADF(2,JGPFAC,IFACE)
          UDFACE(3,JGPFAC,JEFACE) = UGRADF(3,JGPFAC,IFACE)
          TDFACE(JGPFAC,JEFACE) = TGRADF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YDFACE(ISPEC,JGPFAC,JEFACE) = YGRADF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         ---------------------------------------------------------------------

        ENDDO
C       END OF RUN THROUGH THE GAUSS POINTS

C       =======================================================================

      ELSE

C       =======================================================================

C       PRE-EXISTING CELL FACE VALUES
C       -----------------------------
C       IEFACE CORRESPONDS TO LOCAL SUBFACE OF 2h NEIGHBOUR CELL

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)WRITE(6,'("PRE",I5)')IEFACE

C       RUN THROUGH THE GAUSS POINTS
        DO JGPFAC = 1, NGPFAC

C         ---------------------------------------------------------------------

C         GET THE CELL FACE VALUES
C         ------------------------

C         RETRIEVED VALUES
C         ----------------
C         STORED IN L

C         CELL FACE VALUES
          DEVALL = DEFACE(JGPFAC,IEFACE)
          UEVALL(1) = UEFACE(1,JGPFAC,IEFACE)
          UEVALL(2) = UEFACE(2,JGPFAC,IEFACE)
          UEVALL(3) = UEFACE(3,JGPFAC,IEFACE)
          TEVALL = TEFACE(JGPFAC,IEFACE)
          PEVALL = PEFACE(JGPFAC,IEFACE)
          DO ISPEC = 1, NSPEC
            YEVALL(ISPEC) = YEFACE(ISPEC,JGPFAC,IEFACE)
          ENDDO
          CPVALL = CPFACE(JGPFAC,IEFACE)
          RGVALL = RGFACE(JGPFAC,IEFACE)

C         CELL FACE VALUES FOR MOLECULAR TRANSPORT
          DMVALL = DMFACE(JGPFAC,IEFACE)
          TMVALL = TMFACE(JGPFAC,IEFACE)
          PMVALL = PMFACE(JGPFAC,IEFACE)
          DO ISPEC = 1, NSPEC
            YMVALL(ISPEC) = YMFACE(ISPEC,JGPFAC,IEFACE)
          ENDDO

C         VISCOUS STRESS TENSOR
          VISTNL = UDFACE(1,JGPFAC,IEFACE)
          VIST1L = UDFACE(2,JGPFAC,IEFACE)
          VIST2L = UDFACE(3,JGPFAC,IEFACE)

C         CELL FACE GRADIENTS
          TDVALL = TDFACE(JGPFAC,IEFACE)
          DO ISPEC = 1, NSPEC
            YDVALL(ISPEC) = YDFACE(ISPEC,JGPFAC,IEFACE)
          ENDDO

C         ---------------------------------------------------------------------

C         NEW CELL FACE VALUES
C         --------------------
C         STORED IN R

C         CELL FACE VALUES
          DEVALR = DGVALF(JGPFAC,IFACE)
          UEVALR(1) = UGVALF(1,JGPFAC,IFACE)
          UEVALR(2) = UGVALF(2,JGPFAC,IFACE)
          UEVALR(3) = UGVALF(3,JGPFAC,IFACE)
          TEVALR = TGVALF(JGPFAC,IFACE)
          PEVALR = PGVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YEVALR(ISPEC) = YGVALF(ISPEC,JGPFAC,IFACE)
          ENDDO
          CPVALR = CPVALF(JGPFAC,IFACE)
          RGVALR = RGVALF(JGPFAC,IFACE)

C         CELL FACE VALUES FOR MOLECULAR TRANSPORT
          DMVALR = DMVALF(JGPFAC,IFACE)
          TMVALR = TMVALF(JGPFAC,IFACE)
          PMVALR = PMVALF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YMVALR(ISPEC) = YMVALF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         VISCOUS STRESS TENSOR
          VISTNR = UGRADF(1,JGPFAC,IFACE)
          VIST1R = UGRADF(2,JGPFAC,IFACE)
          VIST2R = UGRADF(3,JGPFAC,IFACE)

C         CELL FACE GRADIENTS
          TDVALR = TGRADF(JGPFAC,IFACE)
          DO ISPEC = 1, NSPEC
            YDVALR(ISPEC) = YGRADF(ISPEC,JGPFAC,IFACE)
          ENDDO

C         ---------------------------------------------------------------------

C         SPEED OF SOUND
          GAMMAL = CPVALL*RGVALL/(CPVALL - RGVALL)
          GAMMAR = CPVALR*RGVALR/(CPVALR - RGVALR)
          AEVALL = SQRT(GAMMAL*TEVALL)
          AEVALR = SQRT(GAMMAR*TEVALR)

C         ---------------------------------------------------------------------

C         CONVECTIVE TERMS
C         RIEMANN PROBLEM AT EACH GAUSS POINT

C         CHECK L/R BASED ON COORDINATE DIRECTION AT THE FACE
          IF(MSIGN(IFACE).GT.0)THEN

C           L AND R ARE CORRECT
            CALL RIEMAN(DEVALL,UEVALL,PEVALL,YEVALL,AEVALL,
     +                  DEVALR,UEVALR,PEVALR,YEVALR,AEVALR,
     +                  DEVALF,UEVALF,PEVALF,YEVALF,IDIRN)

          ELSE

C           SWAP L AND R
            CALL RIEMAN(DEVALR,UEVALR,PEVALR,YEVALR,AEVALR,
     +                  DEVALL,UEVALL,PEVALL,YEVALL,AEVALL,
     +                  DEVALF,UEVALF,PEVALF,YEVALF,IDIRN)

CC           DIAGNOSTICS
C            WRITE(6,*)"h2RF",IPROC,ITIME,IRKSTP,ICELL,IFACE

          ENDIF

C         ---------------------------------------------------------------------

C         TEMPERATURE FROM THERMAL EOS
          CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C         SPECIES ENTHALPIES AND MIXTURE INTERNAL ENERGY FROM CALORIC EOS
          CALL ENTHPY(DEVALF,TEVALF,HEVALF)

C         ---------------------------------------------------------------------

C         MIXTURE INTERNAL ENERGY
          EEVALF = UEVALF(1)*UEVALF(1)
     +           + UEVALF(2)*UEVALF(2)
     +           + UEVALF(3)*UEVALF(3)
          EEVALF = HALF*EEVALF
          DO ISPEC = 1, NSPEC
            EEVALF = EEVALF + YEVALF(ISPEC)*HEVALF(ISPEC)
          ENDDO

C         ---------------------------------------------------------------------

C         MOLECULAR TRANSPORT COEFFICIENTS
          TMVALH = HALF*(TMVALL + TMVALR)
          CPVALH = HALF*(CPVALL + CPVALR)
          CALL TRANCO(TMVALH,CPVALH,VISCOH,TCONDH,DIFFMH)

C         ---------------------------------------------------------------------


C         VISCOUS TERMS
          IDRP1 = MNVIST(1,IDIRN)
          IDRP2 = MNVIST(2,IDIRN)
          VISTAU(IDIRN,IDIRN) = HALF*VISCOH*(VISTNL + VISTNR)
          VISTAU(IDRP1,IDIRN) = HALF*VISCOH*(VIST1L + VIST1R)
          VISTAU(IDRP2,IDIRN) = HALF*VISCOH*(VIST2L + VIST2R)
          VISTRF = UEVALF(1)*VISTAU(1,IDIRN)
     +           + UEVALF(2)*VISTAU(2,IDIRN)
     +           + UEVALF(3)*VISTAU(3,IDIRN)

C         THERMAL CONDUCTIVITY TERMS
          TCONDF = HALF*TCONDH*(TDVALL + TDVALR)

C         MASS DIFFUSIVITY TERMS
          DO ISPEC = 1, NSPEC
            YDIFFF(ISPEC) = HALF*DIFFMH(ISPEC)*(YDVALL(ISPEC)
     +                                        + YDVALR(ISPEC))
          ENDDO

C         ---------------------------------------------------------------------

C         FLUXES
C         ------
C         MASS
          DFLUXF = DEVALF*UEVALF(IDIRN)

C         MOMENTUM
          UFLUXF(1) = DFLUXF*UEVALF(1) + PEVALF*RDELTA(1,IDIRN)
     +              - VISTAU(1,IDIRN)
          UFLUXF(2) = DFLUXF*UEVALF(2) + PEVALF*RDELTA(2,IDIRN)
     +              - VISTAU(2,IDIRN)
          UFLUXF(3) = DFLUXF*UEVALF(3) + PEVALF*RDELTA(3,IDIRN)
     +              - VISTAU(3,IDIRN)

C         ENERGY
          EFLUXF = DFLUXF*EEVALF - TCONDF - VISTRF

C         SPECIES MASS FRACTION
C         ALSO COMPUTE THE DIFFUSIVE CORRECTION
          YDCORR = ZERO
          DO ISPEC = 1, NSPEC
            YFLUXF(ISPEC) = DFLUXF*YEVALF(ISPEC) - YDIFFF(ISPEC)
            YDCORR = YDCORR + YDIFFF(ISPEC)
          ENDDO

C         SPECIES MASS FRACTION
C         APPLY DIFFUSIVE CORRECTION
C         ENERGY EQUATION
C         SPECIES ENTHALPY TRANSPORT
          DO ISPEC = 1, NSPEC
            YDCORY = YDCORR*YEVALF(ISPEC)
            YFLUXF(ISPEC) = YFLUXF(ISPEC) + YDCORY
            EFLUXF = EFLUXF - (YDIFFF(ISPEC)-YDCORY)*HEVALF(ISPEC)
          ENDDO

C         ---------------------------------------------------------------------

C         ADD FLUXES TO RHS ACCUMULATORS
C         ------------------------------

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,'(4X,5(1PE15.7))')DFLUXF,
     +                UFLUXF(1),UFLUXF(2)
C     +                UFLUXF(1),UFLUXF(2),EFLUXF,YFLUXF(1),YFLUXF(2)

C         LOCAL CELL
C          FFACTR = GPFLUX*OCSIZE*RSIGN(IFACE)
          FFACTR = GPFLUX*RSIGN(IFACE)
          DRHS(ICELL) = DRHS(ICELL) + FFACTR*DFLUXF
          URHS(1,ICELL) = URHS(1,ICELL) + FFACTR*UFLUXF(1)
          URHS(2,ICELL) = URHS(2,ICELL) + FFACTR*UFLUXF(2)
          URHS(3,ICELL) = URHS(3,ICELL) + FFACTR*UFLUXF(3)
          ERHS(ICELL) = ERHS(ICELL) + FFACTR*EFLUXF
          DO ISPEC = 1, NSPEC
            YRHS(ISPEC,ICELL) = YRHS(ISPEC,ICELL) + FFACTR*YFLUXF(ISPEC)
          ENDDO

C         NEIGHBOUR (WITH RESCALING)
C          FFACTR = HALF*GPFLUX*FFACTR
          FFACTR = GPFLUX*FFACTR
          DRHS(INEXT) = DRHS(INEXT) - FFACTR*DFLUXF
          URHS(1,INEXT) = URHS(1,INEXT) - FFACTR*UFLUXF(1)
          URHS(2,INEXT) = URHS(2,INEXT) - FFACTR*UFLUXF(2)
          URHS(3,INEXT) = URHS(3,INEXT) - FFACTR*UFLUXF(3)
          ERHS(INEXT) = ERHS(INEXT) - FFACTR*EFLUXF
          DO ISPEC = 1, NSPEC
            YRHS(ISPEC,INEXT) = YRHS(ISPEC,INEXT) - FFACTR*YFLUXF(ISPEC)
          ENDDO

C         ---------------------------------------------------------------------

        ENDDO
C       END OF RUN THROUGH THE GAUSS POINTS

C       -----------------------------------------------------------------------

C       PUSH A FREE SLOT ONTO THE FREE SLOT STACK
        IF(IEFACE.LT.NFSTAK(1))THEN
          ISTACK = ISTACK + 1
          NFSTAK(ISTACK) = IEFACE
        ELSE
          NFSTAK(1) = NFSTAK(1) - 1
        ENDIF
        NEFACE(IFACE,ICELL) = 0

C       -----------------------------------------------------------------------

      ENDIF
C     END OF CHECK FOR PRE-EXISTING CELL FACE VALUES

C     =========================================================================


      RETURN
      END
