      SUBROUTINE CHRATE(DEVAL,TEVAL,YEVAL)
 
C     *************************************************************************
C
C     CHRATE
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     04-MAR-2008:  CREATED
C      
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES CHEMICAL REACTION RATES
C     USING MULTI-SPECIES MULTI-STEP CHEMISTRY
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION YSMALL,YDENOM
      PARAMETER(YSMALL = 1.0D-30,YDENOM = 1.0D-15)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION YEVAL(NSPCMX)
      DOUBLE PRECISION DEVAL,TEVAL


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YRATE(NSPCMX)
      DOUBLE PRECISION RACNST,RNCNST,REOVRR
      DOUBLE PRECISION RFORWD,RFORLN,RBCKWD,RBCKLN,RFLIND,RFTROE,RFSRIF
      DOUBLE PRECISION OVWMAS,GIBBSP,SCOEFF,RYCONC,TBCONC
      DOUBLE PRECISION PREDUC,FLCNST
      DOUBLE PRECISION TALPHA,OMALPH,OVTST1,TSTAR2,OVTST3
      DOUBLE PRECISION CFCST1,CFCST2,ENCST1,ENCST2,DTCNST
      DOUBLE PRECISION FBROAD,FTCENT,TRATS1,TRATS2,TRATS3,CFACTR,ENFACT
      DOUBLE PRECISION ACFSRI,BCFSRM,OVCSRM,DCFSRI,ECFSRI
      DOUBLE PRECISION FORNOW
      INTEGER JSPEC,JSSPEC
      INTEGER JSTEP,JBODY
      INTEGER ITINT,ICP
      LOGICAL FLTHRD
 
C     DIAGNOSTICS
      INTEGER LPROC

C     BEGIN
C     =====

C     =========================================================================

C     GLOBAL DATA
C     -----------
C     NSPEC total no of species
C     NSTEP total no of steps
C     NBODY total no of third bodies
C     NGIBB total no of Gibbs steps
C     NLIND total no of Lindemann steps
C     NTROE total no of Troe steps
C     NSRIF total no of SRI steps
C     NSPCMX max no of species
C     NSTPMX max no of steps
C     NSSMAX max length of step species-list
C     NRSMAX max length of step reactant- and product-lists
C     NBDYMX max no of third bodies
C     NLLMAX max no of Lindemann steps
C     RPARAM(1,NSTEP) pre-exponential factor ln(A)
C     RPARAM(2,NSTEP) temperature exponent n
C     RPARAM(3,NSTEP) activation energy E/R0
C     NSSPEC(NSSMAX,NSTEP) is the step species-list
C     NRSPEC(NRSMAX,NSTEP) is the step reactant-list
C     NPSPEC(NRSMAX,NSTEP) is the step product-list
C     NRCPEC(NRSMAX,NSTEP) is the step coefficient reactant-list
C     NPCPEC(NRSMAX,NSTEP) is the step coefficient product-list
C     NSSLEN(NSTPMX) contains the length of the species-list for each step
C     NRSLEN(NSTPMX) contains the length of the reactant-list for each step
C     NPSLEN(NSTPMX) contains the length of the product-list for each step
C     NRCLEN(NSTPMX) contains the length of the reactant coefficient-list for
C                    each step
C     NPCLEN(NSTPMX) contains the length of the product coefficient-list for
C                    each step
C     WMOLAR(NSPCMX) molar mass of species
C     OVWMOL(NSPCMX) reciprocal of molar mass of species
C     DIFFMU(NSSMAX,NSTPMX) is the species delta-mu for each step
C     DIFFMW(NSSMAX,NSTPMX) is the species (delta-mu times molar mass)
C                           for each step
C     CRSPEC(NSSMAX,NSTPMX) is the reactant stoichiometric coefficient-list
C                           for each step
C     CPSPEC(NSSMAX,NSTPMX) is the product stoichiometric coefficient-list
C                           for each step
C     MBLIST(NSTPMX) contains 0 if no third body in this step
C                    else contains the index number of the third body
C     MGLIST(NSTPMX) contains 0 if no Gibbs function evaluation in this step
C                    else contains the index number of the Gibbs step
C     MLLIST(NSTPMX) contains 0 if no Lindemann rate in this step
C                    else contains the index number of the Lindemann step
C     MTLIST(NSTPMX) contains 0 if no Troe rate in this step
C                    else contains the index number of the Lindemann step
C     MSLIST(NSTPMX) contains 0 if no SRI rate in this step
C                    else contains the index number of the Lindemann step
C     EFFY3B(NSPEC,NBDYMX) contains the third-body efficiencies
C     RCLIND(1,MLLIST(NSTPMX)) Lindemann rate pre-exponential factor ln(A)
C     RCLIND(2,MLLIST(NSTPMX)) Lindemann rate temperature exponent n
C     RCLIND(3,MLLIST(NSTPMX)) Lindemann rate activation energy E/R0
C     RCLIND(4,MLLIST(NSTPMX)) Lindemann rate broadening factor
C     RCTROE(1,MTLIST(NSTPMX)) Troe form rate pre-exponential factor ln(A)
C     RCTROE(2,MTLIST(NSTPMX)) Troe form rate temperature exponent n
C     RCTROE(3,MTLIST(NSTPMX)) Troe form rate activation energy E/R0
C     RCTROE(4,MTLIST(NSTPMX)) Troe form rate temperature parameter alpha
C     RCTROE(5,MTLIST(NSTPMX)) Troe form rate temperature parameter no 1*
C     RCTROE(6,MTLIST(NSTPMX)) Troe form rate temperature parameter no 2*
C     RCTROE(7,MTLIST(NSTPMX)) Troe form rate temperature parameter no 3*
C     RCTROE(8,MTLIST(NSTPMX)) Troe form rate c-const no 1
C     RCTROE(9,MTLIST(NSTPMX)) Troe form rate c-const no 2
C     RCTROE(10,MTLIST(NSTPMX)) Troe form rate n-const no 1
C     RCTROE(11,MTLIST(NSTPMX)) Troe form rate n-const no 2
C     RCTROE(12,MTLIST(NSTPMX)) Troe form rate d-const
C     RCSRIF(1,MSLIST(NSTPMX)) SRI form rate pre-exponential factor ln(A)
C     RCSRIF(2,MSLIST(NSTPMX)) SRI form rate temperature exponent n
C     RCSRIF(3,MSLIST(NSTPMX)) SRI form rate activation energy E/R0
C     RCSRIF(4,MSLIST(NSTPMX)) SRI form rate parameter a
C     RCSRIF(5,MSLIST(NSTPMX)) SRI form rate parameter b
C     RCSRIF(6,MSLIST(NSTPMX)) SRI form rate parameter c
C     RCSRIF(7,MSLIST(NSTPMX)) SRI form rate parameter d
C     RCSRIF(8,MSLIST(NSTPMX)) SRI form rate parameter e

C     =========================================================================

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WRITE(6,*)'CHRATE',ITIME,IRKSTP,ICELL
        WRITE(6,'(2(1PE12.4))')DEVAL,TEVAL
        DO JSPEC = 1, NSPEC
          WRITE(6,'(I5,1PE15.7)')JSPEC,YEVAL(JSPEC)
        ENDDO
      ENDIF

C     ZERO THE REACTION RATE ACCUMULATOR ARRAY
      DO JSPEC = 1,NSPEC
        YRATE(JSPEC) = ZERO
      ENDDO

C     =========================================================================

C     RUN THROUGH ALL STEPS IN THE REACTION MECHANISM
C     -----------------------------------------------

      DO JSTEP = 1, NSTEP

C       =======================================================================

C       INCLUDE THIRD BODY CONCENTRATIONS, IF ANY
C       -----------------------------------------

C       SET THIRD BODY EVALUATION FLAG
        FLTHRD = .TRUE.

C       ZERO THE THIRD-BODY CONCENTRATION ACCUMULATOR
        TBCONC = ZERO

C       CHECK THIRD-BODY LIST
        IF(MBLIST(JSTEP).NE.0)THEN

C         GET THE INDEX OF THE THIRD BODY
          JBODY = MBLIST(JSTEP)

CC         ZERO THE THIRD-BODY CONCENTRATION ACCUMULATOR
C          TBCONC = ZERO

C         USE THE THIRD-BODY SPECIES-LIST
C         AND THE THIRD-BODY EFFICIENCY LIST
C         TO GET THE THIRD-BODY CONCENTRATION
          DO JSPEC = 1,NSPEC

C           GET THE THIRD-BODY EFFICIENCY FROM THE LIST
C           AND COMBINE WITH THE RECIPROCAL MOLAR MASS
            OVWMAS = OVWMOL(JSPEC)*EFFY3B(JSPEC,JBODY)

C           EVALUATE THE THIRD BODY CONCENTRATION
C           AND STORE IN TEMPORARY ARRAY
C           CONCENTRATION IS RHO*Y/WMOLAR
            TBCONC = TBCONC + DEVAL*YEVAL(JSPEC)*OVWMAS

          ENDDO
C         THIRD-BODY SPECIES-LIST

        ENDIF
C       END OF THIRD BODY TREATMENT

C       =======================================================================

C       EVALUATE THE SPECIFIC REACTION RATE CONSTANT FOR THIS STEP
C       ----------------------------------------------------------
        RACNST = RPARAM(1,JSTEP)
        RNCNST = RPARAM(2,JSTEP)
        REOVRR = RPARAM(3,JSTEP)

        RFORLN = RACNST
     +         + RNCNST*LOG(TEVAL)
     +         - REOVRR/TEVAL

        RFORWD = EXP(RFORLN)

C       =======================================================================

C       INCLUDE LINDEMANN RATE EVALUATION, IF ANY
C       -----------------------------------------
        IF(MLLIST(JSTEP).NE.0)THEN

          RACNST = RCLIND(1,MLLIST(JSTEP))
          RNCNST = RCLIND(2,MLLIST(JSTEP))
          REOVRR = RCLIND(3,MLLIST(JSTEP))
          FLCNST = RCLIND(4,MLLIST(JSTEP))

C         EVALUATE K0
          RFLIND = RACNST
     +           + RNCNST*LOG(TEVAL)
     +           - REOVRR/TEVAL
          RFLIND = EXP(RFLIND)

C         EVALUATE REDUCED PRESSURE
          PREDUC = TBCONC*RFLIND/RFORWD

C         EVALUATE UPDATED FORWARD RATE
          FORNOW = FLCNST*PREDUC/(ONE+PREDUC)
          RFORWD = RFORWD*FORNOW
          RFORLN = LOG(RFORWD)

C         RESET THIRD BODY EVALUATION FLAG#
          FLTHRD = .FALSE.

        ENDIF
C       END OF LINDEMANN RATE TREATMENT

C       =======================================================================

C       INCLUDE TROE FORM RATE EVALUATION, IF ANY
C       -----------------------------------------
        IF(MTLIST(JSTEP).NE.0)THEN

          RACNST = RCTROE(1,MTLIST(JSTEP))
          RNCNST = RCTROE(2,MTLIST(JSTEP))
          REOVRR = RCTROE(3,MTLIST(JSTEP))
          TALPHA = RCTROE(4,MTLIST(JSTEP))
          OVTST1 = RCTROE(5,MTLIST(JSTEP))
          TSTAR2 = RCTROE(6,MTLIST(JSTEP))
          OVTST3 = RCTROE(7,MTLIST(JSTEP))
          CFCST1 = RCTROE(8,MTLIST(JSTEP))
          CFCST2 = RCTROE(9,MTLIST(JSTEP))
          ENCST1 = RCTROE(10,MTLIST(JSTEP))
          ENCST2 = RCTROE(11,MTLIST(JSTEP))
          DTCNST = RCTROE(12,MTLIST(JSTEP))

          OMALPH = ONE - TALPHA

C         EVALUATE K0
          RFTROE = RACNST
     +           + RNCNST*LOG(TEVAL)
     +           - REOVRR/TEVAL
          RFTROE = EXP(RFTROE)

C         EVALUATE REDUCED PRESSURE
          PREDUC = TBCONC*RFTROE/RFORWD

C         EVALUATE FCENT
          TRATS1 = TEVAL*OVTST1
          TRATS2 = TSTAR2/TEVAL
          TRATS3 = TEVAL*OVTST3
          FTCENT = OMALPH*EXP(TRATS3)
     +           + TALPHA*EXP(TRATS1)
     +           + EXP(TRATS2)
          FTCENT = LOG10(FTCENT)

C         EVALUATE BROADENING FACTOR
          CFACTR = CFCST1 + CFCST2*FTCENT
          ENFACT = ENCST1 + ENCST2*FTCENT
          FORNOW = LOG10(PREDUC) + CFACTR
          FORNOW = FORNOW/(ENFACT - DTCNST*FORNOW)
          FORNOW = ONE + FORNOW*FORNOW
          FBROAD = FTCENT/FORNOW
          FBROAD = EXP(FBROAD*CLNTEN)

C         EVALUATE UPDATED FORWARD RATE CONSTANT
          FORNOW = FBROAD*PREDUC/(ONE+PREDUC)
          RFORWD = RFORWD*FORNOW
          RFORLN = LOG(RFORWD)

C         RESET THIRD BODY EVALUATION FLAG
          FLTHRD = .FALSE.

        ENDIF
C       END OF TROE FORM RATE EVALUATION

C       =======================================================================

C       INCLUDE SRI FORM RATE EVALUATION, IF ANY
C       ----------------------------------------
        IF(MSLIST(JSTEP).NE.0)THEN

          RACNST = RCSRIF(1,MSLIST(JSTEP))
          RNCNST = RCSRIF(2,MSLIST(JSTEP))
          REOVRR = RCSRIF(3,MSLIST(JSTEP))
          ACFSRI = RCSRIF(4,MSLIST(JSTEP))
          BCFSRM = RCSRIF(5,MSLIST(JSTEP))
          OVCSRM = RCSRIF(6,MSLIST(JSTEP))
          DCFSRI = RCSRIF(7,MSLIST(JSTEP))
          ECFSRI = RCSRIF(8,MSLIST(JSTEP))

C         EVALUATE K0
          RFSRIF = RACNST
     +           + RNCNST*LOG(TEVAL)
     +           - REOVRR/TEVAL
          RFSRIF = EXP(RFSRIF)

C         EVALUATE REDUCED PRESURE
          PREDUC = TBCONC*RFSRIF/RFORWD

C         EVALUATE FCENT
          FORNOW = BCFSRM/TEVAL
          FBROAD = ACFSRI*EXP(FORNOW)
          FORNOW = TEVAL*OVCSRM
          FBROAD = FBROAD + EXP(FORNOW)
          FORNOW = LOG10(PREDUC)
          FORNOW = ONE/(ONE+LOG10(FORNOW))
          FBROAD = DCFSRI*EXP(FORNOW*LOG(FBROAD))
          FORNOW = FBROAD*EXP(ECFSRI*LOG(TEVAL))

C         EVALUATE UPDATED FORWARD RATE CONSTANT
          FORNOW = FBROAD*PREDUC/(ONE+PREDUC)
          RFORWD = RFORWD*FORNOW
          RFORLN = LOG(RFORWD)

C         RESET THIRD BODY EVALUATION FLAG
          FLTHRD = .FALSE.

        ENDIF
C       END OF SRI FORM RATE EVALUATION

C       =======================================================================

C       USE STEP REACTANT-LIST
C       ----------------------
C       TO FIND REACTANT SPECIES FOR THIS STEP
C       HAVING POSITIVE INTEGER STOICHIOMETRIC COEFFICIENTS

C       NOTE: THIRD BODIES ARE NOT INCLUDED IN THE STEP REACTANT-LIST
C       AND ARE TREATED SEPARATELY (SEE BELOW)

        DO JSSPEC = 1, NRSLEN(JSTEP)

C         GET THE SPECIES NUMBER FROM THE REACTANT-LIST
          JSPEC = NRSPEC(JSSPEC,JSTEP)

C         EVALUATE REACTANT CONCENTRATIONS AND MULTIPLY UP
C         CONCENTRATION IS RHO*Y/WMOLAR
          FORNOW = DEVAL*MAX(YEVAL(JSPEC),ZERO)
          RFORWD = RFORWD*FORNOW*OVWMOL(JSPEC)

        ENDDO
C       END OF STEP REACTANT-LIST

C       =======================================================================

C       USE STEP REACTANT COEFFICIENT-LIST
C       ----------------------------------
C       TO FIND REACTANT SPECIES FOR THIS STEP
C       HAVING NON-POSITIVE-INTEGER STOICHIOMETRIC COEFFICIENTS
C       AND OBTAIN THE COEFFICIENTS

C       NOTE: THIRD BODIES ARE NOT INCLUDED IN THE STEP REACTANT-LIST
C       AND ARE TREATED SEPARATELY (SEE BELOW)

        IF(NRCLEN(JSTEP).GT.0)THEN

          DO JSSPEC = 1, NRCLEN(JSTEP)

C           GET THE SPECIES NUMBER FROM THE STEP COEFFICIENT REACTANT-LIST
            JSPEC = NRCPEC(JSSPEC,JSTEP)

C           GET THE REACTANT STOICHIOMETRIC COEFFICIENT
            SCOEFF = CRSPEC(JSSPEC,JSTEP)

C           EVALUATE REACTANT CONCENTRATIONS AND MULTIPLY UP
C           CONCENTRATION IS RHO*Y/WMOLAR
            RYCONC = DEVAL*MAX(YEVAL(JSPEC),YSMALL)*OVWMOL(JSPEC)
            FORNOW = EXP(SCOEFF*LOG(RYCONC))
            FORNOW = FORNOW/(ONE+YDENOM*FORNOW)
            RFORWD = RFORWD*FORNOW

          ENDDO

        ENDIF
C       END OF STEP REACTANT COEFFICIENT-LIST

C       =======================================================================

C       INCLUDE BACKWARD RATE EVALUATION, IF ANY
C       ----------------------------------------
        IF(MGLIST(JSTEP).NE.0)THEN

C         =====================================================================

C         GIBBS FUNCTION EVALUATION
C         -------------------------
C          RBCKLN = ZERO
          RBCKLN = RFORLN

C         RUN THROUGH ALL SPECIES FOR THIS STEP
          DO JSSPEC = 1, NSSLEN(JSTEP)

C           GET THE SPECIES NUMBER FROM THE STEP SPECIES-LIST
            JSPEC = NSSPEC(JSSPEC,JSTEP) 

C           LOCATE TEMPERATURE IN AN INTERVAL
            ITINT = 1

C           CONSTRUCT GIBBS FUNCTION FROM ITS POLYNOMIAL COEFFICIENTS
            GIBBSP = AMOLGB(NCPOLY(ITINT,JSPEC),ITINT,JSPEC)
            DO ICP = NCPOM1(ITINT,JSPEC),1,-1
              GIBBSP = AMOLGB(ICP,ITINT,JSPEC) + GIBBSP*TEVAL
            ENDDO
            GIBBSP = AMOLGB(NCENTH(ITINT,JSPEC),ITINT,JSPEC)/TEVAL
     +        - AMOLGB(NCENPY(ITINT,JSPEC),ITINT,JSPEC)*LOG(TEVAL)
     +             - GIBBSP

C           ADD GIBBS FUNCTION CONTRIBUTION TO RATE COEFFICIENT
C           USING STEP SPECIES DELTA-LIST TO GET BACKWARD RATE COEFFICIENT
C            RBCKLN = RFORLN + DIFFMU(JSSPEC,JSTEP)*GIBBSP
            RBCKLN = RBCKLN + DIFFMU(JSSPEC,JSTEP)*GIBBSP

          ENDDO
C         STEP SPECIES-LIST

C         =====================================================================

C         FINALISE BACKWARD RATE COEFFICIENT
C         ----------------------------------
          RBCKWD = EXP(RBCKLN)

C         =====================================================================

C         USE STEP PRODUCT-LIST
C         ---------------------
C         TO FIND PRODUCT SPECIES FOR THIS STEP
C         HAVING POSITIVE INTEGER STOICHIOMETRIC COEFFICIENTS

C         NOTE: THIRD BODIES ARE NOT INCLUDED IN THE STEP PRODUCT-LIST
C         AND ARE TREATED SEPARATELY (SEE BELOW)

          DO JSSPEC = 1, NPSLEN(JSTEP)

C           GET THE SPECIES NUMBER FROM THE PRODUCT-LIST
            JSPEC = NPSPEC(JSSPEC,JSTEP)

C           EVALUATE PRODUCT CONCENTRATIONS AND MULTIPLY UP
C           CONCENTRATION IS RHO*Y/WMOLAR
            FORNOW = DEVAL*MAX(YEVAL(JSPEC),ZERO)
            RBCKWD = RBCKWD*FORNOW*OVWMOL(JSPEC)

          ENDDO
C         END OF STEP PRODUCT-LIST

C         =====================================================================

C         USE STEP PRODUCT COEFFICIENT-LIST
C         ---------------------------------
C         TO FIND PRODUCT SPECIES FOR THIS STEP
C         HAVING NON-POSITIVE-INTEGER STOICHIOMETRIC COEFFICIENTS
C         AND OBTAIN THE COEFFICIENTS

C         NOTE: THIRD BODIES ARE NOT INCLUDED IN THE STEP PRODUCT-LIST
C         AND ARE TREATED SEPARATELY (SEE BELOW)

          IF(NPCLEN(JSTEP).GT.0)THEN

            DO JSSPEC = 1, NPCLEN(JSTEP)

C             GET THE SPECIES NUMBER FROM THE STEP COEFFICIENT PRODUCT-LIST
              JSPEC = NPCPEC(JSSPEC,JSTEP)

C             GET THE PRODUCT STOICHIOMETRIC COEFFICIENT
              SCOEFF = CPSPEC(JSSPEC,JSTEP)

C             EVALUATE PRODUCT CONCENTRATIONS AND MULTIPLY UP
C             CONCENTRATION IS RHO*Y/WMOLAR
              RYCONC = DEVAL*MAX(YEVAL(JSPEC),YSMALL)*OVWMOL(JSPEC)
              FORNOW = EXP(SCOEFF*LOG(RYCONC))
              FORNOW = FORNOW/(ONE+YDENOM*FORNOW)
              RBCKWD = RBCKWD*FORNOW

            ENDDO

          ENDIF
C         END OF STEP PRODUCT COEFFICIENT-LIST

C         =====================================================================

C         EVALUATE NET FORWARD REACTION RATE
C         ----------------------------------
          RFORWD = RFORWD - RBCKWD

C         =====================================================================

        ENDIF
C       END OF BACKWARD RATE EVALUATION

C       =======================================================================

C       INCLUDE THIRD BODY CONCENTRATIONS, IF ANY
C       -----------------------------------------
        IF(MBLIST(JSTEP).NE.0)THEN

C         INCLUDE THIRD BODY CONCENTRATION IN REACTION RATE FOR THE STEP
          IF(FLTHRD)RFORWD = RFORWD*TBCONC

        ENDIF
C       END OF THIRD BODY TREATMENT

C       =======================================================================

C       USE STEP SPECIES-LIST AND STEP SPECIES DELTA-LIST
C       -------------------------------------------------
C       TO GET DIFFERENCE OF STOICHIOMETRIC COEFFICIENTS FOR EACH SPECIES 
C       (ACTUALLY DELTA-MU TIMES WMOLAR)
C       ADD STEP CONTRIBUTION TO TOTAL RATE FOR EACH SPECIES INVOLVED

        DO JSSPEC = 1, NSSLEN(JSTEP)

C         GET THE SPECIES NUMBER FROM THE STEP SPECIES-LIST
          JSPEC = NSSPEC(JSSPEC,JSTEP) 
 
          YRATE(JSPEC) = YRATE(JSPEC) + RFORWD*DIFFMW(JSSPEC,JSTEP)

        ENDDO
C       STEP SPECIES-LIST

C       =======================================================================

      ENDDO

C     END OF RUN THROUGH ALL STEPS IN THE REACTION MECHANISM
C     ------------------------------------------------------

C     =========================================================================

C     UPDATE THE RHS ACCUMULATOR
      DO JSPEC = 1,NSPEC
        YRHS(JSPEC,ICELL) = YRHS(JSPEC,ICELL) + GPFVOL*YRATE(JSPEC)
      ENDDO

C     =========================================================================


      RETURN
      END
