      SUBROUTINE DTINIT
 
C     *************************************************************************
C
C     DTINIT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT -- CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C     20-NOV-2010:  UPDATED FOR LOW-ORDER RK SCHEMES
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES RK TIME-STEPPING SCHEME
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION ACOFRK(NRKMAX),BCOFRK(NRKMAX),BHATRK(NRKMAX)
      DOUBLE PRECISION ALSVRK(NRKMAX),ALRNRK(NRKMAX)
      INTEGER JCELL,JSPEC


C     BEGIN
C     =====

C     =========================================================================

C     VALUES DEFINED IN THIS BLOCK FOR

C     ERK COEFFICIENTS
C     ERK ERROR ESTIMATION NORMALISING VALUES
C     ERK TIME STEP CONTROLLER COEFFICIENTS
C     ERK TOLERANCES AND LIMITS

CC     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK2(1)2[2R]
CC     ----------------------
C      NRKSTP = 2
C
C      ACOFRK(1) = 2.0D0/3.0D0
C      ACOFRK(2) = ZERO
C
C      BCOFRK(1) = 1.0D0/4.0D0
C      BCOFRK(2) = 3.0D0/4.0D0
C
C      BHATRK(1) = 6.0D-1
C      BHATRK(2) = 4.0D-1
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C
CC     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK2(1)2[2R](Huen)
CC     ----------------------------
C      NRKSTP = 2
C
C      ACOFRK(1) = ONE
C      ACOFRK(2) = ZERO
C
C      BCOFRK(1) = HALF
C      BCOFRK(2) = HALF
C
C      BHATRK(1) = 7.5D-1
C      BHATRK(2) = 2.5D-1
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C
CC     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK3(2)3[2R+] Wray 1
CC     ------------------------------
C      NRKSTP = 3
C
C      ACOFRK(1) = 8.0D0/1.5D1
C      ACOFRK(2) = 5.0D0/1.2D1
C      ACOFRK(3) = ZERO
C
C      BCOFRK(1) = 1.0D0/4.0D0
C      BCOFRK(2) = 0.0D0
C      BCOFRK(3) = 3.0D0/4.0D0
C
C      BHATRK(1) = 1.5D-1
C      BHATRK(2) = 5.0D-1
C      BHATRK(3) = 3.5D-1
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C      ALSVRK(3) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C      ALRNRK(3) = ONE
C
CC     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK3(2)3[2R+] Wray 2
CC     ------------------------------
C      NRKSTP = 3
C
C      ACOFRK(1) = 2.0D0/3.0D0
C      ACOFRK(2) = 5.0D0/1.2D1
C      ACOFRK(3) = ZERO
C
C      BCOFRK(1) = 1.0D0/4.0D0
C      BCOFRK(2) = 3.0D0/2.0D1
C      BCOFRK(3) = 3.0D0/5.0D0
C
C      BHATRK(1) = 2.5D-1
C      BHATRK(2) = 5.0D-1
C      BHATRK(3) = 2.5D-1
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C      ALSVRK(3) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C      ALRNRK(3) = ONE
C
CC     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK3(2)3[2R+]M
CC     ------------------------
C      NRKSTP = 3
C
C      ACOFRK(1) = 5.56769001341702D-1
C      ACOFRK(2) = 4.23701919118994D-1
C      ACOFRK(3) = ZERO
C
C      BCOFRK(1) = 2.48730621581008D-1
C      BCOFRK(2) = 4.47675811160560D-2
C      BCOFRK(3) = 7.06501797302936D-1
C
C      BHATRK(1) = 1.7D-1
C      BHATRK(2) = 5.0248D-1
C      BHATRK(3) = 3.2752D-1
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C      ALSVRK(3) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C      ALRNRK(3) = ONE
C
CC     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK3(2)3[2R+]N
CC     ------------------------
C      NRKSTP = 3
C
C      ACOFRK(1) = 7.55726351960521D-1
C      ACOFRK(2) = 3.86954477298316D-1
C      ACOFRK(3) = ZERO
C
C      BCOFRK(1) = 2.45170287301682D-1
C      BCOFRK(2) = 1.84896052190911D-1
C      BCOFRK(3) = 5.69933660507406D-1
C
C      BHATRK(1) = 2.1D-1
C      BHATRK(2) = 5.057D-2
C      BHATRK(3) = 7.8494D-1
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C      ALSVRK(3) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C      ALRNRK(3) = ONE
C
CC     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK3(2)4[2R+]C
CC     ------------------------
C      NRKSTP = 4
C
C      ACOFRK(1) = 11847461282814D0/36547543011857D0
C      ACOFRK(2) = 3943225443063D0/7078155732230D0
C      ACOFRK(3) = -346793006927D0/4029903576067D0
C      ACOFRK(4) = ZERO
C
C      BCOFRK(1) = 1017324711453D0/9774461848756D0
C      BCOFRK(2) = 8237718856693D0/13685301971492D0
C      BCOFRK(3) = 57731312506979D0/19404895981398D0
C      BCOFRK(4) = -101169746363290D0/37734290219643D0
C
C      BHATRK(1) = 15763415370699D0/46270243929542D0
C      BHATRK(2) = 514528521746D0/5659431552419D0
C      BHATRK(3) = 27030193851939D0/9429696342944D0
C      BHATRK(4) = -69544964788955D0/30262026368149D0
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C      ALSVRK(3) = ZERO
C      ALSVRK(4) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C      ALRNRK(3) = ONE
C      ALRNRK(4) = ONE
C
C     -------------------------------------------------------------------------
C
CC     ERK SCHEME RK4(3)5[2R+]C
CC     ------------------------
C      NRKSTP = 5
C
C      ACOFRK(1) = 970286171893D0/4311952581923D0
C      ACOFRK(2) = 6584761158862D0/12103376702013D0
C      ACOFRK(3) = 2251764453980D0/15575788980749D0
C      ACOFRK(4) = 26877169314380D0/34165994151039D0
C      ACOFRK(5) = ZERO
C
C      BCOFRK(1) = 1153189308089D0/22510343858157D0
C      BCOFRK(2) = 1772645290293D0/4653164025191D0
C      BCOFRK(3) = -1672844663538D0/4480602732383D0
C      BCOFRK(4) = 2114624349019D0/3568978502595D0
C      BCOFRK(5) = 5198255086312D0/14908931495163D0
C
C      BHATRK(1) = 1016888040809D0/7410784769900D0
C      BHATRK(2) = 11231460423587D0/58533540763752D0
C      BHATRK(3) = -1563879915014D0/6823010717585D0
C      BHATRK(4) = 606302364029D0/971179775848D0
C      BHATRK(5) = 1097981568119D0/3980877426909D0
C
C      ALSVRK(1) = ZERO
C      ALSVRK(2) = ZERO
C      ALSVRK(3) = ZERO
C      ALSVRK(4) = ZERO
C      ALSVRK(5) = ZERO
C
C      ALRNRK(1) = ONE
C      ALRNRK(2) = ONE
C      ALRNRK(3) = ONE
C      ALRNRK(4) = ONE
C      ALRNRK(5) = ONE
C
C     -------------------------------------------------------------------------

C     TVD SCHEME SHU-OSHER (MOD)
C     --------------------------
      NRKSTP = 3

      ACOFRK(1) = ONE
      ACOFRK(2) = QRTR
      ACOFRK(3) = TTHD

      BCOFRK(1) = ONE
      BCOFRK(2) = QRTR
      BCOFRK(3) = TTHD

      BHATRK(1) = THREE*QRTR
      BHATRK(2) = THREE*QRTR*QRTR
      BHATRK(3) = THREE*QRTR

c      BHATRK(1) = THRF
c      BHATRK(2) = THREE*EIGHTH
c      BHATRK(3) = HALF

      ALSVRK(1) = ONE
      ALSVRK(2) = THREE*QRTR
      ALSVRK(3) = THRD

      ALRNRK(1) = ZERO
      ALRNRK(2) = QRTR
      ALRNRK(3) = TTHD

C     -------------------------------------------------------------------------

C     SET TOLERANCES AND LIMITS FOR THE RK TIME STEP CONTROLLER
C     ---------------------------------------------------------
      ERRTOL = 1.0D-3
C      ERRTOL = 1.0D-4
      ERRLOW = 1.0D-30
C      TRMAX = 1.05D0
C      TRMAX = 1.01D0
C      TRMIN = 1.0D-2
      TRMAX = 1.0D6
      TRMIN = 1.0D-10
      TSMAX = 1.0D0
      TSMIN = 1.0D-15

C     -------------------------------------------------------------------------

C     SET PAST ERROR VALUES FOR THE RK TIME STEP CONTROLLER
C     -----------------------------------------------------
      IF(MOD(ICFLAG,2).EQ.0)THEN
        ERROLD = ERRTOL
        ERRLDR = ERRTOL
      ENDIF

C     -------------------------------------------------------------------------

C     SET INDICATORS FOR VARIABLE AND LOCATION
      JINDIC = 0
      JERLOC = 0

C     -------------------------------------------------------------------------

C     INITIALISE THE NORMALISING VALUES FOR RK ERROR ESTIMATION
C     ---------------------------------------------------------
      ERDNRM = DRIN
      ERUNRM = ONE
C      ERUNRM = 3.4D2
      IF(ABS(URIN).GT.ERRTOL)ERUNRM = DRIN*ABS(URIN)
      ERVNRM = ONE
      IF(ABS(VRIN).GT.ERRTOL)ERVNRM = DRIN*ABS(VRIN)
      ERWNRM = ONE
      IF(ABS(WRIN).GT.ERRTOL)ERWNRM = DRIN*ABS(WRIN)
      ERENRM = ONE
      IF(ABS(ERIN).GT.ERRTOL)ERENRM = DRIN*ABS(ERIN)
      DO JSPEC = 1,NSPEC
        ERYNRM(JSPEC) = ONE
      ENDDO

      ERDNRM = ONE/ERDNRM
      ERUNRM = ONE/ERUNRM
      ERVNRM = ONE/ERVNRM
      ERWNRM = ONE/ERWNRM
      ERENRM = ONE/ERENRM

C     -------------------------------------------------------------------------

C     SET THE COEFFICIENTS FOR THE RK TIME STEP CONTROLLER
C     ----------------------------------------------------
C     I CONTROLLER
C     CTALPH = 1.0/ORDER OF EMBEDDED SCHEME+1
C      CTMULT = 9.0D-1
C      CTALPH = ONE/THREE
C      CTALPH = ONE/FOUR
C     PI CONTROLLER
C     CTALPH = 0.7/ORDER OF EMBEDDED SCHEME
C     CTBETA = 0.4/ORDER OF EMBEDDED SCHEME
C      CTMULT = 9.0D-1
C      CTMULT = 5.0D-1
C      CTMULT = 1.0D0
C      CTALPH = 7.0D-1/THREE
C      CTBETA = 4.0D-1/THREE

C      CTALPH = 6.0D-1/THREE
C      CTBETA = 2.0D-1/THREE

      CTMULT = 9.0D-1
C      CTMULT = 8.0D-1
      CTALPH = 6.0D-1/TWO
      CTBETA = 2.0D-1/TWO

C      CTALPH = 7.0D-1
C      CTBETA = 4.0D-1
CC     PID CONTROLLER
CC     CTALPH = 0.49/ORDER OF EMBEDDED SCHEME
CC     CTBETA = 0.34/ORDER OF EMBEDDED SCHEME
CC     CTGAMA = 0.10/ORDER OF EMBEDDED SCHEME
C      CTMULT = 9.0D-1
CC      CTALPH = 4.9D-1/THREE
CC      CTBETA = 3.4D-1/THREE
CC      CTGAMA = 1.0D-1/THREE
C      CTALPH = 4.9D-1/TWO
C      CTBETA = 3.4D-1/TWO
C      CTGAMA = 1.0D-1/TWO

C     -------------------------------------------------------------------------

C     SET TIME STEP ADAPTION FLAG
C     ---------------------------
      FLADPT = .TRUE.
      IF(NSTPSW.EQ.0)FLADPT = .FALSE.

C     -------------------------------------------------------------------------

C     END OF BLOCK FOR

C     ERK COEFFICIENTS
C     ERK ERROR ESTIMATION NORMALISING VALUES
C     ERK TIME STEP CONTROLLER COEFFICIENTS
C     ERK TOLERANCES AND LIMITS

C     =========================================================================

C     NO OF SUBSTEPS MINUS ONE
C     ------------------------
      NRKSM1 = NRKSTP - 1

C     =========================================================================

C     RK COEFFICIENTS
C     ---------------
      DO IRKSTP = 1,NRKSTP
        RKLHC(IRKSTP) = BCOFRK(IRKSTP)
        RKRHC(IRKSTP) = ACOFRK(IRKSTP)
        RKSVC(IRKSTP) = ALSVRK(IRKSTP)
        RKRNC(IRKSTP) = ALRNRK(IRKSTP)
        RKERC(IRKSTP) = BCOFRK(IRKSTP) - BHATRK(IRKSTP)
      ENDDO

C     =========================================================================

C     TIME-ADVANCEMENT COEFFICIENTS
C     -----------------------------
      DO IRKSTP = 1,NRKSTP
        RKLHS(IRKSTP) = RKLHC(IRKSTP)*TSTEP
        RKRHS(IRKSTP) = RKRHC(IRKSTP)*TSTEP
        RKSAV(IRKSTP) = RKSVC(IRKSTP)
        RKRUN(IRKSTP) = RKRNC(IRKSTP)
        RKERR(IRKSTP) = RKERC(IRKSTP)*TSTEP
      ENDDO

C     =========================================================================

C     TIME LEVELS FOR EACH RK SUBSTEP
C     -------------------------------
C     SPECIAL FOR SHU-OSHER
      RKTMC(1) = ZERO
      RKTMC(2) = ONE
      RKTMC(3) = HALF
      DO IRKSTP = 1,NRKSTP
        RKTIM(IRKSTP) = RKTMC(IRKSTP)*TSTEP
      ENDDO
      DO IRKSTP = 1,NRKSTP-1
        RKDFC(IRKSTP) = RKTMC(IRKSTP+1) - RKTMC(IRKSTP)
      ENDDO
      RKDFC(NRKSTP) = ONE - RKTMC(NRKSTP-1)
      DO IRKSTP = 1,NRKSTP
        RKDIF(IRKSTP) = RKDFC(IRKSTP)*TSTEP
      ENDDO
      IRKSTP = 1

C     =========================================================================

C     INITIALISE ERK ERROR ARRAYS
C     ---------------------------
      DO JCELL = 1,NCELL

        DERR(JCELL) = ZERO
        UERR(1,JCELL) = ZERO
        UERR(2,JCELL) = ZERO
        UERR(3,JCELL) = ZERO
        EERR(JCELL) = ZERO

        DO JSPEC = 1,NSPEC
          YERR(JSPEC,JCELL) = ZERO
        ENDDO

      ENDDO

C     =========================================================================


      RETURN
      END
