      FUNCTION ERFINV(ARGMNT)
 
C     *************************************************************************
C
C     ERFINV
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-SEP-2018:  CREATED
C
C     DESCRIPTION
C     -----------
C     COMPUTES THE INVERSE ERROR FUNCTION
C     REF: ABRAMOWITZ AND STEGUN p299 sec 7.1.26
C     AHEM: WIKIPEDIA...
C
C     *************************************************************************


C     PARAMETERS
C     ==========
      DOUBLE PRECISION ONE,FOUR,HALF
      PARAMETER(ONE = 1.0D0, FOUR = 4.0D0, HALF = 5.0D-1)

      INTEGER NCOEFF,NCOFM1
      PARAMETER(NCOEFF = 5, NCOFM1 = NCOEFF-1)


C     FUNCTION
C     ========
      DOUBLE PRECISION ERFINV


C     ARGUMENT
C     ========
      DOUBLE PRECISION ARGMNT


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION CCOEFF(NCOEFF)
      DOUBLE PRECISION PCOEFF
      DOUBLE PRECISION ETOTAL,TVALUE
      DOUBLE PRECISION PI
      INTEGER ICOEFF


C     BEGIN
C     =====

C     =========================================================================

C     PI
      PI = FOUR*ATAN(ONE)

C     SET THE COEFFICIENTS
      PCOEFF = HALF*SQRT(PI)
      CCOEFF(1) = 1.0D0/1.2D1
      CCOEFF(2) = 7.0D0/4.80D2
      CCOEFF(3) = 1.27D2/4.0320D4
      CCOEFF(4) = 4.369D3/5.806080D6
      CCOEFF(5) = 3.4807D4/1.82476800D8

C     EVALUATE ERROR FUNCTION
      TVALUE = PI*ARGMNT*ARGMNT

      ETOTAL = CCOEFF(NCOEFF)*TVALUE
      DO ICOEFF = NCOFM1,1,-1
        ETOTAL = CCOEFF(ICOEFF) + ETOTAL*TVALUE
      ENDDO
      ETOTAL = ETOTAL + ONE

      ERFINV = ETOTAL*PCOEFF*ARGMNT

C     =========================================================================


      RETURN
      END
