      SUBROUTINE FLAMIN 
 
C     *************************************************************************
C
C     FLAMIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     28-DEC-2003:  CREATED FOR SENGA
C     15-JAN-2011:  RSC UPDATED FOR HAMISH

C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SETS INITIAL THERMOCHEMICAL FIELD
C     1D LAMINAR FLAME PROFILE (LEFT OR RIGHT FACING)
C     SPECIAL FOR MULTI-STEP CHEMISTRY
C     SOLUTION READ FROM FILE
C     
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     ESTIMATED FLAME LOCATION AND THICKNESS
      DOUBLE PRECISION XLOCAT,XTHICK,YLOCAT,YTHICK,ZLOCAT,ZTHICK
      PARAMETER(XLOCAT = 5.0D-3, XTHICK = 5.0D-4)
      PARAMETER(YLOCAT = 5.0D-6, YTHICK = 1.0D-5)
      PARAMETER(ZLOCAT = 5.0D-6, ZTHICK = 1.0D-5)

C     SOLUTION OFFSET
      INTEGER INDOFF
      PARAMETER(INDOFF =  300)

      INTEGER NCLFMX
      PARAMETER(NCLFMX = 1000) 
      INTEGER NCDMPF
      PARAMETER(NCDMPF = 4)


C     FUNCTION
C     ========
      DOUBLE PRECISION ERFUNC
      EXTERNAL ERFUNC


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YRUNF(NSPCMX,NCLFMX)
      DOUBLE PRECISION URUNF(NCMPMX,NCLFMX)
      DOUBLE PRECISION XCENTF(NCLFMX),YCENTF(NCLFMX),ZCENTF(NCLFMX)
      DOUBLE PRECISION SRUNF(NCLFMX),DRUNF(NCLFMX),ERUNF(NCLFMX)
      DOUBLE PRECISION HCSIZF(0:NLEVMX)
      DOUBLE PRECISION XGPHYF,YGPHYF,ZGPHYF
      DOUBLE PRECISION ETIMEF,TSTEPF,ERROLF,ERRLDF

      DOUBLE PRECISION YEVAL(NSPCMX)
      DOUBLE PRECISION UEVAL(NCMPMX)
      DOUBLE PRECISION DEVAL,EEVAL,TEVAL

      INTEGER NLEVEF(NCLFMX)
      INTEGER IRPROF,NRPROF
      INTEGER NBLEVF,NLEVLF,NCELLF,NSPECF
      INTEGER INDX1D

      INTEGER MLEVEL
      INTEGER JCELL,JSPEC
      CHARACTER*80 FNDMPF


C     BEGIN
C     =====

C     =========================================================================

C     SPECIFY INITIAL THERMOCHEMICAL FIELD HERE
C     =========================================

C     READ THE 1D FLAME SOLUTION DUMP FILE
      FNDMPF = 'dmpi_flamin.dat'

C     SELECT DUMP FORMAT
      IF(NDIFMT.EQ.0)THEN

C       UNFORMATTED DUMP INPUT

C       READ THE DATA FROM DUMP INPUT FILE 1
C       NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
        OPEN(UNIT=NCDMPF,FILE=FNDMPF,STATUS='OLD',
     +       FORM='UNFORMATTED')
        REWIND(NCDMPF)

        READ(NCDMPF)IRPROF,NRPROF
        READ(NCDMPF)XGPHYF,YGPHYF,ZGPHYF
        READ(NCDMPF)HCSIZF(0),NBLEVF,NLEVLF,NCELLF,NSPECF
        READ(NCDMPF)ETIMEF,TSTEPF,ERROLF,ERRLDF
        READ(NCDMPF)(NLEVEF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(XCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(YCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(ZCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(SRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(DRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(ERUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(URUNF(1,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(URUNF(2,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(URUNF(3,JCELL),JCELL=1,NCELLF)
        DO JSPEC = 1,NSPECF
          READ(NCDMPF)(YRUNF(JSPEC,JCELL),JCELL=1,NCELLF)
        ENDDO

        CLOSE(NCDMPF)

      ELSE

C       FORMATTED DUMP INPUT

C       READ THE DATA FROM DUMP INPUT FILE 1
C       NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
        OPEN(UNIT=NCDMPF,FILE=FNDMPF,STATUS='OLD',
     +       FORM='FORMATTED')
        REWIND(NCDMPF)

        READ(NCDMPF,*)IRPROF,NRPROF
        READ(NCDMPF,*)XGPHYF,YGPHYF,ZGPHYF
        READ(NCDMPF,*)HCSIZF(0),NBLEVF,NLEVLF,NCELLF,NSPECF
        READ(NCDMPF,*)ETIMEF,TSTEPF,ERROLF,ERRLDF
        READ(NCDMPF,*)(NLEVEF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(XCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(YCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(ZCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(SRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(DRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(ERUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(URUNF(1,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(URUNF(2,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(URUNF(3,JCELL),JCELL=1,NCELLF)
        DO JSPEC = 1,NSPEC
          READ(NCDMPF,*)(YRUNF(JSPEC,JCELL),JCELL=1,NCELLF)
        ENDDO

        CLOSE(NCDMPF)

      ENDIF

C     =========================================================================

C     USE 1D FLAME SOLUTION TO SET CELL VALUES

C     WITH OFFSET OF SOLUTION (TO THE LEFT)

      DO JCELL =  1, NCELL

C       INDEX MAPPING
        MLEVEL = NLEVEL(JCELL)
        INDX1D = 1 +  NINT(XCENTR(JCELL)/HCSIZE(MLEVEL) - HALF)

C       OFFSET
        INDX1D = INDX1D + INDOFF
        INDX1D = MAX(INDX1D,1)
        INDX1D = MIN(INDX1D,NCELL)

C       SET VALUES
        DEVAL = DRUNF(INDX1D)
        UEVAL(1) = URUNF(1,INDX1D)
        UEVAL(2) = URUNF(2,INDX1D)
        UEVAL(3) = URUNF(3,INDX1D)
        EEVAL = ERUNF(INDX1D)
        DO JSPEC = 1,NSPEC
          YEVAL(JSPEC)  = YRUNF(JSPEC,INDX1D)
        ENDDO

C       SET TEMPERATURE
        ICELL = JCELL
        TRUN(ICELL) = TRIN
        CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C       SET VALUES
        DRUN(JCELL) = DEVAL
        URUN(1,JCELL) = UEVAL(1)
        URUN(2,JCELL) = UEVAL(2)
        URUN(3,JCELL) = UEVAL(3)
        TRUN(JCELL) = TEVAL
        DO JSPEC = 1,NSPEC
          YRUN(JSPEC,JCELL)  = YEVAL(JSPEC)
        ENDDO

      ENDDO

C     =========================================================================


      RETURN
      END
