      SUBROUTINE PARDOM
 
C     *************************************************************************
C
C     PARDOM
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     CARRIES OUT INITIAL PARALLEL DOMAIN DECOMPOSITION
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION CELTOL,PWRTOL,PARTOL
      PARAMETER(CELTOL = 1.0D-10, PWRTOL = 1.0D-5, PARTOL = 1.0D-6)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION HCINIT,RYCELL,RZCELL,RLEVEL
      DOUBLE PRECISION XLOC,YLOC,ZLOC
      DOUBLE PRECISION XLIM,YLIM,ZLIM
      DOUBLE PRECISION CPPLOC,CCLOWR,CCUPPR
      INTEGER NMCPAR(NINDMX,NSIZMX)
      INTEGER MCRUN(NINDMX)
      INTEGER MX,MY,MZ
      INTEGER NXCELL,NYCELL,NZCELL,NXYZMX
      INTEGER ICLOWR,ICUPPR
      INTEGER INCNEW,INCODE,JNCODE,INCPAR,NNCPAR,ICHILD
      INTEGER ILEVEL,MLEVEL,MEXTRA
      CHARACTER*1 CHATTR
      LOGICAL FLXOUT,FLYOUT,FLZOUT,FLPART

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 0

C     =========================================================================

C     MAX LEVEL IN OCTREE
C     -------------------
      NLEVL = NLEVRQ

C     =========================================================================

C     NUMBER OF DIMENSIONS
C     --------------------
      NDIMS = NDIMAX
      IF(ZGDLEN.LT.CELTOL)THEN
        NDIMS = 2
        IF(YGDLEN.LT.CELTOL)THEN
          NDIMS = 1
        ENDIF
      ENDIF
      NOFSP1 = 2**NDIMS
      NOFSET = NOFSP1 - 1
      DREFAV = ONE/REAL(NOFSP1)

C     =========================================================================

C     NUMBER OF COMPONENTS AND FACES
C     ------------------------------
      NCOMP = NCMPMX
      NFACE = NFACMX

C     =========================================================================

C     NUMBER OF SUB-FACES
C     -------------------
      NFSUB = 2**(NDIMS-1)

C     =========================================================================

C     NUMBER OF SECTORS
C     -----------------
      NSCTR = NFACMX

C     =========================================================================

C     INITIAL MESH
C     ------------
C     INITIAL MESH SPACING
      HCINIT = XGDLEN/REAL(NXGREQ)
      NXCELL = NXGREQ

C     NUMBER OF CELLS IN Y AND Z DIRECTIONS
      RYCELL = YGDLEN/HCINIT
      NYCELL = NINT(RYCELL)
      IF(ABS(RYCELL-REAL(NYCELL)).GT.CELTOL)NYCELL = 1 + INT(RYCELL)
      RZCELL = ZGDLEN/HCINIT
      NZCELL = NINT(RZCELL)
      IF(ABS(RZCELL-REAL(NZCELL)).GT.CELTOL)NZCELL = 1 + INT(RZCELL)

C     MAX NO OF CELLS IN ANY DIRECTION
      NXYZMX = MAX(NXCELL,NYCELL,NZCELL)

C     FIND THE NEXT HIGHEST INTEGER THAT IS A POWER OF TWO
C     EQUAL TO THE LEVEL OF THE INITIAL MESH
      RLEVEL = LOG(REAL(NXYZMX))/RLNTWO
      ILEVEL = NINT(RLEVEL)
      IF(ABS(RLEVEL-REAL(ILEVEL)).GT.PWRTOL)ILEVEL = 1 + INT(RLEVEL)
      RLEVEL = REAL(ILEVEL)
      NLVHWM = ILEVEL

C     SET THE BASE LEVEL
      NBLEVL = MIN(NLVHWM,NLEVBQ)
CC     GVN
C      IF(NLVHWM.LT.NLEVBQ) NLVHWM = NLEVBQ
C      NBLEVL = NLVHWM

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WRITE(6,*)'LEVELS:',NLVHWM,NBLEVL,NLEVBQ
      ENDIF

C     COMPUTE THE CELL SIZE AT EACH LEVEL
      HCSIZE(0) = HCINIT*EXP(RLEVEL*RLNTWO)
      OVSIZE(0) = ONE/HCSIZE(0)
      DO ILEVEL = 1,NLEVL
        HCSIZE(ILEVEL) = HALF*HCSIZE(ILEVEL-1)
        OVSIZE(ILEVEL) = ONE/HCSIZE(ILEVEL)
      ENDDO

C     PHYSICAL BOUNDS OF GLOBAL COMPUTATIONAL DOMAIN
      XGPHYL = ZERO
      XGPHYR = REAL(NXCELL)*HCINIT
      YGPHYL = ZERO
      YGPHYR = REAL(NYCELL)*HCINIT
      ZGPHYL = ZERO
      ZGPHYR = REAL(NZCELL)*HCINIT

C     MIN AND MAX CELL NUMBERS IN EACH DIRECTION
      DO ILEVEL = 1,NLEVL
        MPMIN(1,ILEVEL) = 0
        MPMAX(1,ILEVEL) = NINT(XGPHYR*OVSIZE(ILEVEL)) - 1
        MPMAX(1,ILEVEL) = MAX(MPMAX(1,ILEVEL),0)
        MPMIN(2,ILEVEL) = 0
        MPMAX(2,ILEVEL) = NINT(YGPHYR*OVSIZE(ILEVEL)) - 1
        MPMAX(2,ILEVEL) = MAX(MPMAX(2,ILEVEL),0)
        MPMIN(3,ILEVEL) = 0
        MPMAX(3,ILEVEL) = NINT(ZGPHYR*OVSIZE(ILEVEL)) - 1
        MPMAX(3,ILEVEL) = MAX(MPMAX(3,ILEVEL),0)
      ENDDO

C     CHECK DIVISIBILITY
      ICLOWR = NBLEVL
      IF(NDIMS.EQ.1)THEN
        DO ILEVEL = ICLOWR,NLVHWM-1
          XLIM = XGPHYR - REAL(MPMAX(1,ILEVEL)+1)*HCSIZE(ILEVEL)
          IF(ABS(XLIM).GT.CELTOL)NBLEVL = NBLEVL + 1
        ENDDO
      ENDIF
      IF(NDIMS.EQ.2)THEN
        DO ILEVEL = ICLOWR,NLVHWM-1
          XLIM = XGPHYR - REAL(MPMAX(1,ILEVEL)+1)*HCSIZE(ILEVEL)
          YLIM = YGPHYR - REAL(MPMAX(2,ILEVEL)+1)*HCSIZE(ILEVEL)
          IF((ABS(XLIM).GT.CELTOL)
     +     .OR.(ABS(YLIM).GT.CELTOL))NBLEVL = NBLEVL + 1
        ENDDO
      ENDIF
      IF(NDIMS.EQ.3)THEN
        DO ILEVEL = ICLOWR,NLVHWM-1
          XLIM = XGPHYR - REAL(MPMAX(1,ILEVEL)+1)*HCSIZE(ILEVEL)
          YLIM = YGPHYR - REAL(MPMAX(2,ILEVEL)+1)*HCSIZE(ILEVEL)
          ZLIM = ZGPHYR - REAL(MPMAX(3,ILEVEL)+1)*HCSIZE(ILEVEL)
          IF((ABS(XLIM).GT.CELTOL)
     +     .OR.(ABS(YLIM).GT.CELTOL)
     +       .OR.(ABS(ZLIM).GT.CELTOL))NBLEVL = NBLEVL + 1
        ENDDO
      ENDIF

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        DO ILEVEL = 0,NLEVL
          WRITE(6,'(I5,2(1PE12.4))')ILEVEL,HCSIZE(ILEVEL),OVSIZE(ILEVEL)
        ENDDO
        DO ILEVEL = 0,NLEVL
         WRITE(6,'(7I10)')ILEVEL,
     +                    MPMIN(1,ILEVEL),MPMAX(1,ILEVEL),
     +                    MPMIN(2,ILEVEL),MPMAX(2,ILEVEL),
     +                    MPMIN(3,ILEVEL),MPMAX(3,ILEVEL)
        ENDDO
        DO ILEVEL = 0,NLEVL
         WRITE(6,'(I10,3(1PE12.4))')ILEVEL,
     +                    XGPHYR-REAL(MPMAX(1,ILEVEL)+1)*HCSIZE(ILEVEL),
     +                    YGPHYR-REAL(MPMAX(2,ILEVEL)+1)*HCSIZE(ILEVEL),
     +                    ZGPHYR-REAL(MPMAX(3,ILEVEL)+1)*HCSIZE(ILEVEL)
        ENDDO
      ENDIF

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WRITE(6,'(5I7)')NXCELL,NYCELL,NZCELL,NXYZMX,NBLEVL
        WRITE(6,'(6(1PE12.4))')XGPHYL,XGPHYR,YGPHYL,YGPHYR,ZGPHYL,ZGPHYR
      ENDIF

C     =========================================================================

C     INITIALISE M-CODE MANIPULATION
C     ------------------------------
      CALL MCINIT

C     =========================================================================

C     INITIAL OCTREE
C     --------------
C     PLANT A LOCAL COPY OF THE OCTREE ON EACH PROCESSOR
      CALL OCINIT(NOCPAR,NOCHLD,COCODE,NOCTMX,NOCHWM)

C     =========================================================================

C     CREATE INITIAL M-CODE
C     ---------------------
C     M-CODE IS ZERO AT LEVEL ZERO
      CALL MCCNEW(MCRUN)
      NNCODE = 1

C     STORE IN MAIN M-CODE ARRAY
      CALL MCSTOR(MCRUN,NMCODE,NSIZMX,NNCODE)

C     =========================================================================

C     GROW THE LOCAL OCTREE
C     ---------------------
      ILEVEL = 1
      FLPART = .FALSE.
 
C     ACTION AT EACH LEVEL
1000  CONTINUE

C       =======================================================================

C       ROLL OVER CURRENT M-CODE ARRAY INTO THE PARENT ARRAY
        CALL MCMTOR(NMCODE,NSIZMX,NMCPAR,NSIZMX,1,NNCODE)
        NNCPAR = NNCODE

C       =======================================================================

C       IN/OUT STEP
C       -----------

C       INITIALISE CHILD M-CODE COUNTER
        INCODE = 0

C       RUN THROUGH ALL PARENT M-CODES AT THIS LEVEL ON THIS PROCESSOR
        DO INCPAR = 1, NNCPAR

C         EXTRACT THE NEXT PARENT M-CODE
          CALL MCRTRV(MCRUN,NMCPAR,NSIZMX,INCPAR)

C         SET THE CHILD COUNTER
          ICHILD = 0

C         EXTEND THE PARENT M-CODE
          CALL MCHIRE(MCRUN,ICHILD)
          
C         RUN THROUGH NEW CHILD M-CODES
2000      CONTINUE

C           SET UP THE IN/OUT TEST
            CALL MCCXYZ(MCRUN,ILEVEL,MX,MY,MZ)
            XLOC = REAL(MX)*HCSIZE(ILEVEL)
            YLOC = REAL(MY)*HCSIZE(ILEVEL)
            ZLOC = REAL(MZ)*HCSIZE(ILEVEL)
            XLIM = MAX(XGPHYR,HCSIZE(ILEVEL))
            YLIM = MAX(YGPHYR,HCSIZE(ILEVEL))
            ZLIM = MAX(ZGPHYR,HCSIZE(ILEVEL))
            FLXOUT = (XLOC.GT.XLIM-CELTOL)
            FLYOUT = (YLOC.GT.YLIM-CELTOL)
            FLZOUT = (ZLOC.GT.ZLIM-CELTOL)

            IF(FLXOUT.OR.FLYOUT.OR.FLZOUT)THEN

C             CELL IS OUT - SET ATTRIBUTE
              CHATTR = CHOUTC
              CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCRUN,CHATTR)

            ELSE

C             CELL IS IN - RECORD THE M-CODE
              INCODE = INCODE + 1
              CALL MCSTOR(MCRUN,NMCODE,NSIZMX,INCODE)

            ENDIF

C           CHECK CHILD COUNTER
            IF(ICHILD.LT.7)THEN

C             INCREMENT M-CODE AND REPEAT
              CALL MCINCL(MCRUN,ILEVEL)
              ICHILD = ICHILD + 1
              GOTO 2000

            ENDIF
C         END OF LOOP 2000: RUN THROUGH NEW CHILD M-CODES

        ENDDO
C       RUN THROUGH ALL PARENT M-CODES
        NNCODE = INCODE

C       =======================================================================

C       PARTITION STEP
C       --------------

        IF(FLPART)THEN

C         ALREADY PARTITIONED: DO LOAD-BALANCING ON M-CODES ONLY
          CALL PARBIN

C         REBUILD THE PARTITION INTERVAL TABLE
          CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NNCODE)
          CALL PTABLE(MCRUN)
            
C         REBUILD THE LOCAL OCTREE
          CALL RETREE
                        
        ELSE

C         NOT PARTITIONED: CHECK NUMBERS OF M-CODES WRT PROCESSORS
          IF(NNCODE.GE.NPROC)THEN

C           PARTITION REQUIRED
C           DECIDE UPPER+LOWER M-CODES PER PROCESSOR
            CPPLOC = REAL(NNCODE)/REAL(NPROC)
C           ALWAYS ROUND UP LOWER LIMIT
            CCLOWR = REAL(IPROC)*CPPLOC
            ICLOWR = NINT(CCLOWR)
            IF(ABS(CCLOWR-REAL(ICLOWR)).GT.PARTOL)THEN
              ICLOWR = 1 + INT(CCLOWR)
            ELSE
              ICLOWR = 1 + ICLOWR
            ENDIF
C           ROUND DOWN UPPER LIMIT UNLESS IT IS ALREADY AN INTEGER
            CCUPPR = REAL(IPROC+1)*CPPLOC
            ICUPPR = NINT(CCUPPR)
            IF(ABS(CCUPPR-REAL(ICUPPR)).GT.PARTOL)ICUPPR = INT(CCUPPR)
C           HIGHEST PROCESSOR
            IF(IPROC.EQ.NPRM1)ICUPPR = NNCODE

C           RUN THROUGH THE PRE-STORED M-CODES

C           REMOTE: FETCH THE M-CODE AND SET ATTRIBUTE
            CHATTR = CHPARL
            DO INCODE = 1,ICLOWR-1
              CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)
              CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCRUN,CHATTR)
            ENDDO
            DO INCODE = ICUPPR+1,NNCODE
              CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)
              CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCRUN,CHATTR)
            ENDDO

C           SAVE REMAINING M-CODES
            INCNEW = 0
            DO INCODE = ICLOWR,ICUPPR
              INCNEW = INCNEW + 1
              CALL MCSCPY(NMCODE,NSIZMX,INCODE,INCNEW)
            ENDDO
            NNCODE = INCNEW

C           BUILD THE PARTITION INTERVAL TABLE
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NNCODE)
            CALL PTABLE(MCRUN)
 
C           RESET PARTITION FLAG
            FLPART = .TRUE.

          ENDIF

        ENDIF

C       =======================================================================

C       GROWTH STEP
C       -----------

C       CHECK FOR FINAL PASS
C        IF(ILEVEL.LT.NLVHWM)THEN
        IF(ILEVEL.LT.NBLEVL)THEN

C         NOT FINAL PASS: MAKE A NEW OCTREE LEVEL

C         SET DEFAULT ATTRIBUTES
          CHATTR = CHUNST

C         RUN THROUGH ALL SURVIVING CHILDREN
          DO INCODE = 1,NNCODE

C           FETCH THE M-CODE
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)

C           MAKE A NEW OCTANT
            CALL OCTADD(NOCPAR,NOCHLD,COCODE,NOCTMX,NOCHWM,MCRUN,CHATTR)

          ENDDO

C         INCREMENT LEVEL AND DO IT AGAIN
          ILEVEL = ILEVEL + 1
          GOTO 1000

        ELSE

C         FINAL PASS: MAKE CELLS

C         RUN THROUGH ALL SURVIVING CHILDREN
          DO INCODE = 1,NNCODE

C           FETCH THE M-CODE
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)

C           MAKE A NEW CELL
            CALL OCLEAF(NOCHLD,COCODE,NOCTMX,MCRUN,INCODE,CHATTR)
            NPARNT(INCODE) = INCODE

          ENDDO
C         RUN THROUGH ALL SURVIVING CHILDREN
          NCELL = NNCODE

        ENDIF
C       CHECK FOR FINAL PASS

C     END OF LOOP 1000: ACTION AT EACH LEVEL

C     =========================================================================

C     INITIAL GLOBAL REFINEMENT STEP
C      IS NOT REALISED EVER NOW (GVN)
      IF(NLVHWM.GT.NBLEVL)THEN

C       LOOP OVER LEVELS
        DO ILEVEL = NBLEVL,NLVHWM-1

C         SUBDIVIDE EVERY CELL
          JNCODE = NNCODE
          DO INCODE = 1, NNCODE

C           RETRIEVE THE M-CODE
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)

C           INCREMENT THE LEVEL
            MEXTRA = 0
            CALL MCHIRE(MCRUN,MEXTRA)
            CALL MCGLVL(MCRUN,MLEVEL)

C           STORE THE UPDATED BASE M-CODE
            CALL MCSTOR(MCRUN,NMCODE,NSIZMX,INCODE)

C           MAKE THE NEW CELLS
            DO ICHILD = 1, NOFSET
              JNCODE = JNCODE + 1
              CALL MCINCR(MCRUN)
              CALL MCSTOR(MCRUN,NMCODE,NSIZMX,JNCODE)
            ENDDO

          ENDDO
C         SUBDIVIDE EVERY CELL
          NNCODE = JNCODE
          NCELL = NNCODE

C         SORT THE M-CODES
          CALL SORTMC(NMCODE,NSIZMX,NNCODE)

C         UPDATE THE PARTITION INTERVAL TABLE
          CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NNCODE)
          CALL PTABLE(MCRUN)

C         REBUILD THE OCTREE
          CALL RETREE
          CALL RELEAF

        ENDDO
C       LOOP OVER LEVELS

      ENDIF
C     INITIAL REFINEMENT STEP

C     =========================================================================

C     COMPUTE THE CELL CENTRE COORDINATES
C     AND STORE THE LEVEL
      DO INCODE = 1, NNCODE

        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)
        CALL MCGLVL(MCRUN,ILEVEL)
        CALL MCCXYZ(MCRUN,ILEVEL,MX,MY,MZ)

        XCENTR(INCODE) = (REAL(MX)+HALF)*HCSIZE(ILEVEL)
        YCENTR(INCODE) = (REAL(MY)+HALF)*HCSIZE(ILEVEL)
        ZCENTR(INCODE) = (REAL(MZ)+HALF)*HCSIZE(ILEVEL)
        NLEVEL(INCODE) = ILEVEL

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)WRITE(6,'(4I5,3(1PE15.7))')INCODE,MX,MY,MZ,
C     +                    XCENTR(INCODE),YCENTR(INCODE),ZCENTR(INCODE)

      ENDDO

C     =========================================================================


      RETURN
      END
