      SUBROUTINE REFDIN

C     *************************************************************************
C
C     REFDIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     15-AUG-2017:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES DATA FOR INITIAL REFINEMENT/DEREFINEMENT
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     CELL LEVEL FUNCTION PARAMETERS
      DOUBLE PRECISION XCENTF,YCENTF,ZCENTF
      DOUBLE PRECISION RLOCAT,RTHICK
      DOUBLE PRECISION CLBASE,CLPEAK
      PARAMETER(CLBASE = 7.0D0, CLPEAK = 1.0D1)
      PARAMETER(XCENTF = 5.0D-3)
      PARAMETER(YCENTF = 5.0D-3)
      PARAMETER(ZCENTF = 5.0D-6)
C      PARAMETER(RLOCAT = 4.2D-3, RTHICK = 3.0D-4)
      PARAMETER(RLOCAT = 4.1D-3, RTHICK = 3.0D-4)
      INTEGER LCLMAX
      PARAMETER(LCLMAX = 9)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION XCOORD,YCOORD,ZCOORD
      DOUBLE PRECISION RCOORD
      DOUBLE PRECISION ARGMNT
      DOUBLE PRECISION CLEVEL
      INTEGER MCRUN(NINDMX)
      INTEGER MLEVEL,MX,MY,MZ
      INTEGER LCLEVL
      INTEGER JCELL
      INTEGER ICDREF,ICREFN

C     DIAGNOSTICS
      INTEGER LPROC

C     DIAGNOSTICS
      LPROC = 0

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In REFDIN'

C     =========================================================================

C     BEGIN
C     =====

C     =========================================================================

C     ZERO THE COUNTERS
      N2DREF = 0
      N2REFN = 0
      ICDREF = 0
      ICREFN = 0

C     =========================================================================

C     LOOP OVER ALL CELLS
C     -------------------
      DO JCELL = 1, NCELL

C       INITIALISE CELL REFINEMENT STATUS CODE
        NRSTAT(JCELL) = NRNULL

C       INITIALISE OLD CELL INDEX LIST
        NCOCEL(JCELL) = JCELL

C       =======================================================================

C       LOCAL CELL-CENTRE COORDINATES
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
        CALL MCGLVL(MCRUN,MLEVEL)
        CALL MCCXYZ(MCRUN,MLEVEL,MX,MY,MZ)
        XCOORD = (REAL(MX) + HALF)*HCSIZE(MLEVEL)
        YCOORD = (REAL(MY) + HALF)*HCSIZE(MLEVEL)
        ZCOORD = (REAL(MZ) + HALF)*HCSIZE(MLEVEL)

CC       SIMPLE 1D GAUSSIAN DISTRIBUTION OF CELL LEVELS
C        ARGMNT = (XCOORD-XLOCAT)/XTHICK
C        CLEVEL = CLBASE + CLPEAK*EXP(-ARGMNT*ARGMNT)
C        LCLEVL = INT(CLEVEL)

C       ANNULAR 1D GAUSSIAN DISTRIBUTION OF CELL LEVELS
        XCOORD = XCOORD - XCENTF
        YCOORD = YCOORD - YCENTF
        ZCOORD = ZCOORD - ZCENTF
        RCOORD = XCOORD*XCOORD + YCOORD*YCOORD
        RCOORD = SQRT(RCOORD)
        ARGMNT = (RCOORD - RLOCAT)/RTHICK
        CLEVEL = CLBASE + CLPEAK*EXP(-ARGMNT*ARGMNT)
        LCLEVL = INT(CLEVEL)
        LCLEVL = MIN(LCLEVL,LCLMAX)

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'(5I10)')JCELL,MLEVEL,LCLEVL,NMLMIN,NMLMAX
        ENDIF

C       =======================================================================

C       EVALUATE DEREFINEMENT CRITERIA
C       ------------------------------
        IF(MLEVEL.GT.NMLMIN)THEN

          IF(LCLEVL.LT.MLEVEL)THEN
            ICDREF = ICDREF + 1
            NINDDR(ICDREF) = JCELL
          ENDIF

        ENDIF

C       =======================================================================

C       EVALUATE REFINEMENT CRITERIA
C       ----------------------------
        IF(MLEVEL.LT.NMLMAX)THEN

          IF(LCLEVL.GT.MLEVEL)THEN
            ICREFN = ICREFN + 1
            NINDRR(ICREFN) = JCELL
          ENDIF

        ENDIF

C       =======================================================================

      ENDDO
C     END OF LOOP OVER CELLS

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'REFDIN: end of loop over cells'


C     FINAL NUMBERS TO DE/REFINE
      N2DREF = ICDREF
      N2REFN = ICREFN

C     =========================================================================

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WRITE(6,'(2I8)')N2DREF,N2REFN
        DO ICDREF = 1, N2DREF
          JCELL = NINDDR(ICDREF)
          WRITE(6,'(3I9)')ICDREF,JCELL,NRSTAT(JCELL)
        ENDDO
        WRITE(6,*)
        DO ICREFN = 1, N2REFN
          JCELL = NINDRR(ICREFN)
          WRITE(6,'(3I9)')ICREFN,JCELL,NRSTAT(JCELL)
        ENDDO
      ENDIF
C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'End of REFDIN'


      RETURN
      END
