      SUBROUTINE RKLAST
 
C     *************************************************************************
C
C     RKLAST
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     15-MAR-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES FINAL SOLUTION VALUES IN ERK SCHEME
C     BY DOING A LINEAR COMBINATION OF LEFT- AND RIGHT-HAND SIDES
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION RKLHSC,RKERRC
      DOUBLE PRECISION FORNOW,OCSIZE
      INTEGER JCELL,JSPEC,ILEVL

CC     DIAGNOSTICS
C      INTEGER LPROC

C     =========================================================================

C     BEGIN
C     =====

CC     DIAGNOSTICS
C      LPROC = 0

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In RKLAST:',ITIME,IRKSTP

C     =========================================================================

C     FINAL ERK SUBSTEP
C     =================

C     -------------------------------------------------------------------------
C     NOTE: ALL ERK ERROR ARRAYS ARE INITIALISED TO ZERO IN SUBROUTINE ADAPTT
C     -------------------------------------------------------------------------

C     LOOP OVER ALL CELLS
C     -------------------
      DO JCELL = 1, NCELL

C       =======================================================================

C       SCALING
C       -------
        ILEVL = NLEVEL(JCELL)
        OCSIZE = OVSIZE(ILEVL)
        RKLHSC = RKLHS(NRKSTP)*OCSIZE
        RKERRC = RKERR(NRKSTP)*OCSIZE

C       =======================================================================

C       CONTINUITY
C       ----------
        DERR(JCELL) = DERR(JCELL) + RKERRC*DRHS(JCELL)

        FORNOW = RKSAV(NRKSTP)*DSAV(JCELL) + RKRUN(NRKSTP)*DRUN(JCELL)
        DRUN(JCELL) = FORNOW + RKLHSC*DRHS(JCELL)
        DRHS(JCELL) = DRUN(JCELL)
        DSAV(JCELL) = DRUN(JCELL)

C       -----------------------------------------------------------------------

C       U-MOMENTUM
C       ----------
        UERR(1,JCELL) = UERR(1,JCELL) + RKERRC*URHS(1,JCELL)

        FORNOW = RKSAV(NRKSTP)*USAV(1,JCELL)+RKRUN(NRKSTP)*URUN(1,JCELL)
        URUN(1,JCELL) = FORNOW + RKLHSC*URHS(1,JCELL)
        URHS(1,JCELL) = URUN(1,JCELL)
        USAV(1,JCELL) = URUN(1,JCELL)

C       -----------------------------------------------------------------------

C       V-MOMENTUM
C       ----------
        UERR(2,JCELL) = UERR(2,JCELL) + RKERRC*URHS(2,JCELL)

        FORNOW = RKSAV(NRKSTP)*USAV(2,JCELL)+RKRUN(NRKSTP)*URUN(2,JCELL)
        URUN(2,JCELL) = FORNOW + RKLHSC*URHS(2,JCELL)
        URHS(2,JCELL) = URUN(2,JCELL)
        USAV(2,JCELL) = URUN(2,JCELL)

C       -----------------------------------------------------------------------

C       W-MOMENTUM
C       ----------
        UERR(3,JCELL) = UERR(3,JCELL) + RKERRC*URHS(3,JCELL)

        FORNOW = RKSAV(NRKSTP)*USAV(3,JCELL)+RKRUN(NRKSTP)*URUN(3,JCELL)
        URUN(3,JCELL) = FORNOW + RKLHSC*URHS(3,JCELL)
        URHS(3,JCELL) = URUN(3,JCELL)
        USAV(3,JCELL) = URUN(3,JCELL)

C       -----------------------------------------------------------------------

C       ENERGY
C       ------
        EERR(JCELL) = EERR(JCELL) + RKERRC*ERHS(JCELL)

        FORNOW = RKSAV(NRKSTP)*ESAV(JCELL) + RKRUN(NRKSTP)*ERUN(JCELL)
        ERUN(JCELL) = FORNOW + RKLHSC*ERHS(JCELL)
        ERHS(JCELL) = ERUN(JCELL)
        ESAV(JCELL) = ERUN(JCELL)

C       -----------------------------------------------------------------------

C       SPECIES MASS FRACTIONS
C       ----------------------
        DO JSPEC = 1,NSPEC

          YERR(JSPEC,JCELL) = YERR(JSPEC,JCELL)
     +                      + RKERRC*YRHS(JSPEC,JCELL)

          FORNOW = RKSAV(NRKSTP)*YSAV(JSPEC,JCELL)
     +           + RKRUN(NRKSTP)*YRUN(JSPEC,JCELL)
          YRUN(JSPEC,JCELL) = FORNOW
     +                      + RKLHSC*YRHS(JSPEC,JCELL)
          YRHS(JSPEC,JCELL) = YRUN(JSPEC,JCELL)
          YSAV(JSPEC,JCELL) = YRUN(JSPEC,JCELL)

        ENDDO

C       =======================================================================

      ENDDO
C     END OF LOOP OVER CELLS

C     =========================================================================


      RETURN
      END
