      SUBROUTINE RKSTEP
 
C     *************************************************************************
C
C     RKSTEP
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     15-MAR-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES INTERMEDIATE SOLUTION VALUES IN ERK SCHEME
C     BY DOING LINEAR COMBINATIONS OF LEFT- AND RIGHT-HAND SIDES
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION RKLHSC,RKRHSC,RKERRC
      DOUBLE PRECISION FORNOW,OCSIZE
      INTEGER JCELL,JSPEC,ILEVL

CC     DIAGNOSTICS
C      INTEGER LPROC

C     =========================================================================

C     BEGIN
C     =====

CC     DIAGNOSTICS
C      LPROC = 0

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In RKSTEP:',ITIME,IRKSTP

C     =========================================================================

C     ERK SUBSTEP
C     ===========

C     -------------------------------------------------------------------------
C     NOTE: ALL ERK ERROR ARRAYS ARE INITIALISED TO ZERO IN SUBROUTINE ADAPTT
C     -------------------------------------------------------------------------

C     LOOP OVER ALL CELLS
C     -------------------
      DO JCELL = 1, NCELL

C       =======================================================================

C       SCALING
C       -------
        ILEVL = NLEVEL(JCELL)
        OCSIZE = OVSIZE(ILEVL)
        RKLHSC = RKLHS(IRKSTP)*OCSIZE
        RKRHSC = RKRHS(IRKSTP)*OCSIZE
        RKERRC = RKERR(IRKSTP)*OCSIZE

C       =======================================================================

C       CONTINUITY
C       ----------
        DERR(JCELL) = DERR(JCELL) + RKERRC*DRHS(JCELL)

        FORNOW = RKSAV(IRKSTP)*DSAV(JCELL) + RKRUN(IRKSTP)*DRUN(JCELL)
        DRUN(JCELL) = FORNOW + RKLHSC*DRHS(JCELL)
        DRHS(JCELL) = FORNOW + RKRHSC*DRHS(JCELL)

C       -----------------------------------------------------------------------

C       U-MOMENTUM
C       ----------
        UERR(1,JCELL) = UERR(1,JCELL) + RKERRC*URHS(1,JCELL)

        FORNOW = RKSAV(IRKSTP)*USAV(1,JCELL)+RKRUN(IRKSTP)*URUN(1,JCELL)
        URUN(1,JCELL) = FORNOW + RKLHSC*URHS(1,JCELL)
        URHS(1,JCELL) = FORNOW + RKRHSC*URHS(1,JCELL)

C       -----------------------------------------------------------------------
            
C       V-MOMENTUM
C       ----------
        UERR(2,JCELL) = UERR(2,JCELL) + RKERRC*URHS(2,JCELL)

        FORNOW = RKSAV(IRKSTP)*USAV(2,JCELL)+RKRUN(IRKSTP)*URUN(2,JCELL)
        URUN(2,JCELL) = FORNOW + RKLHSC*URHS(2,JCELL)
        URHS(2,JCELL) = FORNOW + RKRHSC*URHS(2,JCELL)

C       -----------------------------------------------------------------------

C       W-MOMENTUM
C       ----------
        UERR(3,JCELL) = UERR(3,JCELL) + RKERRC*URHS(3,JCELL)

        FORNOW = RKSAV(IRKSTP)*USAV(3,JCELL)+RKRUN(IRKSTP)*URUN(3,JCELL)
        URUN(3,JCELL) = FORNOW + RKLHSC*URHS(3,JCELL)
        URHS(3,JCELL) = FORNOW + RKRHSC*URHS(3,JCELL)

C       -----------------------------------------------------------------------

C       ENERGY
C       ------
        EERR(JCELL) = EERR(JCELL) + RKERRC*ERHS(JCELL)

        FORNOW = RKSAV(IRKSTP)*ESAV(JCELL) + RKRUN(IRKSTP)*ERUN(JCELL)
        ERUN(JCELL) = FORNOW + RKLHSC*ERHS(JCELL)
        ERHS(JCELL) = FORNOW + RKRHSC*ERHS(JCELL)

C       -----------------------------------------------------------------------

C       SPECIES MASS FRACTIONS
C       ----------------------
        DO JSPEC = 1,NSPEC

          YERR(JSPEC,JCELL) = YERR(JSPEC,JCELL)
     +                      + RKERRC*YRHS(JSPEC,JCELL)

          FORNOW = RKSAV(IRKSTP)*YSAV(JSPEC,JCELL)
     +           + RKRUN(IRKSTP)*YRUN(JSPEC,JCELL)
          YRUN(JSPEC,JCELL) = FORNOW + RKLHSC*YRHS(JSPEC,JCELL)
          YRHS(JSPEC,JCELL) = FORNOW + RKRHSC*YRHS(JSPEC,JCELL)

        ENDDO

C       =======================================================================

      ENDDO
C     END OF LOOP OVER CELLS

C     =========================================================================


      RETURN
      END
