      SUBROUTINE STNCON
 
C     *************************************************************************
C
C     STNCON
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     18-AUG-2014:  CREATED
C     16-APR-2018:  RSC UPDATED FOR FULLY PERIODIC BOUNDARIES
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     STENCIL CONNECTIVITY FOR POLYNOMIAL RECONSTRUCTION
C     CENTRAL STENCIL ONLY
C     RENO SCHEME
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION TOLDOT
      PARAMETER(TOLDOT = 1.0D-6)

      INTEGER NHCTMX
      PARAMETER(NHCTMX = NREMAX/8)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION HALFSZ
      INTEGER NCLAYR(NLAYMX,NSIZMX)
      INTEGER NOLAYR(NSIZMX)
      INTEGER NLAYLO(NSIZMX)
      INTEGER NCNEED(NSIZMX)
      INTEGER NXCODE(NINDMX,NREMAX),NACODE(NINDMX,NREMAX)
      INTEGER NHCODE(NINDMX,NREMAX)
      INTEGER NRCFAC(NREMAX),NRCCOD(NREMAX)
      INTEGER NCENTC(NREMAX)
      INTEGER NLCODE(NINDMX,NTSTMX)
      INTEGER NHCHLD(0:7,NHCTMX)
      INTEGER NHCPAR(NHCTMX)
      INTEGER NCSTMP(NTSTMX),NCLTMP(NTSTMX),NCPTMP(NTSTMX)
      INTEGER NLCFAC(NTSTMX)
      INTEGER MCRUN(NINDMX),MCNXT(NINDMX),MCPER(NINDMX),MCTMP(NINDMX)
      INTEGER MPLOC(NCMPMX),MPNXT(NCMPMX)
      INTEGER NRQLOC(1:4)
      INTEGER NHCHWM
      INTEGER ICCELL,JCCELL,JCELL,KCELL,KCELLX,LCELL,LCELLX
      INTEGER JFACE,KFACE,KLINK,JDIRN,NSDIRN
      INTEGER JLEVEL,KLEVEL,KLEVLX,LLEVEL,LLEVLX,IGLOBC
      INTEGER KCELLH,KCELHH,KCELHX,KLEVLH,KLVLHX
      INTEGER LCELLH,LCELHH,LCELHX,LLEVLH,LLVLHX
      INTEGER JCENTC,JCSTAT
      INTEGER NLCELL,NXCELL,NACELL,NHCELL
      INTEGER JLCELL
      INTEGER JRCELL,JRCLOC,JRCOLD,JRFACE,JCFACE,JRNCEL,JRLIST
      INTEGER JBCVAL
      INTEGER JCSTEN,NSOFAR
      INTEGER ICPSTN,NOPSTN
      INTEGER NONEED,ICNEED,JCNEED
      INTEGER NSTART,NFINIS
      INTEGER NH2HCF
      INTEGER ILAYER,JOLAYR
      INTEGER IOFSET,JQUART,MOCTHI
      LOGICAL FLINLH(NREMAX)
      LOGICAL FLCONN,FLADST,FLFNHL,FLNEQC
      LOGICAL FLNIND,FLNIXL,FLNIXR,FLNIYL,FLNIYR,FLNIZL,FLNIZR
      CHARACTER*1 CHCODE(0:7,NOCTMX)
      CHARACTER*1 CFOUND,CFPERI,CFXTND,CFOUNH,CFXTNH

C     DIAGNOSTICS
      INTEGER LPROC
      INTEGER IFCODE(NFACMX)
      INTEGER ISCODE


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In STNCON'

C     =========================================================================

C     GLOBAL DATA
C     -----------
C     NTSTMX max number of cells in the temporary stencil
C     NLAYMX max number of cells in each temporary stencil layer
C     NSSTMX max number of cells in each sectoral stencil

C     RENO SCHEME
C     NSSTEN(NSSTMX,NSIZMX) index list for central stencil (only)
C     NOSSTN(NSIZMX) number of cells in central stencil (only)

C     NMCODE(NINDMX,NSIZMX) global list of M-codes

C     =========================================================================

C     LOCAL DATA
C     -----------
C     NCSTMP(NSIZMX) local index list for the temporary stencil
C     NCLAYR(NLAYMX,NSIZMX) index list for previous layer of the temp stencil
C     NOLAYR(NSIZMX) number of cells in the previous layer
C     NCNEED(NSIZMX) list of central cells with incomplete stencils

C     NLCODE(NINDMX,NREMAX) list of requested remote cell M-codes
C                           for the current central cell
C     NXCODE(NINDMX,NREMAX) list of requested remote cell M-codes
C                           for the current layer
C     NACODE(NINDMX,NREMAX) list of actual remote cell M-codes
C                           for the current layer
C     NLCELL number of requested remote cells for the current central cell
C     NXCELL number of requested remote cells for the current layer
C     NACELL number of actual remote cells for the current layer
C     NARLOC index list for actual remote cells in the current layer

C     =========================================================================

C     BUILD THE STENCILS FOR POLYNOMIAL RECONSTRUCTION
C     ------------------------------------------------

C     INITIALISE LAYER COUNTER
      ILAYER = 1

C     INITIALISE NUMBER OF CELLS WITH INCOMPLETE STENCILS
      NONEED = NCELL

C     RUN THROUGH ALL CELLS
      DO JCELL = 1, NCELL

C       INITIALISE THE CENTRAL STENCIL SIZE
        NOSSTN(JCELL) = 0

C       INITIALISE LIST OF CELLS WITH INCOMPLETE STENCILS
        NCNEED(JCELL) = JCELL

C       INITIALISE NUMBER OF CELLS IN THE PREVIOUS (ZEROTH) LAYER
        NOLAYR(JCELL) = 1

C       INITIALISE THE (ONLY) CELL ID IN THE PREVIOUS (ZEROTH) LAYER
        NCLAYR(1,JCELL) = JCELL

C       INITIALISE THE HALO-LAYER ID
        NLAYLO(JCELL) = 0

C       CONNECTIVITY
        DO JFACE =1, NFACE
          NXDOOR(JFACE,JCELL) = 0
        ENDDO
        NSCODE(JCELL) = NCLEAN
        JBCVAL = 0

      ENDDO

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        DO JCELL = 1, NCELL
C          CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
C          CALL MCGLVL(MCRUN,JLEVEL)
C          WRITE(6,'(2I6)')JCELL,JLEVEL
C        ENDDO
C      ENDIF

C     INITIALISE NUMBER OF FACE NEIGHBOUR HALO CELLS
      NRNCEL = 0

C     INITIALISE START AND FINISH INDICES FOR HALO CELLS
      NRCELL = 0
      NCELP1 = NCELL + 1
      NCELHW = NCELL

C     INITIALISE THE HALO-SO-FAR OCTREE
      CALL LHTREE(NHCODE,NREMAX,NRCELL,
     +            NHCPAR,NHCHLD,CHCODE,NHCTMX,NHCHWM)

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)CALL OCRITE(NOCPAR,NOCHLD,COCODE,NOCTMX,NOCHWM)

C     =========================================================================

C     LOOP OVER LAYERS
1000  CONTINUE

C       =======================================================================

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)WRITE(6,*)'Layer no.',ILAYER

C       INITIALISE REMOTE CELL COUNTERS
        NXCELL = 0
        NACELL = 0

C       INITIALISE COUNTER FOR CELLS WITH INCOMPLETE STENCILS
        JCNEED = 0

C       RUN THROUGH ALL CENTRAL CELLS WITH INCOMPLETE STENCILS
        DO ICNEED = 1, NONEED

C         CELL NUMBER OF CENTRAL CELL
          ICCELL = NCNEED(ICNEED)

C         MAKE A LOCAL COPY OF THE STENCIL SO FAR
          DO JCSTEN = 1, NOSSTN(ICCELL)
            NCSTMP(JCSTEN) = NSSTEN(JCSTEN,ICCELL)
          ENDDO

C         INITIALISE STENCIL CELL COUNTER
          JCSTEN = NOSSTN(ICCELL)

C         INITIALISE STENCIL REMOTE CELL COUNTER
          JLCELL = 0

C         INITIALISE CONNECTIVITY FLAG
          FLFNHL = ILAYER.EQ.1
          FLCONN = ((ILAYER.EQ.2).AND.(NLAYLO(ICCELL).EQ.1))
          FLCONN = FLCONN.OR.FLFNHL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,ICCELL)
            CALL MCGLVL(MCRUN,JLEVEL)
            WRITE(6,'("need",6I5)')ILAYER,ICNEED,ICCELL,JLEVEL,
     +                             JCSTEN,NLAYLO(ICCELL)
            WRITE(6,'(10I5)')
     +     (NCLAYR(JOLAYR,ICCELL),JOLAYR=1,NOLAYR(ICCELL))
          ENDIF

C         =====================================================================

C         RUN THROUGH ALL CELLS IN PREVIOUS LAYER
          DO JOLAYR = 1, NOLAYR(ICCELL)

C           GET THE CELL NUMBER
            JCCELL = NCLAYR(JOLAYR,ICCELL)

C           FETCH THE M-CODE
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCCELL)

C           GET THE LEVEL
            CALL MCGLVL(MCRUN,JLEVEL)

C           GET THE CELL INTEGER COORDINATES 
            CALL MCCXYZ(MCRUN,JLEVEL,MPLOC(1),MPLOC(2),MPLOC(3))

C           ===================================================================

C           RUN THROUGH ALL FACES
            DO JFACE = 1,NFACE

CC             DIAGNOSTICS
C              IF(IPROC.EQ.LPROC)WRITE(6,'(3I7)')JCCELL,JLEVEL,JFACE

C             INCREMENT THE CELL COORDINATES
              MPNXT(1) = MPLOC(1) + MATINC(1,JFACE)
              MPNXT(2) = MPLOC(2) + MATINC(2,JFACE)
              MPNXT(3) = MPLOC(3) + MATINC(3,JFACE)

C             =================================================================

C             NEIGHBOUR CELL COORDINATE IN-OUT TEST
              JDIRN = MDIRN(JFACE)
              FLNIXL = (MPNXT(1).LT.MPMIN(1,JLEVEL))
              FLNIXR = (MPNXT(1).GT.MPMAX(1,JLEVEL))
              FLNIYL = (MPNXT(2).LT.MPMIN(2,JLEVEL))
              FLNIYR = (MPNXT(2).GT.MPMAX(2,JLEVEL))
              FLNIZL = (MPNXT(3).LT.MPMIN(3,JLEVEL))
              FLNIZR = (MPNXT(3).GT.MPMAX(3,JLEVEL))
              FLNIND = .TRUE.
              IF(FLNIXL.OR.FLNIXR)FLNIND = .FALSE.
              IF(FLNIYL.OR.FLNIYR)FLNIND = .FALSE.
              IF(FLNIZL.OR.FLNIZR)FLNIND = .FALSE.

              IF(FLNIND)THEN

C             =================================================================

C               NEIGHBOUR IS INSIDE THE DOMAIN
C               ------------------------------
C               ASSEMBLE THE NEIGHBOUR M-CODE
                CALL MCXYZC(MCNXT,JLEVEL,MPNXT(1),MPNXT(2),MPNXT(3))

C               SEARCH FOR THE NEIGHBOUR
                CALL OCFIND(NOCHLD,COCODE,NOCTMX,
     +                      MCNXT,KLEVEL,KCELL,CFOUND)

C               CHECK FINDINGS
                IF(CFOUND.EQ.CHTWIG)THEN

C                 ------------------------------------------------------------- 

C                 NEIGHBOUR IS LOCAL
C                 ------------------
C                 CHECK THE LEVEL
                  IF(KLEVEL.EQ.JLEVEL)THEN

C                   CLEAN FACE
C                   ----------
                    JCSTEN = JCSTEN + 1
                    NCSTMP(JCSTEN) = KCELL

C                   CONNECTIVITY
                    IF(FLCONN)NXDOOR(JFACE,ICCELL) = KCELL

C                   DIAGNOSTICS
                    IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("Nclean",4I5)')JLCELL,JCCELL,JFACE,KCELL
                      CALL MCRITE(MCNXT)
                    ENDIF

                  ELSEIF(KLEVEL.EQ.JLEVEL-1)THEN

C                   h-2h TRANSITION
C                   ---------------
                    JCSTEN = JCSTEN + 1
                    NCSTMP(JCSTEN) = KCELL

C                   CONNECTIVITY
                    IF(FLCONN)THEN
                      NXDOOR(JFACE,ICCELL) = KCELL
                      CALL MCGOCR(MCRUN,MOCTHI)
                      NH2HCF = NHTO2H + MCQTBL(MOCTHI,JFACE)
                      NSCODE(ICCELL) = NSCODE(ICCELL)
     +                               + NH2HCF*NSBASE**(JFACE-1)
                    ENDIF

C                   DIAGNOSTICS
                    IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("NhTO2h",4I5)')JLCELL,JCCELL,JFACE,KCELL
                      CALL MCRITE(MCNXT)
                    ENDIF

                  ELSE

C                   ERROR CONDITION - local neighbour cell not found
C                   ---------------
                    CALL ERHAND(2,1,4)

                  ENDIF
 
C                 ------------------------------------------------------------- 

                ELSEIF(CFOUND.EQ.CHNRML)THEN

C                 ------------------------------------------------------------- 

C                 NEIGHBOUR NOT FOUND AT THIS LEVEL
C                 ---------------------------------
C                 2h-h TRANSITION

C                 FIND ALL THE FACE-NEIGHBOUR h-CELLS
                  DO IOFSET = 1,NFSUB

                    CALL MCCOPY(MCNXT,MCTMP)
                    MOCTHI = MCFTBL(IOFSET,JFACE)
                    CALL MCHIRE(MCTMP,MOCTHI)
                      
C                   SEARCH FOR THE EXTENDED NEIGHBOUR
                    CALL OCFIND(NOCHLD,COCODE,NOCTMX,
     +                          MCTMP,KLEVLX,KCELLX,CFXTND)

C                   CHECK FINDINGS
                    IF(CFXTND.EQ.CHTWIG)THEN

C                     EXTENDED NEIGHBOUR IS LOCAL
                      JCSTEN = JCSTEN + 1
                      NCSTMP(JCSTEN) = KCELLX

C                     CONNECTIVITY
                      IF(FLCONN)THEN
                        IF(IOFSET.EQ.1)THEN
                          NXDOOR(JFACE,ICCELL) = KCELLX
                          NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                   + N2HTOH*NSBASE**(JFACE-1)
                        ENDIF
                      ENDIF

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("N2hTOh",4I5)')JLCELL,JCCELL,JFACE,KCELLX
                        CALL MCRITE(MCTMP)
                      ENDIF

                    ELSEIF(CFXTND.EQ.CHPARL)THEN

C                     EXTENDED NEIGHBOUR IS REMOTE

C                     SEARCH THE HALO-SO-FAR OCTREE
                      CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                            MCTMP,KLEVLH,KCELLH,CFOUNH)

C                     CHECK FINDINGS FOR REMOTE NEIGHBOUR
                      IF(CFOUNH.EQ.CHTWIG)THEN

C                       EXTENDED NEIGHBOUR IS IN THE HALO

C                       CELL ID FROM HALO INDIRECTION LIST
                        KCELHH = NRCLOC(KCELLH)

C                       REMOTE NEIGHBOUR IS LOCAL
                        JCSTEN = JCSTEN + 1
                        NCSTMP(JCSTEN) = KCELHH

C                       CONNECTIVITY
                        IF(FLCONN)THEN
                          IF(IOFSET.EQ.1)THEN
                            NXDOOR(JFACE,ICCELL) = KCELHH
                            NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                     + N2HTOH*NSBASE**(JFACE-1)
                          ENDIF
                        ENDIF

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("NR2hloc",4I5)')JLCELL,JCCELL,JFACE,KCELHH
                          CALL MCRITE(MCTMP)
                        ENDIF

                      ELSE

C                       EXTENDED NEIGHBOUR IS REMOTE

C                       INCREMENT THE LOCAL REMOTE CELL COUNTER
                        JLCELL = JLCELL + 1

C                       STORE THE NEIGHBOUR M-CODE IN THE LOCAL REMOTE CELL LIST
                        CALL MCSTOR(MCTMP,NLCODE,NTSTMX,JLCELL)

C                       UPDATE THE FACE CODE
                        NLCFAC(JLCELL) = JFACE*NSBASE + N2HTOH

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("N2hrem",3I5)')JLCELL,JCCELL,NLCFAC(JLCELL)
                          CALL MCRITE(MCTMP)
                        ENDIF

                      ENDIF
C                     CHECK FINDINGS FOR REMOTE NEIGHBOUR

                    ELSE

C                     EXTENDED NEIGHBOUR NOT FOUND

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                   WRITE(6,*)'Inner',JCCELL,JLEVEL,JFACE,IOFSET,MOCTHI
C                        WRITE(6,'(6I8)')MPLOC(1),MPLOC(2),MPLOC(3),
C     +                                  MPNXT(1),MPNXT(2),MPNXT(3)
                      ENDIF

                      CONTINUE

                    ENDIF
C                   CHECK FINDINGS

                  ENDDO
C                 FIND ALL THE FACE-NEIGHBOUR h-CELLS

C                 ------------------------------------------------------------- 

                ELSEIF(CFOUND.EQ.CHPARL)THEN

C                 ------------------------------------------------------------- 

C                 NEIGHBOUR IS REMOTE
C                 -------------------

C                 SEARCH THE HALO-SO-FAR OCTREE
                  CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                        MCNXT,KLEVLH,KCELLH,CFOUNH)

C                 CELL ID FROM HALO INDIRECTION LIST
                  KCELHH = NRCLOC(KCELLH)

C                 CHECK FINDINGS FOR REMOTE NEIGHBOUR
                  IF(CFOUNH.EQ.CHTWIG)THEN

C                   ----------------------------------------------------------- 

C                   REMOTE NEIGHBOUR IS LOCAL
C                   -------------------------
C                   CHECK THE LEVEL
                    IF(KLEVLH.EQ.JLEVEL)THEN

C                     CLEAN FACE
C                     ----------
                      JCSTEN = JCSTEN + 1
                      NCSTMP(JCSTEN) = KCELHH

C                     CONNECTIVITY
                      IF(FLCONN)NXDOOR(JFACE,ICCELL) = KCELHH

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("NRclean",4I5)')JLCELL,JCCELL,JFACE,KCELHH
                        CALL MCRITE(MCNXT)
                      ENDIF

                    ELSEIF(KLEVLH.EQ.JLEVEL-1)THEN

C                     h-2h TRANSITION
C                     ---------------
                      JCSTEN = JCSTEN + 1
                      NCSTMP(JCSTEN) = KCELHH

C                     CONNECTIVITY
                      IF(FLCONN)THEN
                        NXDOOR(JFACE,ICCELL) = KCELHH
                        CALL MCGOCR(MCRUN,MOCTHI)
                        NH2HCF = NHTO2H + MCQTBL(MOCTHI,JFACE)
                        NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                 + NH2HCF*NSBASE**(JFACE-1)
                      ENDIF

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("NRhTO2h",4I5)')JLCELL,JCCELL,JFACE,KCELHH
                        CALL MCRITE(MCNXT)
                      ENDIF

                    ELSE

C                     ERROR CONDITION - local remote neighbour cell not found
C                     ---------------
                      CALL ERHAND(2,1,4)

                    ENDIF
 
C                   ----------------------------------------------------------- 

                  ELSEIF(CFOUNH.EQ.CHNRML)THEN

C                   ----------------------------------------------------------- 

C                   REMOTE NEIGHBOUR NOT FOUND AT THIS LEVEL
C                   ----------------------------------------
C                   2h-h TRANSITION

C                   FIND ALL THE FACE-NEIGHBOUR h-CELLS
                    DO IOFSET = 1,NFSUB

                      CALL MCCOPY(MCNXT,MCTMP)
                      MOCTHI = MCFTBL(IOFSET,JFACE)
                      CALL MCHIRE(MCTMP,MOCTHI)
                      
C                     SEARCH THE HALO-SO-FAR OCTREE FOR THE EXTENDED NEIGHBOUR
                      CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                            MCTMP,KLVLHX,KCELHX,CFXTNH)

C                     CELL ID FROM HALO INDIRECTION LIST
                      KCELHH = NRCLOC(KCELHX)

C                     CHECK FINDINGS
                      IF(CFXTNH.EQ.CHTWIG)THEN

                        JCSTEN = JCSTEN + 1
                        NCSTMP(JCSTEN) = KCELHH

C                       CONNECTIVITY
                        IF(FLCONN)THEN
                          IF(IOFSET.EQ.1)THEN
                            NXDOOR(JFACE,ICCELL) = KCELHH
                            NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                     + N2HTOH*NSBASE**(JFACE-1)
                          ENDIF
                        ENDIF

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("NR2hTOh",4I5)')JLCELL,JCCELL,JFACE,KCELHH
                          CALL MCRITE(MCTMP)
                        ENDIF

                      ELSE

C                       EXTENDED NEIGHBOUR IS REMOTE

C                       INCREMENT THE LOCAL REMOTE CELL COUNTER
                        JLCELL = JLCELL + 1

C                       STORE THE NEIGHBOUR M-CODE IN THE LOCAL REMOTE CELL LIST
                        CALL MCSTOR(MCTMP,NLCODE,NTSTMX,JLCELL)

C                       UPDATE THE FACE CODE
                        NLCFAC(JLCELL) = JFACE*NSBASE + N2HTOH

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("N2hTOhr",3I5)')JLCELL,JCCELL,NLCFAC(JLCELL)
                          CALL MCRITE(MCTMP)
                        ENDIF

                      ENDIF
C                     CHECK FINDINGS

                    ENDDO
C                   FIND ALL THE FACE-NEIGHBOUR h-CELLS

C                   ----------------------------------------------------------- 

                  ELSE

C                   ----------------------------------------------------------- 

C                   NEIGHBOUR NOT FOUND IN THE HALO-SO-FAR
C                   --------------------------------------

C                   INCREMENT THE LOCAL REMOTE CELL COUNTER
                    JLCELL = JLCELL + 1

C                   STORE THE NEIGHBOUR M-CODE IN THE LOCAL REMOTE CELL LIST
                    CALL MCSTOR(MCNXT,NLCODE,NTSTMX,JLCELL)

C                   UPDATE THE FACE CODE
                    NLCFAC(JLCELL) = JFACE*NSBASE

C                   DIAGNOSTICS
                    IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("Nremot",3I5)')JLCELL,JCCELL,NLCFAC(JLCELL)
                      CALL MCRITE(MCNXT)
                    ENDIF

                  ENDIF
C                 CHECK FINDINGS ON REMOTE NEIGHBOUR

C                 ------------------------------------------------------------- 

                ELSE

C                 ------------------------------------------------------------- 

C                 ERROR CONDITION - interior cell not found
C                 ---------------
                  CALL ERHAND(2,3,4)

C                 ------------------------------------------------------------- 

                ENDIF
C               CHECK FINDINGS ON NEIGHBOUR INSIDE THE DOMAIN

C               ===============================================================

              ELSE

C               ===============================================================

C               NEIGHBOUR IS OUTSIDE THE DOMAIN
C               -------------------------------
C               GET THE OUTER BC ATTRIBUTE
                IGLOBC = NGLOBC(JFACE)

C               CHECK THE OUTER BC ATTRIBUTE
                IF(IGLOBC.EQ.NSPERI)THEN

C                 DIAGNOSTICS
                  IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,*)'Peri',MPNXT(1),MPNXT(2),MPNXT(3)
                  ENDIF

C                 SPECIAL CASE OF PERIODIC BC
C                 ---------------------------
C                 RSC 16-APR-2018 UPDATED FOR FULLY PERIODIC BOUNDARIES
C                 ASSEMBLE PERIODIC CELL M-CODE
C                 USING MIN/MAX COORDINATES FOR EACH COMPONENT AT THIS LEVEL
                  IF(FLNIXL)MPNXT(1) = MPMAX(1,JLEVEL) + 1
     +                               +(MPNXT(1)-MPMIN(1,JLEVEL))    
                  IF(FLNIXR)MPNXT(1) = MPMIN(1,JLEVEL) - 1
     +                               +(MPNXT(1)-MPMAX(1,JLEVEL))    
                  IF(FLNIYL)MPNXT(2) = MPMAX(2,JLEVEL) + 1
     +                               +(MPNXT(2)-MPMIN(2,JLEVEL))    
                  IF(FLNIYR)MPNXT(2) = MPMIN(2,JLEVEL) - 1
     +                               +(MPNXT(2)-MPMAX(2,JLEVEL))    
                  IF(FLNIZL)MPNXT(3) = MPMAX(3,JLEVEL) + 1
     +                               +(MPNXT(3)-MPMIN(3,JLEVEL))    
                  IF(FLNIZR)MPNXT(3) = MPMIN(3,JLEVEL) - 1
     +                               +(MPNXT(3)-MPMAX(3,JLEVEL))    
                  CALL MCXYZC(MCPER,JLEVEL,MPNXT(1),MPNXT(2),MPNXT(3))

C                 DIAGNOSTICS
                  IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,*)'Pmod',MPNXT(1),MPNXT(2),MPNXT(3)
                  ENDIF

C                 SEARCH FOR THE PERIODIC CELL
                  CALL OCFIND(NOCHLD,COCODE,NOCTMX,
     +                        MCPER,LLEVEL,LCELL,CFPERI)

C                 CHECK FINDINGS ON PERIODIC CELL
                  IF(CFPERI.EQ.CHTWIG)THEN

C                   ----------------------------------------------------------- 

C                   PERIODIC CELL IS LOCAL
C                   ----------------------
C                   CHECK THE LEVEL
                    IF(LLEVEL.EQ.JLEVEL)THEN

C                     CLEAN FACE
C                     ----------
                      JCSTEN = JCSTEN + 1
                      NCSTMP(JCSTEN) = LCELL

C                     CONNECTIVITY
                      IF(FLCONN)NXDOOR(JFACE,ICCELL) = LCELL

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
               WRITE(6,'("NPlocclean",3I5)')JLCELL,JCCELL,LCELL
                        CALL MCRITE(MCPER)
                      ENDIF

                    ELSEIF(LLEVEL.EQ.JLEVEL-1)THEN

C                     h-2h TRANSITION
C                     ---------------
                      JCSTEN = JCSTEN + 1
                      NCSTMP(JCSTEN) = LCELL

C                     CONNECTIVITY
                      IF(FLCONN)THEN
                        NXDOOR(JFACE,ICCELL) = LCELL
                        CALL MCGOCR(MCRUN,MOCTHI)
                        NH2HCF = NHTO2H + MCQTBL(MOCTHI,JFACE)
                        NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                 + NH2HCF*NSBASE**(JFACE-1)
                      ENDIF

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
               WRITE(6,'("NPlochTO2h",3I5)')JLCELL,JCCELL,LCELL
                        CALL MCRITE(MCNXT)
                      ENDIF

                    ELSE
    
C                     ERROR CONDITION - periodic local cell not found
C                     ---------------
                      CALL ERHAND(2,4,4)

                    ENDIF

C                   ----------------------------------------------------------- 

                  ELSEIF(CFPERI.EQ.CHNRML)THEN

C                   ----------------------------------------------------------- 

C                   PERIODIC CELL NOT FOUND AT THIS LEVEL
C                   -------------------------------------
C                   2h-h TRANSITION

C                   FIND ALL THE FACE-NEIGHBOUR h-CELLS
                    DO IOFSET = 1, NFSUB

C                     EXTEND M-CODE
                      CALL MCCOPY(MCPER,MCTMP)
                      MOCTHI = MCFTBL(IOFSET,JFACE)
                      CALL MCHIRE(MCTMP,MOCTHI)

C                     SEARCH FOR THE EXTENDED NEIGHBOUR
                      CALL OCFIND(NOCHLD,COCODE,NOCTMX,
     +                            MCTMP,LLEVLX,LCELLX,CFXTND)

C                     CHECK FINDINGS
                      IF(CFXTND.EQ.CHTWIG)THEN

C                       EXTENDED NEIGHBOUR IS LOCAL
                        JCSTEN = JCSTEN + 1
                        NCSTMP(JCSTEN) = LCELLX

C                       CONNECTIVITY
                        IF(FLCONN)THEN
                          IF(IOFSET.EQ.1)THEN
                            NXDOOR(JFACE,ICCELL) = LCELLX
                            NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                     + N2HTOH*NSBASE**(JFACE-1)
                          ENDIF
                        ENDIF

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("NPloc2hTOh",4I5)')JLCELL,JCCELL,JFACE,LCELLX
                          CALL MCRITE(MCTMP)
                        ENDIF

                      ELSEIF(CFXTND.EQ.CHPARL)THEN

C                       EXTENDED NEIGHBOUR IS REMOTE

C                       SEARCH THE HALO-SO-FAR OCTREE
                        CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                              MCTMP,LLEVLH,LCELLH,CFOUNH)

C                       CHECK FINDINGS FOR REMOTE NEIGHBOUR
                        IF(CFOUNH.EQ.CHTWIG)THEN

C                         EXTENDED NEIGHBOUR IS IN THE HALO

C                         CELL ID FROM HALO INDIRECTION LIST
                          LCELHH = NRCLOC(LCELLH)

C                         REMOTE NEIGHBOUR IS LOCAL
                          JCSTEN = JCSTEN + 1
                          NCSTMP(JCSTEN) = LCELHH

C                         CONNECTIVITY
                          IF(FLCONN)THEN
                            IF(IOFSET.EQ.1)THEN
                              NXDOOR(JFACE,ICCELL) = LCELHH
                              NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                       + N2HTOH*NSBASE**(JFACE-1)
                            ENDIF
                          ENDIF

C                         DIAGNOSTICS
                          IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("NPR2hloc",4I5)')JLCELL,JCCELL,JFACE,LCELHH
                            CALL MCRITE(MCTMP)
                          ENDIF

                        ELSE

C                         EXTENDED NEIGHBOUR IS REMOTE

C                         INCREMENT THE LOCAL REMOTE CELL COUNTER
                          JLCELL = JLCELL + 1

C                         STORE THE NEIGHBOUR M-CODE
C                         IN THE LOCAL REMOTE CELL LIST
                          CALL MCSTOR(MCTMP,NLCODE,NTSTMX,JLCELL)

C                         UPDATE THE FACE CODE
                          NLCFAC(JLCELL) = JFACE*NSBASE + N2HTOH

C                         DIAGNOSTICS
                          IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("NPR2hrem",3I5)')JLCELL,JCCELL,NLCFAC(JLCELL)
                            CALL MCRITE(MCTMP)
                          ENDIF

                        ENDIF
C                       CHECK FINDINGS FOR REMOTE NEIGHBOUR

                      ELSE

C                       EXTENDED NEIGHBOUR NOT FOUND

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,*)"NPRInner",JCCELL,JLEVEL,JFACE,IOFSET,MOCTHI
C                          WRITE(6,'(6I8)')MPLOC(1),MPLOC(2),MPLOC(3).
C     +                                    MPNXT(1),MPNXT(2),MPNXT(3)
                        ENDIF

                        CONTINUE

                      ENDIF
C                     CHECK FINDINGS

                    ENDDO
C                   FIND ALL THE FACE NEIGHBOUR h-CELLS

C                   ----------------------------------------------------------- 

                  ELSEIF(CFPERI.EQ.CHPARL)THEN

C                   ----------------------------------------------------------- 

C                   PERIODIC CELL IS REMOTE
C                   -----------------------

C                   SEARCH THE HALO-SO-FAR OCTREE
                    CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                          MCPER,LLEVLH,LCELLH,CFOUNH)

C                   CELL ID FROM HALO INDIRECTION LIST
                    LCELHH = NRCLOC(LCELLH)

C                   CHECK FINDINGS FOR REMOTE NEIGHBOUR
                    IF(CFOUNH.EQ.CHTWIG)THEN

C                     --------------------------------------------------------- 

C                     REMOTE NEIGHBOUR IS LOCAL
C                     -------------------------
C                     CHECK THE LEVEL
                      IF(LLEVLH.EQ.JLEVEL)THEN

C                       CLEAN FACE
C                       ----------
                        JCSTEN = JCSTEN + 1
                        NCSTMP(JCSTEN) = LCELHH

C                       CONNECTIVITY
                        IF(FLCONN)NXDOOR(JFACE,ICCELL) = LCELHH

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("NPRclean",4I5)')JLCELL,JCCELL,JFACE,LCELHH
                          CALL MCRITE(MCNXT)
                        ENDIF

                      ELSEIF(LLEVLH.EQ.JLEVEL-1)THEN

C                       h-2h TRANSITION
C                       ---------------
                        JCSTEN = JCSTEN + 1
                        NCSTMP(JCSTEN) = LCELHH

C                       CONNECTIVITY
                        IF(FLCONN)THEN
                          NXDOOR(JFACE,ICCELL) = LCELHH
                          CALL MCGOCR(MCRUN,MOCTHI)
                          NH2HCF = NHTO2H + MCQTBL(MOCTHI,JFACE)
                          NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                   + NH2HCF*NSBASE**(JFACE-1)
                        ENDIF

C                       DIAGNOSTICS
                        IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("NPRhTO2h",4I5)')JLCELL,JCCELL,JFACE,LCELHH
                          CALL MCRITE(MCPER)
                        ENDIF

                      ELSE

C                       ERROR CONDITION - local remote neighbour cell not found
C                       ---------------
                        CALL ERHAND(2,1,4)

                      ENDIF
 
C                     --------------------------------------------------------- 

                    ELSEIF(CFOUNH.EQ.CHNRML)THEN

C                     --------------------------------------------------------- 

C                     REMOTE NEIGHBOUR NOT FOUND AT THIS LEVEL
C                     ----------------------------------------
C                     2h-h TRANSITION

C                     FIND ALL THE FACE-NEIGHBOUR h-CELLS
                      DO IOFSET = 1,NFSUB

C                       EXTEND M-CODE
                        CALL MCCOPY(MCPER,MCTMP)
                        MOCTHI = MCFTBL(IOFSET,JFACE)
                        CALL MCHIRE(MCTMP,MOCTHI)
                      
C                       SEARCH THE HALO-SO-FAR OCTREE
C                       FOR THE EXTENDED NEIGHBOUR
                        CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                              MCTMP,LLVLHX,LCELHX,CFXTNH)

C                       CELL ID FROM HALO INDIRECTION LIST
                        LCELHH = NRCLOC(LCELHX)

C                       CHECK FINDINGS
                        IF(CFXTNH.EQ.CHTWIG)THEN

                          JCSTEN = JCSTEN + 1
                          NCSTMP(JCSTEN) = LCELHH

C                         CONNECTIVITY
                          IF(FLCONN)THEN
                            IF(IOFSET.EQ.1)THEN
                              NXDOOR(JFACE,ICCELL) = LCELHH
                              NSCODE(ICCELL) = NSCODE(ICCELL)
     +                                       + N2HTOH*NSBASE**(JFACE-1)
                            ENDIF
                          ENDIF

C                         DIAGNOSTICS
                          IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("NPR2hTOh",4I5)')JLCELL,JCCELL,JFACE,LCELHH
                            CALL MCRITE(MCTMP)
                          ENDIF

                        ELSE

C                         EXTENDED NEIGHBOUR IS REMOTE

C                         INCREMENT THE LOCAL REMOTE CELL COUNTER
                          JLCELL = JLCELL + 1

C                     STORE THE NEIGHBOUR M-CODE IN THE LOCAL REMOTE CELL LIST
                          CALL MCSTOR(MCTMP,NLCODE,NTSTMX,JLCELL)

C                         UPDATE THE FACE CODE
                          NLCFAC(JLCELL) = JFACE*NSBASE + N2HTOH

C                         DIAGNOSTICS
                          IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("NP2hTOhr",3I5)')JLCELL,JCCELL,NLCFAC(JLCELL)
                            CALL MCRITE(MCTMP)
                          ENDIF

                        ENDIF
C                       CHECK FINDINGS

                      ENDDO
C                     FIND ALL THE FACE-NEIGHBOUR h-CELLS

C                     --------------------------------------------------------- 

                    ELSE

C                     --------------------------------------------------------- 

C                     NEIGHBOUR NOT FOUND IN THE HALO-SO-FAR
C                     --------------------------------------

C                     INCREMENT THE LOCAL REMOTE CELL COUNTER
                      JLCELL = JLCELL + 1

C                     STORE THE NEIGHBOUR M-CODE IN THE LOCAL REMOTE CELL LIST
                      CALL MCSTOR(MCPER,NLCODE,NTSTMX,JLCELL)

C                     UPDATE THE FACE CODE
                      NLCFAC(JLCELL) = JFACE*NSBASE

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("NPremot",3I5)')JLCELL,JCCELL,NLCFAC(JLCELL)
                        CALL MCRITE(MCNXT)
                      ENDIF

                    ENDIF
C                   CHECK FINDINGS ON REMOTE NEIGHBOUR

C                   ----------------------------------------------------------- 

                  ELSE

C                   ----------------------------------------------------------- 

C                   ERROR CONDITION - periodic cell not found
C                   ---------------
                    CALL ERHAND(2,6,4)

C                 ------------------------------------------------------------- 

                  ENDIF
C                 CHECK FINDINGS ON NEIGHBOUR INSIDE THE DOMAIN

C                 -------------------------------------------------------------

                ELSE

C                 -------------------------------------------------------------

C                 NON-PERIODIC BC
C                 ---------------
C                 CONNECTIVITY
C                 FIRST LAYER ONLY
C                  IF(ILAYER.EQ.1)THEN
                  IF(FLCONN)THEN

C                   ALLOCATE A BC INDEX
                    JBCVAL = JBCVAL + 1
 
C                   SET THE NEIGHBOUR CELL ID
                    NXDOOR(JFACE,ICCELL) = JBCVAL

C                   SET CELL FACE ATTRIBUTE
                    NSCODE(ICCELL) = NSCODE(ICCELL)
     +                             + IGLOBC*NSBASE**(JFACE-1)

C                   SET THE BC PHYSICAL COORDINATES
                    HALFSZ = HALF*HCSIZE(JLEVEL)
           BCOORD(1,JBCVAL) = (REAL(MPLOC(1))+REAL(MPNXT(1))+ONE)*HALFSZ
           BCOORD(2,JBCVAL) = (REAL(MPLOC(2))+REAL(MPNXT(2))+ONE)*HALFSZ
           BCOORD(3,JBCVAL) = (REAL(MPLOC(3))+REAL(MPNXT(3))+ONE)*HALFSZ

C                   SET THE BC CELL
                    NBCCEL(JBCVAL) = ICCELL

C                   SET THE BC FACE
                    NBCFAC(JBCVAL) = JFACE

                  ENDIF

C                 DIAGNOSTICS
                  IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("Nbcond",4I5)')JLCELL,JCCELL,JFACE,
     +                                   NXDOOR(JFACE,JCCELL)
                    CALL MCRITE(MCNXT)
                  ENDIF

                ENDIF
C               CHECK OUTER BC ATTRIBUTE

C               ===============================================================

              ENDIF
C             NEIGHBOUR CELL COORDINATE IN-OUT TEST

C             =================================================================

            ENDDO
C           RUN THROUGH ALL FACES

C           ===================================================================

          ENDDO
C         RUN THROUGH ALL CELLS IN PREVIOUS LAYER

C         =====================================================================

C         DEAL WITH ANY REMOTE CELLS IN THE CURRENT STENCIL
          NLCELL = JLCELL
          IF(NLCELL.GT.0)THEN

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'("LRem1",4I5)')ILAYER,ICCELL,NLCELL,NXCELL
              DO JRCELL = 1,NLCELL
                WRITE(6,'(3I5)')JRCELL,ICCELL,NLCFAC(JRCELL)
                CALL MCRTRV(MCTMP,NLCODE,NTSTMX,JRCELL)
                CALL MCRITE(MCTMP)
              ENDDO
            ENDIF

C           SORT THE LIST OF LOCAL REMOTE CELLS
C           BY M-CODE
            CALL SORTEM(NLCODE,NLCFAC,NTSTMX,NLCELL)

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'("LRemSort",4I5)')ILAYER,ICCELL,NLCELL,NXCELL
              DO JRCELL = 1,NLCELL
                WRITE(6,'(3I5)')JRCELL,ICCELL,NLCFAC(JRCELL)
                CALL MCRTRV(MCTMP,NLCODE,NTSTMX,JRCELL)
                CALL MCRITE(MCTMP)
              ENDDO
            ENDIF

C           REMOVE DUPLICATES
C           ONLY WHERE BOTH THE M-CODE AND THE FACE CODE ARE DUPLICATED
            CALL REMDMI(NLCODE,NLCFAC,NTSTMX,NLCELL)

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'("LRemRemD",4I5)')ILAYER,ICCELL,NLCELL,NXCELL
              DO JRCELL = 1,NLCELL
                WRITE(6,'(3I5)')JRCELL,ICCELL,NLCFAC(JRCELL)
                CALL MCRTRV(MCTMP,NLCODE,NTSTMX,JRCELL)
                CALL MCRITE(MCTMP)
              ENDDO
            ENDIF

C           ADD LOCAL REMOTE CELLS TO LIST OF REMOTE CELLS FOR THIS LAYER
            NSTART = NXCELL + 1
            NFINIS = NXCELL + NLCELL
            CALL MCMTOR(NLCODE,NTSTMX,NXCODE,NREMAX,NSTART,NFINIS)

C           ADD CENTRAL CELL ID TO GLOBAL LIST
C           ALL THE SAME FOR ANY ONE LAYER
C           ADD FACE CODE TO GLOBAL LIST
            DO JLCELL = 1, NLCELL
              JRCELL = NXCELL + JLCELL
              NCENTC(JRCELL) = ICCELL
              NRCFAC(JRCELL) = NLCFAC(JLCELL)
            ENDDO

C           NUMBER OF REQUESTED REMOTE CELLS SO FAR IN THIS LAYER
            NXCELL = NXCELL + NLCELL

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'("LRem2",4I5)')ILAYER,ICCELL,NLCELL,NXCELL
              DO JRCELL = 1,NXCELL
                WRITE(6,'(3I5)')JRCELL,NCENTC(JRCELL),NRCFAC(JRCELL)
                CALL MCRTRV(MCTMP,NXCODE,NREMAX,JRCELL)
                CALL MCRITE(MCTMP)
              ENDDO
            ENDIF

          ENDIF

C         =====================================================================

C         NUMBER OF CELLS IN STENCIL SO FAR, EXCLUDING REMOTE CELLS
          NSOFAR = JCSTEN

C         EXTRACT THE SORTED PREVIOUS LAYERS
          DO JCSTEN = 1, NOSSTN(ICCELL)
            NCPTMP(JCSTEN) = NCSTMP(JCSTEN)
          ENDDO
          NOPSTN = NOSSTN(ICCELL)
C         EXTRACT THE NEWEST LAYER
          JOLAYR = 0
          DO JCSTEN = NOSSTN(ICCELL)+1,NSOFAR
            JOLAYR = JOLAYR + 1
            NCLTMP(JOLAYR) = NCSTMP(JCSTEN)
          ENDDO
          
C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("extract",3I5)')NOSSTN(ICCELL),NSOFAR,JOLAYR
            WRITE(6,'(10I5)')(NCSTMP(JCSTEN),JCSTEN=1,NSOFAR)
            WRITE(6,'(10I5)')(NCLTMP(JCSTEN),JCSTEN=1,JOLAYR)
          ENDIF

C         SORT THE STENCIL SO FAR
C         BY CELL NUMBER
          CALL SORTIN(NCSTMP,NTSTMX,NSOFAR)

CC         DIAGNOSTICS
C          IF(IPROC.EQ.LPROC)THEN
C            WRITE(6,*)NSOFAR
C            DO JCSTEN = 1, NSOFAR
C              WRITE(6,*)JCSTEN,NCSTMP(JCSTEN),NCLABL(JCSTEN)
C            ENDDO
C          ENDIF

C         REMOVE DUPLICATES
          CALL REMDIN(NCSTMP,NTSTMX,NSOFAR)

CC         DIAGNOSTICS
C          IF(IPROC.EQ.LPROC)THEN
C            WRITE(6,*)NSOFAR
C            DO JCSTEN = 1, NSOFAR
C              WRITE(6,*)JCSTEN,NCSTMP(JCSTEN),NCLABL(JCSTEN)
C            ENDDO
C          ENDIF

C         EXCLUDE THE CENTRAL CELL
          CALL REMCEN(NCSTMP,ICCELL,NTSTMX,NSOFAR)
 
CC         DIAGNOSTICS
C          IF(IPROC.EQ.LPROC)THEN
C            WRITE(6,*)NSOFAR
C            DO JCSTEN = 1, NSOFAR
C              WRITE(6,*)JCSTEN,NCSTMP(JCSTEN),NCLABL(JCSTEN)
C            ENDDO
C          ENDIF

C         STORE THE STENCIL SO FAR
C         EXCLUDING REMOTE CELLS
C         EXCLUDING THE CENTRAL CELL
          DO JCSTEN = 1,NSOFAR
            NSSTEN(JCSTEN,ICCELL) = NCSTMP(JCSTEN)
          ENDDO
          NOSSTN(ICCELL) = NSOFAR

C         CHECK FOR REQUIRED NO OF CELLS IN STENCIL
C         BASED ON ACTUAL NUMBER OF LOCAL CELLS ONLY
          IF(NSOFAR.LT.NRQSTN)THEN

C           NEW PREVIOUS LAYER

C           SORT AND REMOVE DUPLICATES
            CALL SORTIN(NCLTMP,NTSTMX,JOLAYR)
            CALL REMDIN(NCLTMP,NTSTMX,JOLAYR)

C           EXCLUDE THE CENTRAL CELL
            CALL REMCEN(NCLTMP,ICCELL,NTSTMX,JOLAYR)

C           EXCLUDE DUPLICATES FROM PREVIOUS LAYERS
            DO JCSTEN = 1, NOPSTN
              ICPSTN = NCPTMP(JCSTEN)
              CALL REMCEN(NCLTMP,ICPSTN,NTSTMX,JOLAYR)
            ENDDO

C           STORE THE NEW PREVIOUS LAYER
            NOLAYR(ICCELL) = JOLAYR
            DO JOLAYR = 1, NOLAYR(ICCELL)
              NCLAYR(JOLAYR,ICCELL) = NCLTMP(JOLAYR)
            ENDDO

         ENDIF

C         =====================================================================

C         CHECK FOR REQUIRED NO OF CELLS IN STENCIL
C         BASED ON ACTUAL NUMBER OF LOCAL CELLS
C         AND PROJECTED NUMBER OF REMOTE CELLS
          JCSTEN = NSOFAR + NLCELL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            ISCODE = NSCODE(ICCELL)
            DO JOLAYR = 1,6
              IFCODE(JOLAYR) = MOD(ISCODE,NSBASE)
              ISCODE = ISCODE/NSBASE
            ENDDO
            WRITE(6,'("reqd",5I5)')ICCELL,ILAYER,NSOFAR,NLCELL,JCSTEN
            WRITE(6,'(10I5)')(NCSTMP(JOLAYR),JOLAYR=1,NSOFAR)
            WRITE(6,'(12I5)')(NXDOOR(JOLAYR,ICCELL),JOLAYR=1,6),
     +                      (IFCODE(JOLAYR),JOLAYR=1,6)
          ENDIF

          IF(JCSTEN.GE.NRQSTN)THEN

C           STENCIL COMPLETE
            NSOFAR = NOSSTN(ICCELL)
            DO JCSTEN = 1,NSOFAR
              NCSTMP(JCSTEN) = NSSTEN(JCSTEN,ICCELL)
            ENDDO
            DO JCSTEN = 1, NSOFAR
              JLCELL = NCSTMP(JCSTEN)
              CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JLCELL)
              CALL MCSTOR(MCRUN,NLCODE,NTSTMX,JCSTEN)
            ENDDO
            CALL SORTEM(NLCODE,NCSTMP,NTSTMX,NSOFAR)
            DO JCSTEN = 1, NSOFAR
              NSSTEN(JCSTEN,ICCELL) = NCSTMP(JCSTEN)
            ENDDO

          ELSE

C           UPDATE THE LIST OF CELLS WITH INCOMPLETE STENCILS
            JCNEED = JCNEED + 1
            NCNEED(JCNEED) = ICCELL

          ENDIF

C         =====================================================================

        ENDDO
C       RUN THROUGH ALL CENTRAL CELLS WITH INCOMPLETE STENCILS

C       NUMBER OF CELLS WITH INCOMPLETE STENCILS
        NONEED = JCNEED

C       =======================================================================

C       DEAL WITH REMOTE CELLS IN THE CURRENT LAYER
C       -------------------------------------------

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("Req list",2I5)')ILAYER,NXCELL
          DO JRCELL = 1, NXCELL
            WRITE(6,'(3I5)')JRCELL,NCENTC(JRCELL),NRCFAC(JRCELL)
            CALL MCRTRV(MCRUN,NXCODE,NREMAX,JRCELL)
            CALL MCRITE(MCRUN)
          ENDDO
        ENDIF

C       BUILD THE INITIAL LAYER-HALO M-CODE LIST
C       ----------------------------------------
        NHCELL = NXCELL
        IF(NHCELL.GT.0)THEN

C         EXTRACT THE LIST OF REMOTE M-CODES FROM THE REQUIREMENT LIST
          CALL MCMTRV(NHCODE,NREMAX,NXCODE,NREMAX,1,NXCELL)

C         SORT THE LIST OF REMOTE M-CODES
          CALL SORTMC(NHCODE,NREMAX,NHCELL)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("Req list sorted",2I5)')ILAYER,NHCELL
            DO JRCELL = 1, NHCELL
              CALL MCRTRV(MCRUN,NHCODE,NREMAX,JRCELL)
              CALL MCRITE(MCRUN)
            ENDDO
          ENDIF

C         REMOVE DUPLICATES
          CALL REMDMC(NHCODE,NREMAX,NHCELL)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("Req list remdup",2I5)')ILAYER,NHCELL
            DO JRCELL = 1, NHCELL
              CALL MCRTRV(MCRUN,NHCODE,NREMAX,JRCELL)
              CALL MCRITE(MCRUN)
            ENDDO
          ENDIF

C         INITIALISE THE CELL STATUS CODES
          DO JRCELL = 1, NHCELL
            NRCCOD(JRCELL) = NCLEAN 
          ENDDO

        ENDIF

C       =======================================================================

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("PARLAYb",3I5)')IPROC,ILAYER,NHCELL
        ENDIF

C       EXCHANGE DATA ABOUT THE REMOTE CELLS IN THE CURRENT LAYER
C       ---------------------------------------------------------
        CALL PARLAY(NHCODE,NRCCOD,NHCELL)

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          DO JRCELL = 1, NHCELL
            WRITE(6,'("PARLAYa",2I5)')JRCELL,NRCCOD(JRCELL)
          ENDDO
        ENDIF

C       =======================================================================

C       BUILD THE ACTUAL LAYER-HALO M-CODE LIST
C       ---------------------------------------
        IF(NHCELL.GT.0)THEN

C         NOTE THAT NRCCOD NOW CONTAINS THE ACTUAL CELL STATUS CODES
C                   NHCODE AND NHCELL REMAIN UNCHANGED

C         INITIALISE COUNTER FOR ACTUAL LAYER-HALO M-CODE LIST
          JRLIST = 0

C         RUN THROUGH THE ACTUAL REMOTE CELL STATUS CODES
          DO JRCELL = 1, NHCELL

C           -------------------------------------------------------------------

C           RETRIEVE THE REQUESTED M-CODE
            CALL MCRTRV(MCRUN,NHCODE,NREMAX,JRCELL)

C           CHECK THE REMOTE CELL STATUS CODE
            IF(NRCCOD(JRCELL).EQ.NCLEAN)THEN

C             -----------------------------------------------------------------

C             REMOTE CELL IS NORMAL
C             ---------------------
C             SAVE THE REMOTE CELL M-CODE
C             IN THE ACTUAL LAYER-HALO M-CODE LIST
              JRLIST = JRLIST + 1
              CALL MCSTOR(MCRUN,NACODE,NREMAX,JRLIST)

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("ActualCC",2I5)')JRCELL,JRLIST
              ENDIF

C             -----------------------------------------------------------------

            ELSEIF(NRCCOD(JRCELL).EQ.NHTO2H)THEN

C             -----------------------------------------------------------------

C             h-2h TRANSITION
C             ---------------
C             SAVE THE TRUNCATED 2h-CELL M-CODE
C             IN THE ACTUAL LAYER-HALO M-CODE LIST
              JRLIST = JRLIST + 1
              CALL MCCOPY(MCRUN,MCNXT)
              CALL MCLOWR(MCNXT)
              CALL MCSTOR(MCNXT,NACODE,NREMAX,JRLIST)

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("ActualH2",2I5)')JRCELL,JRLIST
              ENDIF

C             -----------------------------------------------------------------

            ELSEIF(NRCCOD(JRCELL).EQ.N2HTOH)THEN

C             -----------------------------------------------------------------

C             2h-h TRANSITION
C             ---------------
C             SAVE THE EXTENDED h-CELL M-CODES
C             IN THE ACTUAL LAYER-HALO M-CODE LIST
              DO IOFSET = 0,NOFSET

                JRLIST = JRLIST + 1
                MOCTHI = IOFSET
                CALL MCCOPY(MCRUN,MCNXT)
                CALL MCHIRE(MCNXT,MOCTHI)
                CALL MCSTOR(MCNXT,NACODE,NREMAX,JRLIST)

C               DIAGNOSTICS
                IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("Actual2H",2I5)')JRCELL,JRLIST
                ENDIF

              ENDDO

C             -----------------------------------------------------------------

            ELSE

C             -----------------------------------------------------------------

C             ERROR CONDITION - invalid remote cell status code
C             ---------------
              CALL ERHAND(2,7,4)

C             -----------------------------------------------------------------

            ENDIF
C           CHECK THE REMOTE CELL STATUS CODE

          ENDDO
C         RUN THROUGH THE ACTUAL REMOTE CELL STATUS CODES

C         NUMBER OF REMOTE M-CODES IN THE ACTUAL LAYER-HALO M-CODE LIST
          NACELL = JRLIST

C         ---------------------------------------------------------------------

C         SORT THE ACTUAL LAYER-HALO M-CODE LIST
          CALL SORTMC(NACODE,NREMAX,NACELL)

C         REMOVE DUPLICATES
          CALL REMDMC(NACODE,NREMAX,NACELL)

C         =====================================================================

C         MAKE A LAYER-HALO OCTREE BASED ON THE ACTUAL LAYER-HALO M-CODE LIST
C         -------------------------------------------------------------------
          CALL LHTREE(NACODE,NREMAX,NACELL,
     +                NHCPAR,NHCHLD,CHCODE,NHCTMX,NHCHWM)
      
C         =====================================================================

C         SEARCH THE ACTUAL LAYER-HALO M-CODE LIST
C         ----------------------------------------
C         USING THE ORIGINAL LIST OF REMOTE M-CODES IN THE REQUIREMENT LIST
C         TO FIND ANY UNWANTED M-CODES
      
C         INITIALISE IN-LAYER-HALO FLAGS
          DO JRCELL = 1, NACELL
            FLINLH(JRCELL) = .FALSE.
          ENDDO

C         RUN THROUGH THE ORIGINAL LIST OF M-CODES IN THE REQUIREMENT LIST
          DO JRCELL = 1, NXCELL

C           FETCH THE M-CODE
            CALL MCRTRV(MCRUN,NXCODE,NREMAX,JRCELL)

C           FIND THE REQUIRED LEVEL
            CALL MCGLVL(MCRUN,JLEVEL)

C           SEARCH FOR THIS M-CODE IN THE LAYER-HALO OCTREE
            CALL OCFIND(NHCHLD,CHCODE,NHCTMX,MCRUN,KLEVEL,KCELL,CFOUND)

C           CHECK FINDINGS
            IF(CFOUND.EQ.CHTWIG)THEN

C             ----------------------------------------------------------------- 

C             M-CODE IS FOUND
C             ---------------
C             CHECK THE LEVEL
              IF(KLEVEL.EQ.JLEVEL)THEN

C               CLEAN CELL
                FLINLH(KCELL) = .TRUE.

C               DIAGNOSTICS
                IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'("SearchAct",2I5)')JRCELL,KCELL
                  CALL MCRITE(MCRUN)
                ENDIF

              ELSEIF(KLEVEL.EQ.JLEVEL-1)THEN

C               h-2h TRANSITION
                FLINLH(KCELL) = .TRUE.

              ENDIF

C             ----------------------------------------------------------------- 

            ELSEIF(CFOUND.EQ.CHNRML)THEN

C             ----------------------------------------------------------------- 

C             M-CODE IS NOT FOUND AT THIS LEVEL
C             ---------------------------------
C             2h-h TRANSITION

C             EXTEND M-CODE
C             LOWEST FACE-NEIGHBOUR h-CELL
              CALL MCCOPY(MCRUN,MCTMP)
              JFACE = NRCFAC(JRCELL)/NSBASE
              MOCTHI = MCFTBL(1,JFACE)
              CALL MCHIRE(MCTMP,MOCTHI)

C             SEARCH FOR EXTENDED M-CODE IN THE LAYER-HALO OCTREE
              CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                    MCTMP,KLEVLX,KCELLX,CFOUND)

C             CHECK FINDINGS
              IF(CFOUND.EQ.CHTWIG)THEN

C               EXTENDED M-CODE IS FOUND
C               ------------------------
C               FIND ALL THE FACE-NEIGHBOUR h-CELLS
                DO IOFSET = 1, NFSUB

                  CALL MCCOPY(MCRUN,MCTMP)
                  MOCTHI = MCFTBL(IOFSET,JFACE)
                  CALL MCHIRE(MCTMP,MOCTHI)

C                 SEARCH FOR EXTENDED M-CODE IN THE LAYER-HALO OCTREE
                  CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                        MCTMP,KLEVLX,KCELLX,CFOUND)

                  FLINLH(KCELLX) = .TRUE.

                ENDDO
C               FIND ALL THE FACE-NEIGHBOUR h-CELLS

              ELSE

C               ERROR CONDITION: h-CELL NOT FOUND
                CONTINUE

              ENDIF
C             CHECK FINDINGS

C             ----------------------------------------------------------------- 

            ELSE

C             ----------------------------------------------------------------- 

C             ERROR CONDITION: h-CELL NOT FOUND
              CONTINUE

C             ----------------------------------------------------------------- 

            ENDIF
C           CHECK FINDINGS
          
          ENDDO
C         RUN THROUGH THE ORIGINAL LIST OF M-CODES IN THE REQUIREMENT LIST

C         =====================================================================

C         SEARCH THE ACTUAL LAYER-HALO M-CODE LIST
C         USING THE EXISTING HALO
C         TO FIND ANY M-CODES ALREADY IN THE HALO
      
CC         DIAGNOSTICS
C          IF(IPROC.EQ.LPROC)WRITE(6,'("exist",2I5)')NCELP1,NCELHW

C         RUN THROUGH THE HALO LAYER
          DO JRCELL = NCELP1, NCELHW

C           FETCH THE M-CODE
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JRCELL)

CC           FIND THE REQUIRED LEVEL
C            CALL MCGLVL(MCRUN,JLEVEL)

C           SEARCH FOR THIS M-CODE IN THE LAYER-HALO OCTREE
            CALL OCFIND(NHCHLD,CHCODE,NHCTMX,MCRUN,KLEVEL,KCELL,CFOUND)

C           CHECK FINDINGS
            IF(CFOUND.EQ.CHTWIG)THEN

C             ----------------------------------------------------------------- 

C             M-CODE IS FOUND
C             ---------------
C             EXISTING HALO M-CODE APPEARS IN THE LAYER-HALO
C             HENCE LAYER-HALO M-CODE IS ALREADY IN THE HALO

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)WRITE(6,'("existYes",2I5)')JRCELL,KCELL

C             SET LAYER-HALO CELL LOCATION TO THAT IN THE EXISTING HALO LAYER
              CALL OCTURN(NHCHLD,CHCODE,NHCTMX,MCRUN,JRCELL,CFOUND)

C             SET FLAG TO EXCLUDE THIS M-CODE FROM NEW CELL ASSIGNMENT
              FLINLH(KCELL) = .FALSE.

C             ----------------------------------------------------------------- 

            ELSE

C             ----------------------------------------------------------------- 

C             M-CODE IS NOT FOUND
C             -------------------
C             EXISTING HALO M-CODE IS NOT IN THE LAYER-HALO

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)WRITE(6,'("existNot",I5)')JRCELL

              CONTINUE

C             ----------------------------------------------------------------- 

            ENDIF
C           CHECK FINDINGS
          
          ENDDO
C         RUN THROUGH THE ORIGINAL LIST OF M-CODES IN THE REQUIREMENT LIST

C         =====================================================================

C         ASSIGN NEW HALO CELLS
C         ---------------------

C         INITIALISE HALO CELL INDEX
          JRCLOC = NCELHW

C         RUN THROUGH THE LAYER-HALO
          DO JRCELL = 1, NACELL

C           CHECK THE IN-LAYER-HALO FLAG
            IF(FLINLH(JRCELL))THEN

C             FETCH THE LAYER-HALO M-CODE
              CALL MCRTRV(MCRUN,NACODE,NREMAX,JRCELL)

C             MAKE A NEW HALO CELL
              JRCLOC = JRCLOC + 1

C             STORE THE M-CODE IN THE HALO
              CALL MCSTOR(MCRUN,NMCODE,NSIZMX,JRCLOC)

C             SET LAYER-HALO CELL LOCATION TO NEW HALO CELL LOCATION
              CALL OCTURN(NHCHLD,CHCODE,NHCTMX,MCRUN,JRCLOC,CFOUND)

C             SET HALO LAYER ID
              NLAYLO(JRCLOC) = ILAYER

C             INITIALISE NEAREST-NEIGHBOUR CONNECTIVITY
              IF(ILAYER.EQ.1)THEN
                DO JFACE = 1,NFACE
                  NXDOOR(JFACE,JRCLOC) = JRCLOC
                ENDDO
                NSCODE(JRCLOC) = NCLEAN
              ENDIF

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("Assign",3I5)')JRCELL,JRCLOC,NLAYLO(JRCLOC)
                CALL MCRITE(MCRUN)
              ENDIF

            ENDIF

          ENDDO
C         RUN THROUGH THE LAYER-HALO

C         END INDEX FOR HALO CELLS
          NCELHW = JRCLOC

C         =====================================================================

C         SEARCH THE ACTUAL LAYER-HALO M-CODE LIST
C         ----------------------------------------
C         USING THE ORIGINAL LIST OF REMOTE M-CODES IN THE REQUIREMENT LIST
C         TO COMPLETE THE STENCILS
      
C         FACE NEIGHBOUR HALO LIST COUNTER
          JRNCEL = 0

C         RUN THROUGH THE ORIGINAL LIST OF M-CODES IN THE REQUIREMENT LIST
          DO JRCELL = 1, NXCELL

C           CENTRAL CELL
            JCENTC = NCENTC(JRCELL)

C           FACE CODE
            JCFACE = NRCFAC(JRCELL)/NSBASE

C           FACE STATUS CODE
            JCSTAT = MOD(NRCFAC(JRCELL),NSBASE)

C           OPPOSITE FACE CODE
            JRFACE = MAPOPP(JCFACE)

C           FETCH THE M-CODE
            CALL MCRTRV(MCRUN,NXCODE,NREMAX,JRCELL)

C           FIND THE REQUIRED LEVEL
            CALL MCGLVL(MCRUN,JLEVEL)

C           SEARCH FOR THIS M-CODE IN THE LAYER-HALO OCTREE
            CALL OCFIND(NHCHLD,CHCODE,NHCTMX,MCRUN,KLEVEL,JRCLOC,CFOUND)

C           ACTION FLAGS
            FLNEQC = (JRCLOC.NE.JCENTC)
C           ADD TO STENCIL AND PREVIOUS LAYER
            FLADST = (JCSTAT.NE.NBCOND)
C            FLADST = FLADST.AND.FLNEQC
C           FIRST LAYER FACE NEIGHBOUR ACTIONS
            FLFNHL = ILAYER.EQ.1
C           NEAREST-NEIGHBOUR CONNECTIVITY
            FLCONN = ((ILAYER.EQ.2).AND.(NLAYLO(JCENTC).EQ.1))
            FLCONN = FLCONN.OR.FLFNHL

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("Stn",4I5,2X,A)')JRCELL,JRCLOC,JCENTC,JCSTAT,CFOUND
              CALL MCRITE(MCRUN)
            ENDIF

C           CHECK FINDINGS
            IF(CFOUND.EQ.CHTWIG)THEN

C             ----------------------------------------------------------------- 

C             M-CODE IS FOUND
C             ---------------
C             CHECK THE LEVEL
              IF(KLEVEL.EQ.JLEVEL)THEN

C               --------------------------------------------------------------- 

C               CLEAN CELL
C               ----------

C               ADD TO STENCIL AND PREVIOUS LAYER
C               EXCEPT PERIODIC BC CELLS
C               RSC 16-APR-2018 UPDATED FOR FULLY PERIODIC BOUNDARIES
C                IF(FLADST)THEN

                  NOSSTN(JCENTC) = NOSSTN(JCENTC) + 1
                  NSSTEN(NOSSTN(JCENTC),JCENTC) = JRCLOC
                  NOLAYR(JCENTC) = NOLAYR(JCENTC) + 1
                  NCLAYR(NOLAYR(JCENTC),JCENTC) = JRCLOC

C                 DIAGNOSTICS
                  IF(IPROC.EQ.LPROC)THEN
                WRITE(6,'("StnCCadd",6I5)')JRCELL,JCENTC,JCSTAT,JRCLOC,
     +                                         JRFACE,NOSSTN(JCENTC)
        WRITE(6,'(30I5)')(NSSTEN(JCSTEN,JCENTC),JCSTEN=1,NOSSTN(JCENTC))
        WRITE(6,'(30I5)')(NCLAYR(JCSTEN,JCENTC),JCSTEN=1,NOLAYR(JCENTC))
                  ENDIF

C                ENDIF
C               ADD TO STENCIL AND PREVIOUS LAYER

C               NEAREST-NEIGHBOUR CONNECTIVITY
                IF(FLCONN)THEN

C                 CHECK CELL FACE STATUS
C                 INCLUDE PERIODIC BC CELLS
                  IF(JCSTAT.LE.1)THEN
        
C                   CLEAN CELL
                    NXDOOR(JCFACE,JCENTC) = JRCLOC
                    NSCODE(JCENTC) = NSCODE(JCENTC)
     +                             + NCLEAN*NSBASE**(JCFACE-1)

C                   DIAGNOSTICS
                    IF(IPROC.EQ.LPROC)THEN
                      ISCODE = NSCODE(JCENTC)
                      DO JOLAYR = 1,6
                        IFCODE(JOLAYR) = MOD(ISCODE,NSBASE)
                        ISCODE = ISCODE/NSBASE
                      ENDDO
                  WRITE(6,'("StnCCaddCONN",7I5)')NXDOOR(JCFACE,JCENTC),
     +                                     (IFCODE(JOLAYR),JOLAYR=1,6)
                    ENDIF

                  ELSEIF(JCSTAT.EQ.N2HTOH)THEN

C                   h-2h TRANSITION
                    CALL MCGOCR(MCRUN,MOCTHI)
                    JQUART = MCQTBL(MOCTHI,JRFACE)
C                   DIAGNOSTICS
                    IF(IPROC.EQ.LPROC)THEN
                      WRITE(6,'("StnCCaddCON2h",7I5)')MOCTHI,JQUART
                    ENDIF

C                   LOWEST QUARTET MEMBER ONLY
                    IF(JQUART.EQ.0)THEN
                      NXDOOR(JCFACE,JCENTC) = JRCLOC
C                      NH2HCF = NHTO2H + JQUART
C                      NSCODE(JCENTC) = NSCODE(JCENTC)
C     +                               + NH2HCF*NSBASE**(JCFACE-1)
                      NSCODE(JCENTC) = NSCODE(JCENTC)
     +                               + N2HTOH*NSBASE**(JCFACE-1)

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                        ISCODE = NSCODE(JCENTC)
                        DO JOLAYR = 1,6
                          IFCODE(JOLAYR) = MOD(ISCODE,NSBASE)
                          ISCODE = ISCODE/NSBASE
                        ENDDO
                  WRITE(6,'("StnCCaddCON2h",7I5)')NXDOOR(JCFACE,JCENTC),
     +                                      (IFCODE(JOLAYR),JOLAYR=1,6)
                      ENDIF

                    ENDIF

                  ELSE

C                   ERROR CONDITION
                    CONTINUE

                  ENDIF
C                 CHECK CELL FACE STATUS

                ENDIF
C               NEAREST-NEIGHBOUR CONNECTIVITY

C               FIRST LAYER FACE NEIGHBOURS
                IF(FLFNHL)THEN

C                 ADD TO FACE NEIGHBOUR HALO LIST
                  JRNCEL = JRNCEL + 1
                  NRNLOC(JRNCEL) = JRCLOC
                  NRNFAC(JRNCEL) = JRFACE

C                 CHECK CELL FACE STATUS
                  IF(JCSTAT.EQ.0)THEN
        
                    NRNCOD(JRNCEL) = NCLEAN

                  ELSEIF(JCSTAT.EQ.N2HTOH)THEN

                    NH2HCF = NHTO2H + JQUART
                    NRNCOD(JRNCEL) = NH2HCF

                  ELSE

C                   ERROR CONDITION
                    CONTINUE

                  ENDIF

C                 DIAGNOSTICS
                  IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("StnCCaddLAY",3I5)')NRNLOC(JRNCEL),
     +                                            NRNFAC(JRNCEL),
     +                                            NRNCOD(JRNCEL)
                  ENDIF

                ENDIF
C               FIRST LAYER FACE NEIGHBOURS

C               --------------------------------------------------------------- 

              ELSEIF(KLEVEL.EQ.JLEVEL-1)THEN

C               --------------------------------------------------------------- 

C               h-2h TRANSITION
C               ---------------

C               QUARTET ID
                CALL MCGOCR(MCRUN,MOCTHI)
                JQUART = MCQTBL(MOCTHI,JRFACE)

C               ADD TO STENCIL AND PREVIOUS LAYER
C               EXCEPT PERIODIC BC CELLS
C               RSC 16-APR-2018 UPDATED FOR FULLY PERIODIC BOUNDARIES
C                IF(FLADST)THEN

                  NOSSTN(JCENTC) = NOSSTN(JCENTC) + 1
                  NSSTEN(NOSSTN(JCENTC),JCENTC) = JRCLOC
                  NOLAYR(JCENTC) = NOLAYR(JCENTC) + 1
                  NCLAYR(NOLAYR(JCENTC),JCENTC) = JRCLOC

C                 DIAGNOSTICS
                  IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("Stnh2hadd",6I5)')JRCELL,JCENTC,JRCLOC,
     +                                   JRFACE,JQUART,NOSSTN(JCENTC)
        WRITE(6,'(30I5)')(NSSTEN(JCSTEN,JCENTC),JCSTEN=1,NOSSTN(JCENTC))
        WRITE(6,'(30I5)')(NCLAYR(JCSTEN,JCENTC),JCSTEN=1,NOLAYR(JCENTC))
                  ENDIF

C                ENDIF
C               ADD TO STENCIL AND PREVIOUS LAYER

C               NEAREST-NEIGHBOUR CONNECTIVITY
                IF(FLCONN)THEN

C                 LOWEST QUARTET MEMBER ONLY
C                  IF(JQUART.EQ.0)THEN
                    NXDOOR(JCFACE,JCENTC) = JRCLOC
                    NH2HCF = NHTO2H + JQUART
                    NSCODE(JCENTC) = NSCODE(JCENTC)
     +                             + NH2HCF*NSBASE**(JCFACE-1)
C                  ENDIF
C                 LOWEST QUARTET MEMBER ONLY

                ENDIF
C               NEAREST-NEIGHBOUR CONNECTIVITY

C               FIRST LAYER FACE NEIGHBOURS
                IF(FLFNHL)THEN

C                 FACE NEIGHBOUR HALO LIST
C                 LOWEST QUARTET MEMBER ONLY
C                  IF(JQUART.EQ.0)THEN
                  IF(NXDOOR(JRFACE,JRCLOC).EQ.JRCLOC)THEN
                    JRNCEL = JRNCEL + 1
                    NRNLOC(JRNCEL) = JRCLOC
                    NRNFAC(JRNCEL) = JRFACE
                    NRNCOD(JRNCEL) = N2HTOH
                    NXDOOR(JRFACE,JRCLOC) = JCENTC
                  ENDIF
C                 LOWEST QUARTET MEMBER ONLY

                ENDIF
C               FIRST LAYER FACE NEIGHBOURS

C               --------------------------------------------------------------- 

              ENDIF
C             CHECK THE LEVEL

C             --------------------------------------------------------------- 

            ELSEIF(CFOUND.EQ.CHNRML)THEN

C             --------------------------------------------------------------- 

C             M-CODE IS NOT FOUND AT THIS LEVEL
C             ---------------------------------
C             2h-h TRANSITION

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)THEN
                WRITE(6,*)'Here'
              ENDIF

C             FIND ALL THE FACE-NEIGHBOUR h-CELLS
C              DO IOFSET = 1,4
C                NRQLOC(IOFSET) = 0
C              ENDDO
              DO IOFSET = 1, NFSUB

                CALL MCCOPY(MCRUN,MCTMP)
                MOCTHI = MCFTBL(IOFSET,JCFACE)
                CALL MCHIRE(MCTMP,MOCTHI)

C               SEARCH FOR EXTENDED M-CODE IN THE LAYER-HALO OCTREE
                CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                      MCTMP,KLEVLX,JRCLOC,CFOUND)

C               DIAGNOSTICS
                IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,*)'Here',IOFSET,JRCLOC,CFOUND
                  CALL MCRITE(MCTMP)
                ENDIF

C               CHECK FINDINGS
                IF(CFOUND.EQ.CHTWIG)THEN

C                 2h-h TRANSITION
C                 ---------------

C                 QUARTET ID
                  JQUART = MCQTBL(MOCTHI,JRFACE)

C                 ADD TO STENCIL AND PREVIOUS LAYER
C                 EXCEPT PERIODIC BC CELLS
C                 RSC 16-APR-2018 UPDATED FOR FULLY PERIODIC BOUNDARIES
C                  IF(FLADST)THEN

                    NOSSTN(JCENTC) = NOSSTN(JCENTC) + 1
                    NSSTEN(NOSSTN(JCENTC),JCENTC) = JRCLOC
                    NOLAYR(JCENTC) = NOLAYR(JCENTC) + 1
                    NCLAYR(NOLAYR(JCENTC),JCENTC) = JRCLOC

C                 DIAGNOSTICS
                  IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("Stn2hhadd",6I5)')JRCELL,JCENTC,JRCLOC,
     +                                   JRFACE,JQUART,NOSSTN(JCENTC)
        WRITE(6,'(30I5)')(NSSTEN(JCSTEN,JCENTC),JCSTEN=1,NOSSTN(JCENTC))
        WRITE(6,'(30I5)')(NCLAYR(JCSTEN,JCENTC),JCSTEN=1,NOLAYR(JCENTC))
                  ENDIF

C                  ENDIF
C                 ADD TO STENCIL AND PREVIOUS LAYER

C                 NEAREST-NEIGHBOUR CONNECTIVITY
                  IF(FLCONN)THEN

C                   LOWEST QUARTET MEMBER ONLY
                    IF(JQUART.EQ.0)THEN
                      NXDOOR(JCFACE,JCENTC) = JRCLOC
                      NSCODE(JCENTC) = NSCODE(JCENTC)
     +                               + N2HTOH*NSBASE**(JCFACE-1)
                    ENDIF
C                   LOWEST QUARTET MEMBER ONLY

CC                   SAVE THE QUARTET LOCATION
C                    NRQLOC(IOFSET) = JRCLOC

                  ENDIF
C                 NEAREST-NEIGHBOUR CONNECTIVITY

C                 FIRST LAYER FACE NEIGHBOURS
                  IF(FLFNHL)THEN

C                   FACE NEIGHBOUR HALO LIST
                    JRNCEL = JRNCEL + 1
                    NRNLOC(JRNCEL) = JRCLOC
                    NRNFAC(JRNCEL) = JRFACE
                    NH2HCF = NHTO2H + JQUART
                    NRNCOD(JRNCEL) = NH2HCF

                  ENDIF
C                 FIRST LAYER FACE NEIGHBOURS

                ELSE

C                 ERROR CONDITION: h-CELL NOT FOUND
                  CONTINUE

                ENDIF
C               CHECK FINDINGS

              ENDDO
C             FIND ALL THE FACE-NEIGHBOUR h-CELLS

CC             MAKE THE INTERNAL CONNECTIONS WITHIN THE QUARTET
C              IF(FLCONN)THEN
C
C                DO JQUART = 1, NFSUB
C                  JRCLOC = NRQLOC(JQUART)
C                  KFACE = MCFACE(1,JQUART,JFACE)
C                  KLINK = MCFCON(1,JQUART)
C                  NXDOOR(KFACE,JRCLOC) = NRQLOC(KLINK)
C                  KFACE = MCFACE(2,JQUART,JFACE)
C                  KLINK = MCFCON(2,JQUART)
C                  NXDOOR(KFACE,JRCLOC) = NRQLOC(KLINK)
C                ENDDO
C
C              ENDIF
CC             MAKE THE INTERNAL CONNECTIONS WITHIN THE QUARTET

C             ----------------------------------------------------------------- 

            ELSE

C             ----------------------------------------------------------------- 

C             ERROR CONDITION: h-CELL NOT FOUND
              CONTINUE

C             ----------------------------------------------------------------- 

            ENDIF
C           CHECK FINDINGS
          
          ENDDO
C         RUN THROUGH THE ORIGINAL LIST OF M-CODES IN THE REQUIREMENT LIST

C         =====================================================================

C         CHECK UPDATED STENCILS FOR DUPLICATES AND COMPLETION
          NACELL = NXCELL

C         SORT THE LIST OF CENTRAL CELLS AND REMOVE DUPLICATES
          CALL SORTIN(NCENTC,NREMAX,NACELL)
          CALL REMDIN(NCENTC,NREMAX,NACELL)

C         CHECK STENCILS
          JCNEED = NONEED
          DO JRCELL = 1, NACELL

            JCENTC = NCENTC(JRCELL)

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
        WRITE(6,'("StnUpEnt",3I5)')ILAYER,JRCELL,JCENTC,NOSSTN(JCENTC)
        WRITE(6,'(30I5)')(NSSTEN(JCSTEN,JCENTC),JCSTEN=1,NOSSTN(JCENTC))
        WRITE(6,'(30I5)')(NCLAYR(JCSTEN,JCENTC),JCSTEN=1,NOLAYR(JCENTC))
            ENDIF

            NSOFAR = NOSSTN(JCENTC)
            DO JCSTEN = 1, NSOFAR
              NCSTMP(JCSTEN) = NSSTEN(JCSTEN,JCENTC)
            ENDDO
            CALL SORTIN(NCSTMP,NTSTMX,NSOFAR)
            CALL REMDIN(NCSTMP,NTSTMX,NSOFAR)
            DO JCSTEN = 1, NSOFAR
              NSSTEN(JCSTEN,JCENTC) = NCSTMP(JCSTEN)
            ENDDO
            NOSSTN(JCENTC) = NSOFAR

C           CHECK FOR STENCIL COMPLETION
            IF(NSOFAR.GE.NRQSTN)THEN

C             STENCIL COMPLETE
              DO JCSTEN = 1, NSOFAR
                JLCELL = NCSTMP(JCSTEN)
C                IF(JLCELL.LE.NCELL)THEN
                CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JLCELL)
C                ELSE
C                  CALL MCRTRV(MCRUN,NXCODE,NREMAX,JLCELL)
C                ENDIF
                CALL MCSTOR(MCRUN,NLCODE,NTSTMX,JCSTEN)
              ENDDO
              CALL SORTEM(NLCODE,NCSTMP,NTSTMX,NSOFAR)
              DO JCSTEN = 1, NSOFAR
                NSSTEN(JCSTEN,JCENTC) = NCSTMP(JCSTEN)
              ENDDO
              NOSSTN(JCENTC) = NSOFAR

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'("StnComp",3I5)')JRCELL,JCENTC,NOSSTN(JCENTC)
        WRITE(6,'(30I5)')(NSSTEN(JCSTEN,JCENTC),JCSTEN=1,NOSSTN(JCENTC))
              ENDIF

            ELSE

C             STENCIL INCOMPLETE
              NSOFAR = NOLAYR(JCENTC)
              DO JCSTEN = 1, NSOFAR
                NCSTMP(JCSTEN) = NCLAYR(JCSTEN,JCENTC)
              ENDDO
              CALL SORTIN(NCSTMP,NTSTMX,NSOFAR)
              CALL REMDIN(NCSTMP,NTSTMX,NSOFAR)
              DO JCSTEN = 1, NSOFAR
                NCLAYR(JCSTEN,JCENTC) = NCSTMP(JCSTEN)
              ENDDO
              NOLAYR(JCENTC) = NSOFAR

C             UPDATE THE LIST OF CELLS WITH INCOMPLETE STENCILS
              JCNEED = JCNEED + 1
              NCNEED(JCNEED) = JCENTC

C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)THEN
              WRITE(6,'("StnInComp",4I5)')JRCELL,JCENTC,NOSSTN(JCENTC),
     +                                                  NOLAYR(JCENTC)
        WRITE(6,'(30I5)')(NSSTEN(JCSTEN,JCENTC),JCSTEN=1,NOSSTN(JCENTC))
        WRITE(6,'(30I5)')(NCLAYR(JCSTEN,JCENTC),JCSTEN=1,NOLAYR(JCENTC))
              ENDIF

            ENDIF

          ENDDO
          NONEED = JCNEED

C         SORT THE LIST OF CELLS WITH INCOMPLETE STENCILS AND REMOVE DUPLICATES
          CALL SORTIN(NCNEED,NREMAX,NONEED)
          CALL REMDIN(NCNEED,NREMAX,NONEED)

C         REMOVE CELLS WITH COMPLETED STENCILS
          JCCELL = 0
          DO JRCELL = 1, NONEED
            JCENTC = NCNEED(JRCELL)
            NSOFAR = NOSSTN(JCENTC)
            IF(NSOFAR.LT.NRQSTN)THEN
              JCCELL = JCCELL + 1
              NCNEED(JCCELL) = JCENTC
            ENDIF
          ENDDO
          NONEED = JCCELL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            DO JRCELL = 1, NONEED
              JCENTC = NCNEED(JRCELL)
              WRITE(6,'("StnNeed",3I5)')JRCELL,JCENTC,NOSSTN(JCENTC)
        WRITE(6,'(30I5)')(NSSTEN(JCSTEN,JCENTC),JCSTEN=1,NOSSTN(JCENTC))
        WRITE(6,'(30I5)')(NCLAYR(JCSTEN,JCENTC),JCSTEN=1,NOLAYR(JCENTC))
            ENDDO
          ENDIF

C         =====================================================================

C         DEAL WITH THE FACE NEIGHBOUR HALO CELLS
          IF(ILAYER.EQ.1)THEN

C           NUMBER OF FACE NEIGHBOUR HALO CELLS
            NRNCEL = NRNCEL + JRNCEL

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              DO JRNCEL = 1, NRNCEL
                WRITE(6,'("FNH1",7I5)')IPROC,JRNCEL,NRNLOC(JRNCEL),
     +                         NRNFAC(JRNCEL),NRNCOD(JRNCEL)
                WRITE(6,'("FNH2",8I5)')IPROC,
     +          (NXDOOR(JFACE,NRNLOC(JRNCEL)),JFACE=1,6),
     +          NSCODE(NRNLOC(JRNCEL))
              ENDDO
            ENDIF

C           SORT THE FACE NEIGHBOUR HALO CELLS
            CALL SORTI3(NRNLOC,NRNFAC,NRNCOD,NREMAX,NRNCEL)

C           ADD TO LIST OF CELLS WITH INCOMPLETE STENCILS
            JRCOLD = 0
            DO JRNCEL = 1, NRNCEL
              JRCLOC = NRNLOC(JRNCEL)
              IF(JRCLOC.NE.JRCOLD)THEN
                NOSSTN(JRCLOC) = 0
                NOLAYR(JRCLOC) = 1
                NCLAYR(1,JRCLOC) = JRCLOC
                NONEED = NONEED + 1
                NCNEED(NONEED) = JRCLOC
                JRCOLD = JRCLOC
              ENDIF
            ENDDO

          ELSE

C           DIAGNOSTICS
            IF(JRNCEL.NE.0)
     +        WRITE(6,'("FNH warning",3I5)')IPROC,ILAYER,JRNCEL

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)THEN
              DO JRNCEL = 1, NRNCEL
                WRITE(6,'("FNH1",7I5)')IPROC,JRNCEL,NRNLOC(JRNCEL),
     +                         NRNFAC(JRNCEL),NRNCOD(JRNCEL)
C                WRITE(6,'("FNH2",8I5)')IPROC,
C     +          (NXDOOR(JFACE,NRNLOC(JRNCEL)),JFACE=1,6),
C     +          NSCODE(NRNLOC(JRNCEL))
              ENDDO
            ENDIF

          ENDIF

C         =====================================================================

        ENDIF
C       DEAL WITH REMOTE CELLS IN THE CURRENT LAYER

C       =======================================================================

C       MAKE AN OCTREE BASED ON THE HALO M-CODE LIST
C       --------------------------------------------

C       TOTAL NUMBER OF CELLS IN THE HALO-SO-FAR
        NRCELL = NCELHW - NCELL

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          DO JRCELL = NCELP1,NCELHW

            ISCODE = NSCODE(JRCELL)
            DO JOLAYR = 1,6
              IFCODE(JOLAYR) = MOD(ISCODE,NSBASE)
              ISCODE = ISCODE/NSBASE
            ENDDO

            WRITE(6,'("HCD",13I5)')JRCELL,
     +      (NXDOOR(JFACE,JRCELL),JFACE=1,6),
     +      (IFCODE(JOLAYR),JOLAYR=1,6)
          ENDDO
        ENDIF

C       FETCH THE M-CODES FROM THE HALO-SO-FAR
        CALL MCMTRV(NHCODE,NREMAX,NMCODE,NSIZMX,NCELP1,NCELHW)

C       MAKE THE HALO-SO-FAR INDIRECTION LIST
        DO JRCELL = 1, NRCELL
          NRCLOC(JRCELL) = NCELL+JRCELL
        ENDDO

C       SORT THE M-CODES
        CALL SORTEM(NHCODE,NRCLOC,NREMAX,NRCELL)

C       MAKE THE HALO-SO-FAR OCTREE
        CALL LHTREE(NHCODE,NREMAX,NRCELL,
     +              NHCPAR,NHCHLD,CHCODE,NHCTMX,NHCHWM)
      
C       =======================================================================

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)WRITE(6,*)NONEED

C       CHECK MAXIMUM NUMBER OF LAYERS
        IF(ILAYER.LT.NRQLAY)THEN

C         GO ROUND AGAIN
          ILAYER = ILAYER + 1
          GOTO 1000

        ELSE

C         CHECK NUMBER OF INCOMPLETE STENCILS
          IF(NONEED.GT.0)THEN

C         ERROR CONDITION
          WRITE(6,*)'Warning: more layers required',IPROC,ILAYER,NONEED

          ENDIF

        ENDIF
C       CHECK NUMBER OF LAYERS

C       =======================================================================

C     END OF LOOP 1000 OVER LAYERS

C     STORE NUMBER OF BC INDICES
      NBCVAL = JBCVAL

C     =========================================================================

C     EXCHANGE DATA ABOUT ALL THE HALO CELLS
C     --------------------------------------

C     TOTAL NUMBER OF CELLS IN THE HALO
      NRCELL = NCELHW - NCELL

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        DO JRCELL = NCELP1,NCELHW
          WRITE(6,'("HCD",8I5)')JRCELL,
     +    (NXDOOR(JFACE,JRCELL),JFACE=1,6),
     +    NSCODE(JRCELL)
        ENDDO
      ENDIF

C     FETCH THE M-CODES FROM THE COMPLETE HALO
      CALL MCMTRV(NACODE,NREMAX,NMCODE,NSIZMX,NCELP1,NCELHW)

C     MAKE THE HALO CELL INDIRECTION LIST
      DO JRCELL = 1, NRCELL
        NRCLOC(JRCELL) = NCELL+JRCELL
      ENDDO

C     SORT THE M-CODES
      CALL SORTEM(NACODE,NRCLOC,NREMAX,NRCELL)

C     EXCHANGE DATA ABOUT THE HALO CELLS
      CALL PARCEL(NACODE)

C     =========================================================================

C     ENSURE 2h-h NEIGHBOUR-CONNECTIVITY OF THE FACE NEIGHBOUR HALO CELLS
C     -------------------------------------------------------------------

C     MAKE THE HALO-COMPLETED OCTREE
      CALL LHTREE(NACODE,NREMAX,NRCELL,
     +            NHCPAR,NHCHLD,CHCODE,NHCTMX,NHCHWM)
      
C     RUN THROUGH THE FACE-NEIGHBOUR HALO CELLS
      JRCOLD = 0
      DO JRNCEL = 1, NRNCEL

C       FOR EACH FACE-NEIGHBOUR HALO CELL
        JRCLOC = NRNLOC(JRNCEL)
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,*)'2hhstart',JRNCEL,JRCLOC,JRCOLD
        ENDIF
        IF(JRCLOC.NE.JRCOLD)THEN

C         CHECK CELL FACE STATUS
          IF(NSCODE(JRCLOC).NE.NCLEAN)THEN

C           CELL FACE STATUS IS NOT CLEAN
            ISCODE = NSCODE(JRCLOC)
            DO JCFACE = 1,NFACE

              IFCODE(JCFACE) = MOD(ISCODE,NSBASE)
              ISCODE = ISCODE/NSBASE

C             CHECK FOR 2h-h TRANSITION
              IF(IFCODE(JCFACE).EQ.N2HTOH)THEN

C               FETCH THE M-CODE
                CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JRCLOC)

C               GET THE LEVEL
                CALL MCGLVL(MCRUN,JLEVEL)

C               GET THE CELL INTEGER COORDINATES 
                CALL MCCXYZ(MCRUN,JLEVEL,MPLOC(1),MPLOC(2),MPLOC(3))

C               INCREMENT THE CELL COORDINATES
                MPNXT(1) = MPLOC(1) + MATINC(1,JCFACE)
                MPNXT(2) = MPLOC(2) + MATINC(2,JCFACE)
                MPNXT(3) = MPLOC(3) + MATINC(3,JCFACE)

C               ASSEMBLE THE NEIGHBOUR M-CODE
                CALL MCXYZC(MCNXT,JLEVEL,MPNXT(1),MPNXT(2),MPNXT(3))

C               OPPOSITE FACE CODE
                JRFACE = MAPOPP(JCFACE)

C               FIND ALL THE FACE-NEIGHBOUR h-CELLS
                DO IOFSET = 1,4
                  NRQLOC(IOFSET) = 0
                ENDDO
                DO IOFSET = 1, NFSUB

                  CALL MCCOPY(MCNXT,MCTMP)
                  MOCTHI = MCFTBL(IOFSET,JCFACE)
                  CALL MCHIRE(MCTMP,MOCTHI)

C                 SEARCH FOR EXTENDED M-CODE IN THE LAYER-HALO OCTREE
                  CALL OCFIND(NHCHLD,CHCODE,NHCTMX,
     +                        MCTMP,KLEVLX,KCELLH,CFOUND)

C                 CHECK FINDINGS
                  IF(CFOUND.EQ.CHTWIG)THEN

C                   2h-h TRANSITION
C                   ---------------

C                   SAVE THE QUARTET LOCATION
                    KCELHH = NRCLOC(KCELLH)
                    NRQLOC(IOFSET) = KCELHH

C                   DIAGNOSTICS
                    IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,*)'2hhH',JRCLOC,JCFACE,IOFSET,KCELHH,CFOUND
                      CALL MCRITE(MCTMP)
                    ENDIF

                  ELSE

C                   SEARCH FOR EXTENDED M-CODE IN THE MAIN OCTREE
                    CALL OCFIND(NOCHLD,COCODE,NOCTMX,
     +                          MCTMP,KLEVLX,KCELLH,CFOUND)

C                   CHECK FINDINGS
                    IF(CFOUND.EQ.CHTWIG)THEN

C                     2h-h TRANSITION
C                     ---------------

C                     SAVE THE QUARTET LOCATION
                      NRQLOC(IOFSET) = KCELLH

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,*)'2hhM',JRCLOC,JCFACE,IOFSET,KCELLH,CFOUND
                        CALL MCRITE(MCTMP)
                      ENDIF

                    ELSE

C                     ERROR CONDITION
                      CONTINUE

C                     DIAGNOSTICS
                      IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,*)'2hhErr',JRCLOC,JCFACE,IOFSET,CFOUND
                        CALL MCRITE(MCTMP)
                      ENDIF

                    ENDIF

                  ENDIF
C                 CHECK FINDINGS

                ENDDO
C               FIND ALL THE FACE-NEIGHBOUR h-CELLS

C               MAKE THE INTERNAL CONNECTIONS WITHIN THE QUARTET
                NSDIRN = NFSUB/2
                DO JQUART = 1, NFSUB
                  JCENTC = NRQLOC(JQUART)
                  DO JDIRN = 1, NSDIRN
                    KFACE = MCFACE(JDIRN,JQUART,JCFACE)
                    KLINK = MCFCON(JDIRN,JQUART)
                    NXDOOR(KFACE,JCENTC) = NRQLOC(KLINK)
                    IF(IPROC.EQ.LPROC)THEN
                    WRITE(6,'("2hhIn1",12I5)')JQUART,JCENTC,JDIRN,
     +                 KFACE,KLINK,NRQLOC(KLINK),
     +                 (NXDOOR(JFACE,JCENTC),JFACE=1,6)
                    ENDIF
                  ENDDO
C                  KFACE = MCFACE(2,JQUART,JCFACE)
C                  KLINK = MCFCON(2,JQUART)
C                  NXDOOR(KFACE,JCENTC) = NRQLOC(KLINK)
C                  IF(IPROC.EQ.LPROC)THEN
C                    WRITE(6,'("2hhIn2",11I5)')JQUART,JCENTC,KFACE,KLINK,
C     +                 NRQLOC(KLINK),
C     +                 (NXDOOR(JFACE,JCENTC),JFACE=1,6)
C                  ENDIF
                ENDDO

              ENDIF
C             CHECK FOR 2h-h TRANSITION
  
            ENDDO
C           RUN THROUGH ALL FACES

          ENDIF
C         CHECK CELL FACE STATUS

          JRCOLD = JRCLOC

        ENDIF
C       FOR EACH FACE-NEIGHBOUR HALO CELL

      ENDDO
C     RUN THROUGH THE FACE-NEIGHBOUR HALO CELLS

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        DO JRNCEL = 1, NRNCEL
          WRITE(6,'("FNH1",7I5)')IPROC,JRNCEL,NRNLOC(JRNCEL),
     +                   NRNFAC(JRNCEL),NRNCOD(JRNCEL)
                WRITE(6,'("FNH2",8I5)')IPROC,
     +          (NXDOOR(JFACE,NRNLOC(JRNCEL)),JFACE=1,6),
     +          NSCODE(NRNLOC(JRNCEL))
        ENDDO
      ENDIF

C     =========================================================================

C     COMPUTE THE CELL CENTRE COORDINATES
C     AND STORE THE LEVEL
      DO JCELL = 1, NCELL+NRCELL

        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
        CALL MCGLVL(MCRUN,JLEVEL)
        CALL MCCXYZ(MCRUN,JLEVEL,MPLOC(1),MPLOC(2),MPLOC(3))

        XCENTR(JCELL) = (REAL(MPLOC(1))+HALF)*HCSIZE(JLEVEL)
        YCENTR(JCELL) = (REAL(MPLOC(2))+HALF)*HCSIZE(JLEVEL)
        ZCENTR(JCELL) = (REAL(MPLOC(3))+HALF)*HCSIZE(JLEVEL)
        NLEVEL(JCELL) = JLEVEL

      ENDDO
      IF(NDIMS.EQ.2)THEN
        DO JCELL = 1, NCELL+NRCELL
          ZCENTR(JCELL) = HALF*HCSIZE(NBLEVL)
        ENDDO
      ENDIF
      IF(NDIMS.EQ.1)THEN
        DO JCELL = 1, NCELL+NRCELL
          YCENTR(JCELL) = HALF*HCSIZE(NBLEVL)
          ZCENTR(JCELL) = HALF*HCSIZE(NBLEVL)
        ENDDO
      ENDIF

C     =========================================================================

C     CATEGORISE THE STENCILS
      CALL STNCAT

C     =========================================================================

CC     DIAGNOSTICS
C      CALL STNREP
C      IF(ITIME.GT.0)CALL STNREP

C     =========================================================================

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        DO JCELL = 1, NCELL
C          CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
C          CALL MCGLVL(MCRUN,JLEVEL)
C          CALL MCCXYZ(MCRUN,JLEVEL,MPLOC(1),MPLOC(2),MPLOC(3))
C          WRITE(6,*)JCELL,MPLOC(1)+1,MPLOC(2)+1,MPLOC(3)+1,NOSSTN(JCELL)
C        ENDDO
C        DO JCELL = 1, NCELL
C          NOLAYR(JCELL) = JCELL
C          NCNEED(JCELL) = NOSSTN(JCELL)
C        ENDDO
C        CALL SORTI2(NCNEED,NOLAYR,NSIZMX,NCELL)
C        DO JCELL = 1, NCELL
C          WRITE(6,*)NCNEED(JCELL),NOLAYR(JCELL)
C        ENDDO
C        JCNEED = NCNEED(1)
C        JCSTEN = 1
C        DO JCELL = 2, NCELL
C          IF(NCNEED(JCELL).EQ.NCNEED(JCELL-1))THEN
C            JCSTEN = JCSTEN + 1
C          ELSE
C            WRITE(6,*)NCNEED(JCELL-1),JCSTEN
C            JCSTEN = 1
C          ENDIF
C        ENDDO
C        WRITE(6,*)NCNEED(NCELL),JCSTEN
C      ENDIF

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'End of STNCON'


      RETURN
      END
