      SUBROUTINE TEMPRM
 
C     *************************************************************************
C
C     TEMPRM
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     16-MAR-2008:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES TEMPERATURE, PRESSURE AND PRMITIVE VARIABLES
C     FOR USE IN (DE)REFINEMENT CRITERIA
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION TOLTMP
      PARAMETER(TOLTMP = 1.0E-6)
      INTEGER NTITRS
      PARAMETER(NTITRS = 100)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION TCOEFF(0:NCTMAX),TDERIV(1:NCTMAX)
      DOUBLE PRECISION TEMPOR,TFPOLY,TDPOLY,DELTMP
      DOUBLE PRECISION UKUK,RGCMIX,OVDENS
      INTEGER JCELL,ICP,ISPEC,ITINT,ITITRS


C     BEGIN
C     =====

C     =========================================================================

C     RUN THROUGH ALL CELLS
C     INCLUDING HALO CELLS
      DO JCELL = 1, NCELL+NRCELL

C       =======================================================================

C       KINETIC ENERGY
        UKUK = HALF*(URHS(1,JCELL)*URHS(1,JCELL)
     +       +       URHS(2,JCELL)*URHS(2,JCELL)
     +       +       URHS(3,JCELL)*URHS(3,JCELL))/DRHS(JCELL)

C       =======================================================================

C       INITIALISE COEFFICIENTS OF TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
        TCOEFF(0) = UKUK - ERHS(JCELL)
        DO ICP = 1, NCTMAX
          TCOEFF(ICP) = ZERO
          TDERIV(ICP) = ZERO
        ENDDO

C       =======================================================================

C       TEMPERATURE POLYNOMIAL
        DO ISPEC = 1,NSPEC

C         =====================================================================

C         LOCATE EXISTING TEMPERATURE IN AN INTERVAL
          ITINT = 1

C         =====================================================================

C         COEFFICIENTS OF TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
          TCOEFF(0) = TCOEFF(0)
     +      + YRHS(ISPEC,JCELL)*AMASCE(NCENTH(ITINT,ISPEC),ITINT,ISPEC)
          DO ICP = 1, NCPOLY(ITINT,ISPEC)
            TCOEFF(ICP) = TCOEFF(ICP)
     +                  + YRHS(ISPEC,JCELL)*AMASCE(ICP,ITINT,ISPEC)
            TDERIV(ICP) = TDERIV(ICP)
     +                  + YRHS(ISPEC,JCELL)*AMASCV(ICP,ITINT,ISPEC)
          ENDDO

C         =====================================================================

        ENDDO
C       END OF TEMPERATURE POLYNOMIAL

C       =======================================================================

C       INITIAL GUESS FOR TEMPERATURE
        TEMPOR = -TCOEFF(0)/TCOEFF(1)

C       =======================================================================

C       SOLVE FOR TEMPERATURE USING NEWTON-RAPHSON
C       ---------------------
        ITITRS = 1
1000    CONTINUE

C         EVALUATE TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
          TFPOLY = TCOEFF(NCTMAX)
          TDPOLY = TDERIV(NCTMAX)
          DO ICP = NCTMM1,1,-1
            TFPOLY = TCOEFF(ICP) + TFPOLY*TEMPOR
            TDPOLY = TDERIV(ICP) + TDPOLY*TEMPOR
          ENDDO
          TFPOLY = TCOEFF(0) + TFPOLY*TEMPOR

C         EVALUATE TEMPERATURE CORRECTION
          DELTMP = -TFPOLY/TDPOLY

C         CHECK FOR CONVERGENCE
          IF(ABS(DELTMP).GT.TOLTMP)THEN
            IF(ITITRS.LT.NTITRS)THEN
              TEMPOR = TEMPOR + DELTMP
              ITITRS = ITITRS + 1
              GOTO 1000
            ELSE

C             ERROR CONDITION - T iteration failed to converge
            WRITE(6,*)'ERROR CONDITION - T iteration failed to converge'
              WRITE(6,'(4I5,1PE15.7)')ITIME,IRKSTP,JCELL,ITITRS,TEMPOR
              WRITE(6,'(I5,4(1PE15.7))')JCELL,
     +                DRHS(JCELL),URHS(1,JCELL),ERHS(JCELL),TRUN(JCELL)
              DO ISPEC = 1, NSPEC
                WRITE(6,'(5X,1PE15.7)')YRHS(ISPEC,JCELL)
              ENDDO

              CALL ERHAND(9,19,4)

            ENDIF
          ENDIF

C       END OF LOOP 1000

C       =======================================================================

C       SET THE NEW TEMPERATURE
        TRUN(JCELL) = TEMPOR

C       =======================================================================

C       PRESSURE
        RGCMIX = ZERO
        DO ISPEC = 1,NSPEC
          RGCMIX = RGCMIX + YRHS(ISPEC,JCELL)*RGSPEC(ISPEC)
        ENDDO
        PRUN(JCELL) = RGCMIX*TRUN(JCELL) - PREFER

C       =======================================================================

C       PRIMITIVE VARIABLES
        DSOR(JCELL) = DRHS(JCELL)
        OVDENS = ONE/DSOR(JCELL)
        USOR(1,JCELL) = URHS(1,JCELL)*OVDENS
        USOR(2,JCELL) = URHS(2,JCELL)*OVDENS
        USOR(3,JCELL) = URHS(3,JCELL)*OVDENS
        ESOR(JCELL) = ERHS(JCELL)*OVDENS
        DO ISPEC = 1,NSPEC
          YSOR(ISPEC,JCELL) = YRHS(ISPEC,JCELL)*OVDENS
        ENDDO

C       =======================================================================

      ENDDO
C     END OF RUN THROUGH ALL CELLS

C     =========================================================================


      RETURN
      END
