      SUBROUTINE REFVAL
 
C     *************************************************************************
C
C     REFVAL
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     12-JUN-2014:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     UPDATE THE CELL VALUES FOLLOWING THE DE/REFINEMENT PROCESS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YREFIN(NSPCMX,0:7)
      DOUBLE PRECISION UREFIN(NCMPMX,0:7)
      DOUBLE PRECISION DREFIN(0:7),EREFIN(0:7)
      DOUBLE PRECISION YEVAL(NSPCMX)
      DOUBLE PRECISION UEVAL(NCMPMX)
      DOUBLE PRECISION DEVAL,EEVAL
      DOUBLE PRECISION CXSGN,CYSGN,CZSGN

      INTEGER JOFCEL(0:7)
      INTEGER JCELL,JSORT,JSPEC,JOCELL,JOFSET,KOFSET
      LOGICAL FLDRTN,FLRPAR

C     DIAGNOSTICS
      INTEGER LPROC
      INTEGER JCSTEN,JTHING

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In REFVAL'


C     BEGIN
C     =====

C     =========================================================================

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        DO JCELL = 1,NCELL
C          WRITE(6,'(3I5)')JCELL,NCOCEL(JCELL),NRSTAT(JCELL)
C        ENDDO
C      ENDIF

C     REFINEMENT FLAGS
      FLDRTN = .FALSE.
      FLRPAR = .FALSE.

C     USE THE INDIRECTION LIST TO SORT THE CELL DATA
      DO JCELL = 1, NCELL

C       -----------------------------------------------------------------------

C       CHECK THE CELL REFINEMENT STATUS CODE
        IF(NRSTAT(JCELL).EQ.NRNULL)THEN

C         ---------------------------------------------------------------------

C         CELL UNCHANGED

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,*)JCELL,NCOCEL(JCELL),'un'

C         RESTORE THE UNREFINED CELL VALUES
          JSORT = NCOCEL(JCELL)
          DRUN(JCELL) = DRHS(JSORT)
          URUN(1,JCELL) = URHS(1,JSORT)
          URUN(2,JCELL) = URHS(2,JSORT)
          URUN(3,JCELL) = URHS(3,JSORT)
          ERUN(JCELL) = ERHS(JSORT)
          DO JSPEC = 1,NSPEC
            YRUN(JSPEC,JCELL) = YRHS(JSPEC,JSORT)
          ENDDO

C         ---------------------------------------------------------------------

        ELSE IF(NRSTAT(JCELL).EQ.NRDPAR)THEN

C         ---------------------------------------------------------------------

C         DEREFINED PARENT CELL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,*)JCELL,NCOCEL(JCELL),'dp'

C         AVERAGE THE DEREFINED CELL VALUES

C         PARENT CELL
          JSORT = NCOCEL(JCELL)
          DRUN(JCELL) = DRHS(JSORT)
          URUN(1,JCELL) = URHS(1,JSORT)
          URUN(2,JCELL) = URHS(2,JSORT)
          URUN(3,JCELL) = URHS(3,JSORT)
          ERUN(JCELL) = ERHS(JSORT)
          DO JSPEC = 1,NSPEC
            YRUN(JSPEC,JCELL) = YRHS(JSPEC,JSORT)
          ENDDO

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,'(I5,1PE17.9)')JSORT,ERHS(JSORT)

C         CHILD CELLS
          DO JOFSET = 1, NOFSET

            JOCELL = JSORT + JOFSET
            DRUN(JCELL) = DRUN(JCELL) + DRHS(JOCELL)
            URUN(1,JCELL) = URUN(1,JCELL) + URHS(1,JOCELL)
            URUN(2,JCELL) = URUN(2,JCELL) + URHS(2,JOCELL)
            URUN(3,JCELL) = URUN(3,JCELL) + URHS(3,JOCELL)
            ERUN(JCELL) = ERUN(JCELL) + ERHS(JOCELL)
            DO JSPEC = 1,NSPEC
              YRUN(JSPEC,JCELL) = YRUN(JSPEC,JCELL) + YRHS(JSPEC,JOCELL)
            ENDDO

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)WRITE(6,'(I5,1PE17.9)')JOCELL,ERHS(JOCELL)

          ENDDO

C         AVERAGE PARENT CELL VALUE
          DRUN(JCELL) = DRUN(JCELL)*DREFAV
          URUN(1,JCELL) = URUN(1,JCELL)*DREFAV
          URUN(2,JCELL) = URUN(2,JCELL)*DREFAV
          URUN(3,JCELL) = URUN(3,JCELL)*DREFAV
          ERUN(JCELL) = ERUN(JCELL)*DREFAV
          DO JSPEC = 1,NSPEC
            YRUN(JSPEC,JCELL) = YRUN(JSPEC,JCELL)*DREFAV
          ENDDO

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,'(I5,1PE17.9)')JCELL,ERUN(JCELL)


C         ---------------------------------------------------------------------

        ELSE IF(NRSTAT(JCELL).EQ.NRDRTN)THEN

C         ---------------------------------------------------------------------

C         RETAINED CELL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,*)JCELL,NCOCEL(JCELL),'rt'

C         RESTORE THE ORIGINAL UNREFINED CELL VALUES
          JSORT = NCOCEL(JCELL)
          JOFSET = 0
          DRUN(JCELL) = DRHS(JSORT)
          URUN(1,JCELL) = URHS(1,JSORT)
          URUN(2,JCELL) = URHS(2,JSORT)
          URUN(3,JCELL) = URHS(3,JSORT)
          ERUN(JCELL) = ERHS(JSORT)
          DO JSPEC = 1,NSPEC
            YRUN(JSPEC,JCELL) = YRHS(JSPEC,JSORT)
          ENDDO

CC         DIAGNOSTICS
C          IF(IPROC.EQ.LPROC)WRITE(6,'(2I5,2(1PE17.9))')
C     +                      JSORT,JOFSET,DRHS(JCELL),ERHS(JCELL)

          FLDRTN = .TRUE.

C         ---------------------------------------------------------------------

        ELSE IF(NRSTAT(JCELL).EQ.NRRPAR)THEN

C         ---------------------------------------------------------------------

C         REFINED PARENT CELL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,*)JCELL,NCOCEL(JCELL),'rp'

C         GENERATE NEW CELL VALUES

C         NEW CELL OFFSET
          JOFSET = 0
          JOFCEL(JOFSET) = JCELL

C         CELL INDEX
          ICELL = NCOCEL(JCELL)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            DO JCSTEN = 1, NOSSTN(ICELL)
              JTHING = NSSTEN(JCSTEN,ICELL)
              WRITE(6,'(2I6,3(1PE17.9))')JCSTEN,NSSTEN(JCSTEN,ICELL),
     +                  URHS(1,JTHING),URHS(2,JTHING),URHS(3,JTHING)
            ENDDO
C            WRITE(6,'(6I5)')(NXDOOR(JCSTEN,ICELL),JCSTEN=1,6)
          ENDIF

C         RECONSTRUCTION
          CALL CREPOR

C         EVALUATE THE POLYNOMIAL
          CXSGN = OFFSGN(1,JOFSET)
          CYSGN = OFFSGN(2,JOFSET)
          CZSGN = OFFSGN(3,JOFSET)

          CALL CEVALR(CXSGN,CYSGN,CZSGN,DEVAL,UEVAL,EEVAL,YEVAL)

          DREFIN(JOFSET) = DEVAL
          UREFIN(1,JOFSET) = UEVAL(1)
          UREFIN(2,JOFSET) = UEVAL(2)
          UREFIN(3,JOFSET) = UEVAL(3)
          EREFIN(JOFSET) = EEVAL
          DO JSPEC = 1,NSPEC
            YREFIN(JSPEC,JOFSET) = YEVAL(JSPEC)
          ENDDO

C         DIAGNOSTICS
          JSORT = NCOCEL(JCELL)
C          IF(IPROC.EQ.LPROC)WRITE(6,'(2I5,1PE12.4,2(1PE17.9))')
C     +                      JSORT,JOFSET,CXSGN,DEVAL,EEVAL
C            IF(IPROC.EQ.LPROC)WRITE(6,'(2I5,2(1PE12.4),2(1PE17.9))')
C     +                        JSORT,JOFSET,CXSGN,CYSGN,UEVAL(1),UEVAL(2)
          IF(IPROC.EQ.LPROC)WRITE(6,'(2I5,3(F5.1),3(1PE17.9))')
     +                  JSORT,JOFSET,CXSGN,CYSGN,CZSGN,
     +                  UEVAL(1),UEVAL(2),UEVAL(3)

          FLRPAR = .TRUE.

C         ---------------------------------------------------------------------

        ELSE IF(NRSTAT(JCELL).EQ.NRRCHL)THEN

C         ---------------------------------------------------------------------

C         REFINED CHILD CELL

C         NEW CELL
          IF(FLRPAR)THEN

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)WRITE(6,*)JCELL,ICELL,'rcn'

C           GENERATE NEW CELL VALUES

C           NEW CELL OFFSET
            JOFSET = JOFSET + 1
            JOFCEL(JOFSET) = JCELL

C           EVALUATE THE POLYNOMIAL
            CXSGN = OFFSGN(1,JOFSET)
            CYSGN = OFFSGN(2,JOFSET)
            CZSGN = OFFSGN(3,JOFSET)

            CALL CEVALR(CXSGN,CYSGN,CZSGN,DEVAL,UEVAL,EEVAL,YEVAL)

            DREFIN(JOFSET) = DEVAL
            UREFIN(1,JOFSET) = UEVAL(1)
            UREFIN(2,JOFSET) = UEVAL(2)
            UREFIN(3,JOFSET) = UEVAL(3)
            EREFIN(JOFSET) = EEVAL
            DO JSPEC = 1,NSPEC
              YREFIN(JSPEC,JOFSET) = YEVAL(JSPEC)
            ENDDO

C           DIAGNOSTICS
            JSORT = ICELL
            IF(IPROC.EQ.LPROC)WRITE(6,'(2I5,3(F5.1),3(1PE17.9))')
     +                    JSORT,JOFSET,CXSGN,CYSGN,CZSGN,
     +                    UEVAL(1),UEVAL(2),UEVAL(3)

C           LAST NEW CELL
            IF(JOFSET.EQ.NOFSET)THEN

C             UPDATE CELL VALUES
              DO KOFSET = 0, NOFSET

                JOCELL = JOFCEL(KOFSET)
                DRUN(JOCELL) = DREFIN(KOFSET)
                URUN(1,JOCELL) = UREFIN(1,KOFSET)
                URUN(2,JOCELL) = UREFIN(2,KOFSET)
                URUN(3,JOCELL) = UREFIN(3,KOFSET)
                ERUN(JOCELL) = EREFIN(KOFSET)
                DO JSPEC = 1,NSPEC
                  YRUN(JSPEC,JOCELL) = YREFIN(JSPEC,KOFSET)
                ENDDO

              ENDDO

              FLRPAR = .FALSE.

            ENDIF

          ELSEIF(FLDRTN)THEN

C           DIAGNOSTICS
            IF(IPROC.EQ.LPROC)WRITE(6,*)JCELL,NCOCEL(JCELL),'rcr'

C           RESTORE THE ORIGINAL UNREFINED CELL VALUES
            JOFSET = JOFSET + 1
            JOCELL = JSORT + JOFSET

            DRUN(JCELL) = DRHS(JOCELL)
            URUN(1,JCELL) = URHS(1,JOCELL)
            URUN(2,JCELL) = URHS(2,JOCELL)
            URUN(3,JCELL) = URHS(3,JOCELL)
            ERUN(JCELL) = ERHS(JOCELL)
            DO JSPEC = 1,NSPEC
              YRUN(JSPEC,JCELL) = YRHS(JSPEC,JOCELL)
            ENDDO

CC           DIAGNOSTICS
C            IF(IPROC.EQ.LPROC)WRITE(6,'(2I5,2(1PE17.9))')
C     +                        JSORT,JOFSET,DRHS(JCELL),ERHS(JCELL)

            IF(JOFSET.EQ.NOFSET)FLDRTN = .FALSE.

          ELSE

C           ERROR STATUS
C           DIAGNOSTICS
            WRITE(6,*)'REFVAL: cell refinement flag error',
     +                FLRPAR,FLDRTN

          ENDIF

C         ---------------------------------------------------------------------

        ELSE

C         ---------------------------------------------------------------------

C         ERROR STATUS
C         DIAGNOSTICS
          WRITE(6,*)'REFVAL: cell refinement status error',
     +               JCELL,NRSTAT(JCELL)

C         ---------------------------------------------------------------------

        ENDIF

C       -----------------------------------------------------------------------

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)THEN
C          WRITE(6,'(2I5)')JCELL,NRSTAT(JCELL)
C          WRITE(6,'(4(1PE12.4))')DRHS(JCELL),
C     +              URHS(1,JCELL),URHS(2,JCELL),URHS(3,JCELL)
C          WRITE(6,'(3(1PE12.4))')ERHS(JCELL),YRHS(1,JCELL),YRHS(2,JCELL)
C        ENDIF

      ENDDO

C     =========================================================================


      RETURN
      END
