      FUNCTION RANGAB(ISEED)
 
C     *************************************************************************
C
C     RANGAB
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT
C
C     CHANGE RECORD
C     -------------
C     21-JUL-2001:  CREATED
C     27-APR-2012:  RSC VARIANT FOR INLET BC TURBULENT VELOCITY FIELD
C
C     DESCRIPTION
C     -----------
C     UNIVARIATE GAUSSIAN RANDOM NUMBER GENERATOR
C     INITIALISED BY SUBROUTINE RANGIB
C
C     REFERENCES
C     ----------
C     1) NUMERICAL RECIPES pp196-197
C
C     *************************************************************************


C     FUNCTION
C     ========
      DOUBLE PRECISION RANGAB


C     ARGUMENT
C     ========
      INTEGER ISEED


C     PARAMETERS
C     ==========
      DOUBLE PRECISION ONE,TWO
      PARAMETER(ONE = 1.0D0, TWO = 2.0D0)


C     SAVED DATA
C     ==========
      DOUBLE PRECISION RSAVED
      LOGICAL RREADY
      COMMON/GRANNB/RSAVED,RREADY


C     EXTERNAL FUNCTION
C     =================
      DOUBLE PRECISION RANUNB
      EXTERNAL RANUNB


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION RTEMP1,RTEMP2,RTSQRD


C     BEGIN
C     =====
      IF(RREADY)THEN

C       USE SAVED RANDOM VALUE
        RANGAB = RSAVED
        RREADY = .FALSE.

      ELSE

C       GENERATE A NEW PAIR OF RANDOM VALUES
1000    CONTINUE

          RTEMP1 = RANUNB(ISEED)
          RTEMP2 = RANUNB(ISEED)

          RTEMP1 = TWO*RTEMP1 - ONE
          RTEMP2 = TWO*RTEMP2 - ONE

          RTSQRD = RTEMP1*RTEMP1 + RTEMP2*RTEMP2

          IF(RTSQRD.GE.ONE)GOTO 1000

C       END OF LOOP 1000

        RTSQRD = SQRT(-TWO*LOG(RTSQRD)/RTSQRD)
        RTEMP1 = RTEMP1*RTSQRD
        RSAVED = RTEMP2*RTSQRD

        RANGAB = RTEMP1
        RREADY = .TRUE.

      ENDIF


      RETURN
      END
