      SUBROUTINE TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)
 
C     *************************************************************************
C
C     TEMPER
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-MAR-2017:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES TEMPERATURE AND PRIMITIVE VARIABLES
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION TOLTMP
C      PARAMETER(TOLTMP = 1.0E-10)
      PARAMETER(TOLTMP = 1.0E-6)
      INTEGER NTITRS
      PARAMETER(NTITRS = 100)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION YEVAL(NSPCMX)
      DOUBLE PRECISION UEVAL(NCMPMX)
      DOUBLE PRECISION DEVAL,EEVAL,TEVAL


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION TCOEFF(0:NCTMAX),TDERIV(1:NCTMAX)
      DOUBLE PRECISION TEMPOR,TFPOLY,TDPOLY,DELTMP
      DOUBLE PRECISION UKUK,OVDENS
      INTEGER ICP,ISPEC,ITINT,ITITRS

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     =========================================================================

C     SINGLE POINT ONLY

C     =========================================================================

C     KINETIC ENERGY
      OVDENS = ONE/DEVAL
      UKUK = HALF*(UEVAL(1)*UEVAL(1)
     +     +       UEVAL(2)*UEVAL(2)
     +     +       UEVAL(3)*UEVAL(3))*OVDENS

C     =========================================================================

C     INITIALISE COEFFICIENTS OF TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
      TCOEFF(0) = UKUK - EEVAL
      DO ICP = 1, NCTMAX
        TCOEFF(ICP) = ZERO
        TDERIV(ICP) = ZERO
      ENDDO

C     =========================================================================

C     TEMPERATURE POLYNOMIAL
      DO ISPEC = 1,NSPEC

C       =======================================================================

C       LOCATE EXISTING TEMPERATURE IN AN INTERVAL
        ITINT = 1

C       =======================================================================

C       COEFFICIENTS OF TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
        TCOEFF(0) = TCOEFF(0)
     +    + YEVAL(ISPEC)*AMASCE(NCENTH(ITINT,ISPEC),ITINT,ISPEC)
        DO ICP = 1, NCPOLY(ITINT,ISPEC)
          TCOEFF(ICP) = TCOEFF(ICP)
     +                + YEVAL(ISPEC)*AMASCE(ICP,ITINT,ISPEC)
          TDERIV(ICP) = TDERIV(ICP)
     +                + YEVAL(ISPEC)*AMASCV(ICP,ITINT,ISPEC)
        ENDDO

C       =======================================================================

      ENDDO
C     END OF TEMPERATURE POLYNOMIAL

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        ICP = 0
        WRITE(6,*)ICP,TCOEFF(ICP)
        DO ICP = 1, NCPOLY(ITINT,1)
          WRITE(6,*)ICP,TCOEFF(ICP),TDERIV(ICP)
        ENDDO
      ENDIF

C     =========================================================================

C     INITIAL GUESS FOR TEMPERATURE
C      TEMPOR = -TCOEFF(0)/TCOEFF(1)
      TEMPOR = TRUN(ICELL)

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)TEMPOR


C     =========================================================================

C     SOLVE FOR TEMPERATURE USING NEWTON-RAPHSON
C     ---------------------
      ITITRS = 1
1000  CONTINUE

C       EVALUATE TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
        TFPOLY = TCOEFF(NCTMAX)
        TDPOLY = TDERIV(NCTMAX)
        DO ICP = NCTMM1,1,-1
          TFPOLY = TCOEFF(ICP) + TFPOLY*TEMPOR
          TDPOLY = TDERIV(ICP) + TDPOLY*TEMPOR
        ENDDO
        TFPOLY = TCOEFF(0) + TFPOLY*TEMPOR

C       EVALUATE TEMPERATURE CORRECTION
        DELTMP = -TFPOLY/TDPOLY

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)WRITE(6,*)ITITRS,DELTMP,TEMPOR+DELTMP

C       CHECK FOR CONVERGENCE
        IF(ABS(DELTMP).GT.TOLTMP)THEN
          IF(ITITRS.LT.NTITRS)THEN
            TEMPOR = TEMPOR + DELTMP
            ITITRS = ITITRS + 1
            GOTO 1000
          ELSE

C           ERROR CONDITION - T iteration failed to converge
          WRITE(6,*)'ERROR CONDITION - T iteration failed to converge'
            WRITE(6,'(4I5,1PE15.7)')ITIME,IRKSTP,ICELL,ITITRS,TEMPOR
            WRITE(6,'(I5,4(1PE15.7))')ICELL,
     +              DEVAL,UEVAL(1),EEVAL,TEVAL
            DO ISPEC = 1, NSPEC
              WRITE(6,'(5X,1PE15.7)')YEVAL(ISPEC)
            ENDDO

            CALL ERHAND(9,19,4)

          ENDIF
        ENDIF

C     END OF LOOP 1000

C     =========================================================================

C     SET THE NEW TEMPERATURE
      TEVAL = TEMPOR

C     =========================================================================

C     EVALUATE THE PRIMITIVE VARIABLES
      UEVAL(1) = UEVAL(1)*OVDENS
      UEVAL(2) = UEVAL(2)*OVDENS
      UEVAL(3) = UEVAL(3)*OVDENS
      EEVAL = EEVAL*OVDENS
      DO ISPEC = 1,NSPEC
        YEVAL(ISPEC) = YEVAL(ISPEC)*OVDENS
      ENDDO

C     =========================================================================

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WRITE(6,*)'TEMPER',ITIME,IRKSTP
        WRITE(6,'(3(1PE12.4))')UEVAL(1),UEVAL(2),UEVAL(3)
        WRITE(6,'(3(1PE12.4))')DEVAL,EEVAL,TEVAL
        DO ISPEC = 1, NSPEC
          WRITE(6,'(I5,1PE12.4)')ISPEC,YEVAL(ISPEC)
        ENDDO
      ENDIF


      RETURN
      END
