      SUBROUTINE TREBAL
 
C     *************************************************************************
C
C     TREBAL
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     27-JUL-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     BALANCES THE OCTREE
C     BY ENSURING THAT CELL-TO-CELL TRANSITIONS ARE h-2h AT MOST
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      INTEGER NLOOPS
      PARAMETER(NLOOPS = 10)


C     LOCAL DATA
C     ==========
      INTEGER NTRMOT(NINDMX,NREMAX)
      INTEGER NTREFN(NINDMX,NRDMAX)
      INTEGER NTRCEL(NRDMAX)
      INTEGER NTRLVL(NREMAX)
      INTEGER MCRUN(NINDMX),MCNXT(NINDMX),MCPER(NINDMX)
      INTEGER MPLOC(NCMPMX),MPNXT(NCMPMX)
      INTEGER INCODE,KCELL,LCELL,JFACE,JDIRN
      INTEGER JLEVEL,KLEVEL,LLEVEL,LEVDIF,IGLOBC
      INTEGER IRCELL,NRMCEL
      INTEGER ICREFN,NCREFN,NGREFN
      INTEGER ILOOPS
      CHARACTER*1 CFOUND,CFPERI
      LOGICAL FLNIND

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In TREBAL',ITIME
C      WRITE(6,*)'In TREBAL',ITIME

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        DO INCODE = 1,NNCODE
C          CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)
C          CALL MCGLVL(MCRUN,JLEVEL)
C          WRITE(6,'(2I5)')INCODE,JLEVEL
C        ENDDO
C      ENDIF

C     =========================================================================

C     LOOP UNTIL ALL TRANSITIONS GREATER THAN h-2h HAVE BEEN REMOVED
C     ----
      ILOOPS = 1

1000  CONTINUE

C       =======================================================================

C       REBUILD THE GLOBAL OCTREE
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NNCODE)
        CALL PTABLE(MCRUN)
        CALL RETREE
        CALL RELEAF

C       INITIALISE REFINEMENT COUNTER
        ICREFN = 0

C       INITIALISE REMOTE CELL COUNTER
        IRCELL = 0

C       =======================================================================

C       FIND THE NEIGHBOURS
C       -------------------

C       RUN THROUGH ALL M-CODES
        DO INCODE = 1,NNCODE

C         --------------------------------------------------------------------- 

C         FETCH THE M-CODE
          CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)

C         BACK OFF THE LEVEL BY ONE
          CALL MCLOWR(MCRUN)

C         EXTRACT THE LEVEL
          CALL MCGLVL(MCRUN,JLEVEL)

C         GET THE INTEGER COORDINATES
          CALL MCCXYZ(MCRUN,JLEVEL,MPLOC(1),MPLOC(2),MPLOC(3))

C         --------------------------------------------------------------------- 

C         RUN THROUGH ALL NEIGHBOURS
          DO JFACE = 1,NFACE

C           INCREMENT THE CELL COORDINATES AND ASSEMBLE THE NEIGHBOUR M-CODE
            MPNXT(1) = MPLOC(1) + MATINC(1,JFACE)
            MPNXT(2) = MPLOC(2) + MATINC(2,JFACE)
            MPNXT(3) = MPLOC(3) + MATINC(3,JFACE)

C           NEIGHBOUR CELL COORDINATE IN-OUT TEST
            JDIRN = MDIRN(JFACE)
            FLNIND = (MPNXT(JDIRN).GE.MPMIN(JDIRN,JLEVEL))
            FLNIND = FLNIND.AND.(MPNXT(JDIRN).LE.MPMAX(JDIRN,JLEVEL))

            IF(FLNIND)THEN

C             ================================================================= 

C             NEIGHBOUR IS INSIDE THE DOMAIN
C             ------------------------------
C             ASSEMBLE THE NEIGHBOUR M-CODE
              CALL MCXYZC(MCNXT,JLEVEL,MPNXT(1),MPNXT(2),MPNXT(3))

C             SEARCH FOR THE NEIGHBOUR
              CALL OCFIND(NOCHLD,COCODE,NOCTMX,
     +                    MCNXT,KLEVEL,KCELL,CFOUND)

C             CHECK FINDINGS
              IF(CFOUND.EQ.CHTWIG)THEN

C               --------------------------------------------------------------- 

C               NEIGHBOUR IS LOCAL
C               ------------------

C               CHECK DIFFERENCE OF LEVEL
                LEVDIF = JLEVEL - KLEVEL
                IF(LEVDIF.GT.0)THEN

C                 h-2h(+) TRANSITION
C                 NEIGHBOUR IS MORE THAN ONE LEVEL LESS REFINED
C                 MARK NEIGHBOUR FOR REFINEMENT
                  ICREFN = ICREFN + 1
                  CALL MCTLVL(MCNXT,KLEVEL)
                  CALL MCSTOR(MCNXT,NTREFN,NRDMAX,ICREFN)
                  NTRCEL(ICREFN) = KCELL

                ENDIF

C               --------------------------------------------------------------- 

              ELSEIF(CFOUND.EQ.CHNRML)THEN

C               --------------------------------------------------------------- 

C               NEIGHBOUR IS NOT FOUND
C               ----------------------
C               AT LEVEL OF LOCAL CELL OR LOWER

C               2h-h(-) TRANSITION
C               ASSUME NEIGHBOUR IS MORE REFINED - TAKE NO ACTION
                CONTINUE
 
C               --------------------------------------------------------------- 
  
              ELSEIF(CFOUND.EQ.CHPARL)THEN

C               --------------------------------------------------------------- 

C               NEIGHBOUR IS REMOTE
C               -------------------

C               INCREMENT THE REMOTE CELL COUNTER
                IRCELL = IRCELL + 1

C               STORE THE REMOTE M-CODE
C                CALL MCTLVL(MCNXT,KLEVEL)
                CALL MCSTOR(MCNXT,NTRMOT,NREMAX,IRCELL)

C               STORE THE LOCAL CELL LEVEL
                NTRLVL(IRCELL) = JLEVEL

C               --------------------------------------------------------------- 

              ELSE

C               --------------------------------------------------------------- 

C               ERROR CONDITION - interior cell not found
C               ---------------
C                IF(IPROC.EQ.LPROC)THEN
C                  WRITE(6,*)'interior cell not found',INCODE,JFACE
C                ENDIF
                CALL ERHAND(5,3,4)

C               --------------------------------------------------------------- 

              ENDIF
C             CHECK FINDINGS

C             ================================================================= 

            ELSE

C             ================================================================= 

C             NEIGHBOUR IS OUTSIDE THE DOMAIN
C             -------------------------------
C             LOCAL CELL IS ON THE OUTER BOUNDARY

C             GET THE OUTER BC ATTRIBUTE FOR THIS FACE
              IGLOBC = NGLOBC(JFACE)

C             CHECK THE OUTER BC ATTRIBUTE
              IF(IGLOBC.EQ.NSPERI)THEN

C               ---------------------------------------------------------------

C               SPECIAL CASE OF PERIODIC BC
C               ---------------------------
C               ASSEMBLE PERIODIC CELL M-CODE
C               USING MIN/MAX COORDINATES FOR THIS COMPONENT AT THIS LEVEL
                MPNXT(JDIRN) = MPMIN(JDIRN,JLEVEL)
                IF(FLHAND(JFACE))MPNXT(JDIRN) = MPMAX(JDIRN,JLEVEL)
                CALL MCXYZC(MCPER,JLEVEL,MPNXT(1),MPNXT(2),MPNXT(3))

C               SEARCH FOR THE PERIODIC CELL
                CALL OCFIND(NOCHLD,COCODE,NOCTMX,
     +                      MCPER,LLEVEL,LCELL,CFPERI)

C               CHECK FINDINGS ON PERIODIC CELL
                IF(CFPERI.EQ.CHTWIG)THEN

C                 -------------------------------------------------------------

C                 PERIODIC CELL IS LOCAL
C                 ----------------------
C                 CHECK DIFFERENCE OF LEVEL
                  LEVDIF = JLEVEL - LLEVEL
                  IF(LEVDIF.GT.0)THEN

C                   h-2h(+) TRANSITION
C                   PERIODIC CELL IS MORE THAN ONE LEVEL LESS REFINED
C                   MARK PERIODIC CELL FOR REFINEMENT
                    ICREFN = ICREFN + 1
                    CALL MCTLVL(MCPER,LLEVEL)
                    CALL MCSTOR(MCPER,NTREFN,NRDMAX,ICREFN)
                    NTRCEL(ICREFN) = LCELL

                  ENDIF

C                 -------------------------------------------------------------

                ELSEIF(CFPERI.EQ.CHPARL)THEN

C                 -------------------------------------------------------------

C                 PERIODIC CELL IS REMOTE
C                 -----------------------
C                 INCREMENT THE REMOTE CELL COUNTER
                  IRCELL = IRCELL + 1

C                 STORE THE REMOTE M-CODE
                  CALL MCSTOR(MCPER,NTRMOT,NREMAX,IRCELL)

C                 STORE THE LOCAL CELL LEVEL
                  NTRLVL(IRCELL) = JLEVEL

C                 -------------------------------------------------------------

                ELSEIF(CFPERI.EQ.CHNRML)THEN

C                 -------------------------------------------------------------

C                 PERIODIC CELL NOT FOUND
C                 -----------------------
C                 AT LEVEL OF LOCAL CELL OR LOWER

C                 2h-h(-) TRANSITION
C                 ASSUME NEIGHBOUR IS MORE REFINED - TAKE NO ACTION
                  CONTINUE

C                 -------------------------------------------------------------

                ELSE

C                 -------------------------------------------------------------

C                 ERROR CONDITION - periodic cell not found
C                 -----------------------
                  CALL ERHAND(5,6,4)

C                 -------------------------------------------------------------

                ENDIF
C               END OF CHECK FINDINGS ON PERIODIC CELL

C               --------------------------------------------------------------- 

              ELSE

C               --------------------------------------------------------------- 

C               NON-PERIODIC BC
C               ---------------
C               NO ACTION REQUIRED
                CONTINUE

C               --------------------------------------------------------------- 

              ENDIF
C             END OF CHECK OUTER BC ATTRIBUTE

C             ================================================================= 

            ENDIF
C           END OF NEIGHBOUR CELL COORDINATE IN-OUT TEST

C           =================================================================== 

          ENDDO
C         END OF RUN THROUGH ALL NEIGHBOURS

C         ===================================================================== 

        ENDDO
C       END OF RUN THROUGH ALL CELLS

C       =======================================================================

C       NUMBER OF LOCAL CELLS TO REFINE
        NCREFN = ICREFN

C       ESTIMATED GLOBAL MAX NUMBER OF CELLS TO REFINE
        NGREFN = NCREFN

C       NUMBER OF REMOTE CELLS TO REFINE
        NRMCEL = IRCELL
       
C       =======================================================================

C       CHECK THE REMOTE CELLS
C       ----------------------

C       SORT THE REMOTE CELL LIST
        CALL SORTEM(NTRMOT,NTRLVL,NREMAX,NRMCEL)

C       REMOVE DUPLICATES FROM THE REMOTE CELL LIST
        CALL REMDUP(NTRMOT,NTRLVL,NREMAX,NRMCEL)

C       EXCHANGE REMOTE CELL DATA
        CALL PARTRE(NTRMOT,NTRLVL,NRMCEL,NTREFN,NTRCEL,NCREFN,NGREFN)

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)THEN
C          WRITE(6,'(3I8)')NCREFN,NGREFN,NRMCEL
C        ENDIF

C       ========================================================================

C       CHECK FOR REMAINING REFINEMENT ACTIONS
C       --------------------------------------
        IF(NGREFN.GT.0)THEN

C         SORT THE REFINEMENT LIST
          CALL SORTEM(NTREFN,NTRCEL,NRDMAX,NCREFN)

C         REMOVE DUPLICATES FROM THE REFINEMENT LIST
          CALL REMDUP(NTREFN,NTRCEL,NRDMAX,NCREFN)

CC         DIAGNOSTICS
C          IF(IPROC.EQ.LPROC)THEN
C            WRITE(6,'(2I5)')ILOOPS,NCREFN
C            DO ICREFN = 1, NCREFN
C              KCELL = NTRCEL(ICREFN)
C              WRITE(6,'(2X,3I5)')ICREFN,KCELL,NRSTAT(KCELL)
C            ENDDO
C          ENDIF

C         CARRY OUT REFINEMENT ACTIONS
          CALL REFACT(NTREFN,NTRCEL,NCREFN)

C         GARBAGE COLLECTION FOLLOWING REFINEMENT
          CALL REFGRC

C         CHECK LOOP COUNTER
          IF(ILOOPS.LT.NLOOPS)THEN

C           GO ROUND AGAIN
            ILOOPS = ILOOPS + 1

            GOTO 1000

          ELSE

C           TOO MANY LOOPS - ERROR CONDITION
            CONTINUE

          ENDIF

        ENDIF

C     END OF LOOP 1000

C     =========================================================================

C     GARBAGE COLLECTION FOLLOWING TREE BALANCING
C     ------------------
C      CALL TREGRC
C     REBUILD THE GLOBAL OCTREE
      CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NNCODE)
      CALL PTABLE(MCRUN)
      CALL RETREE
      CALL RELEAF

C     =========================================================================


      RETURN
      END
